package antlr4_parser;

import org.antlr.v4.runtime.ParserRuleContext;
import org.antlr.v4.runtime.tree.TerminalNode;

public class NiceFormattingVisitor extends JavaMethodsBaseVisitor<String> {
	private String htmlFormatting;
	private String htmlFormattings;
	
	public NiceFormattingVisitor() {
		htmlFormatting = "<html> ";
		htmlFormattings = "";
	}
	
	@Override 
	public String visitMethodsNode(JavaMethodsParser.MethodsNodeContext ctx) { 
		for(int i = 0; i < ctx.getChildCount(); i++) {
		  visit(ctx.method(i));
		  htmlFormattings += htmlFormatting + " </html> ;";
		  htmlFormatting = "<html> ";
		}
		return htmlFormattings; 
  }
	
	@Override 
	public String visitMethodNode(JavaMethodsParser.MethodNodeContext ctx) { 
//		if(ctx.getChild(0).getText().equals("@Override"))
//			htmlFormatting += "<font color=gray> "+ "@Override" + "</font> ";
		return visitChildren(ctx);
  }
	
	@Override 
	public String visitMethodNameNode(JavaMethodsParser.MethodNameNodeContext ctx) { 
		htmlFormatting += ctx.getText();
		return visitChildren(ctx); 
	}
	
	@Override 
	public String visitParameterNode(JavaMethodsParser.ParameterNodeContext ctx) { 
		String parameterType = visit(ctx.type());
		String parameterName = visit(ctx.parameter_name());
		htmlFormatting += "<font color=blue>"+ parameterType + "</font> " + parameterName;
		return ""; 
	}

	@Override 
	public String visitParametersNode(JavaMethodsParser.ParametersNodeContext ctx) {
		htmlFormatting += " ( ";
		for(int i = 0; i < ctx.getChildCount(); i++) {
			if(ctx.getChild(i) instanceof ParserRuleContext) 
			  visit(ctx.getChild(i));
			else if(ctx.getChild(i) instanceof TerminalNode)  
				htmlFormatting += ", ";
		}
		htmlFormatting += " ) ";
		return ""; 
	}

	@Override 
	public String visitEmptyParametersNode(JavaMethodsParser.EmptyParametersNodeContext ctx) { 
		htmlFormatting +=  " ( ) ";
		return "";
	}

	@Override 
	public String visitReturnTypeNode(JavaMethodsParser.ReturnTypeNodeContext ctx) { 
		String returnType = visit(ctx.type());
		htmlFormatting += "<font color=blue>"+ returnType + "</font> ";
		return returnType;
	}

	@Override public String visitParameterNameNode(JavaMethodsParser.ParameterNameNodeContext ctx) { 
		return ctx.getText(); 
	}
	
	@Override 
	public String visitObjectTypeNode(JavaMethodsParser.ObjectTypeNodeContext ctx) {
		return ctx.getText();
	}
	
	@Override 
	public String visitArrayTypeNode(JavaMethodsParser.ArrayTypeNodeContext ctx) { 
		return ctx.getText();
	}
	
	@Override 
	public String visitExceptionNode(JavaMethodsParser.ExceptionNodeContext ctx) { 
		String exceptionType = visit(ctx.object_type());
		htmlFormatting += "<font color=blue>"+ exceptionType + "</font> ";
		return ""; 
	}

	
	@Override 
	public String visitEmptyExceptionsNode(JavaMethodsParser.EmptyExceptionsNodeContext ctx) { 
		return ""; 
	}

	@Override 
	public String visitExceptionsNode(JavaMethodsParser.ExceptionsNodeContext ctx) { 
		htmlFormatting += " <font color=orange> throws </font>";
		for(int i = 1; i < ctx.getChildCount(); i++) {
			if(ctx.getChild(i) instanceof ParserRuleContext) 
			  visit(ctx.getChild(i));
			else if(ctx.getChild(i) instanceof TerminalNode) {
				htmlFormatting = htmlFormatting.substring(0, htmlFormatting.length() - 1);
				htmlFormatting += ", ";
			}
		}
		return "";
	}
	
	@Override 
	public String visitObjectTypeValueNode(JavaMethodsParser.ObjectTypeValueNodeContext ctx) { 
		return ctx.getText(); 
	}
	
	@Override 
	public String visitArrayTypeValueNode(JavaMethodsParser.ArrayTypeValueNodeContext ctx) { 
		return ctx.getText(); 
	}

}
