package gui;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JScrollPane;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.resource.Icons;
import edu.stanford.smi.protege.ui.ClsTreeFinder;
import edu.stanford.smi.protege.ui.FrameRenderer;
import edu.stanford.smi.protege.ui.ParentChildRoot;
import edu.stanford.smi.protege.util.CollectionUtilities;
import edu.stanford.smi.protege.util.ComponentFactory;
import edu.stanford.smi.protege.util.ComponentUtilities;
import edu.stanford.smi.protege.util.LabeledComponent;
import edu.stanford.smi.protege.util.LazyTreeNode;
import edu.stanford.smi.protege.util.SelectableContainer;
import edu.stanford.smi.protege.util.SelectableTree;

//Class taken from OntovizTab (it is easier to set the tree root node with this)
public class CustomInstanceClsesPanel extends SelectableContainer {

	Project itsProject;
	SelectableTree itsClsTree;

	public CustomInstanceClsesPanel(Project project, Cls root) {
		itsProject = project;
		add(createClsesPanel(root));
		setSelectable(itsClsTree);
	}

	JComponent createClsesPanel(Cls root) {
//		Cls root = itsProject.getKnowledgeBase().getRootCls();
//		Cls root = itsProject.getKnowledgeBase().getCls("http://jade.cselt.it/beangenerator#OntologyException");
		itsClsTree = ComponentFactory.createSelectableTree(null, new ParentChildRoot(root));
		FrameRenderer renderer = new FrameRenderer();
		renderer.setDisplayDirectInstanceCount(true);
		itsClsTree.setCellRenderer(renderer);
		itsClsTree.setSelectionRow(0);
		LabeledComponent c = new LabeledComponent("Classes", new JScrollPane(itsClsTree));
		c.addHeaderButton(getViewClsAction());
		c.setFooterComponent(new ClsTreeFinder(itsProject.getKnowledgeBase(), itsClsTree, "Find Class"));
		return c;
	}

	Action getViewClsAction() {
		return new AbstractAction("View Selected Class", Icons.getViewIcon()) {
			public void actionPerformed(ActionEvent event) {
				Iterator i = ComponentUtilities.getSelection(itsClsTree).iterator();
				while (i.hasNext()) {
					Cls cls = (Cls) i.next();
					itsProject.show(cls);
				}
			}
		};
	}

	void makeClassVisible(String id) {
		Instance instance = itsProject.getKnowledgeBase().getInstance(id);
		Cls cls = null;
		if (instance instanceof Cls)
			cls = (Cls)instance;
		else if (instance != null) {
			cls = instance.getDirectType();
		}
		if (cls != null)
			makeClassVisible(cls);
	}

	void makeClassVisible(Cls cls) {
		ArrayList clses = new ArrayList();
		getPathToRoot(cls, clses);
		Collections.reverse(clses);
		ComponentUtilities.setSelectedObjectPath(itsClsTree, clses);
	}

	void getPathToRoot(Cls cls, Collection clses) {
		Collection rootClses = (Collection)
				((LazyTreeNode)itsClsTree.getModel().getRoot()).getUserObject();
		clses.add(cls);
		Collection superclasses = cls.getDirectSuperclasses();
		Cls parent = (Cls) CollectionUtilities.getFirstItem(superclasses);
		if (parent == null) {
		} else if (rootClses.contains(parent)) {
			clses.add(parent);
		} else {
			getPathToRoot(parent, clses);
		}
	}

} 
