package gui;

import javax.swing.JPanel;
import javax.swing.SpringLayout;
import javax.swing.JLabel;

import java.awt.Font;
import java.awt.Color;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import java.awt.SystemColor;

import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import javax.swing.JTextArea;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.table.TableModel;

import org.antlr.v4.runtime.misc.ParseCancellationException;

import antlr4_parser.Method;
import antlr4_parser.MethodData;
import antlr4_parser.MethodParser;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protegex.owl.model.impl.AbstractRDFSClass;
import edu.stanford.smi.protegex.owl.model.impl.DefaultRDFSNamedClass;

public class EditMethodPanel extends JPanel {
	private EditMethodPanel editMethodPanel = this;
	private EditMethodDialog editMethodDialog;
	private JavaMethodMapperPanel parentPanel;
	
	private JTextField returnTypeTextField;
	private JTextField methodNameTextField;
	private JTextArea textArea;
	
	public JButton applyButton;
	public JComboBox javaComboBox;
	public JComboBox parametersComboBox;
	public JComboBox exceptionsComboBox;
	public String selectedOntologyClass = "";
	
	private Project project;
	private String oldValidMethodName = "";
	
	private boolean editMode;
	private int rowToEdit;
	private String editModeStartingSignature;
	
	private String[] javaKeywords;
	
	//private String ONTOLOGY_EXCEPTION_PREFIX = "OntoScene_";
	//private String ONTOLOGY_EXCEPTION = "Exception";
	//private String ONTOLOGY_EXCEPTION_PREFIX = "Jade_";
	private String ONTOLOGY_EXCEPTION_PREFIX = "Exception_";
	private String ONTOLOGY_EXCEPTION = "Base";
	
  //Error messages
	private final String INVALID_NAME_ERROR =	"Invalid name.";
	private final String DUPLICATE_EXCEPTION_ERROR =	"Duplicate exception.";
	private final String SYNTAX_ERROR =	"Syntax error.";
	private final String DUPLICATE_SIGNATURE_SUPERCLASS_ERROR =	"Duplicate method signature in super-class:\n";
	private final String DUPLICATE_SIGNATURE_SUBCLASS_ERROR =	"Duplicate method signature in sub-class:\n";
	private final String DUPLICATE_SIGNATURE_THIS_CLASS_ERROR =	"Duplicate method signature in this class.";
	private final String OVERLOAD_ERROR =	"Can not overload method with the same signature of another one.\n"
                                                 + "(Java signature = method name + parameter types).";
	private final String RETURN_TYPE_ERROR =	"Return type is neither a Java type nor an Ontology type.";
	private final String PARAMETER_TYPE_ERROR =	"Parameter type is neither a Java type nor an Ontology type.";
	private final String EXCEPTION_TYPE_ERROR =	"Exception type is not a sub-class of" +  ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION + "class.";
	private final String INVALID_METHOD_ERROR =	"Invalid method.";
	private final String INVALID_METHOD_NAME_ERROR =	"Invalid method name.";
	private final String INVALID_PARAMETER_NAME_ERROR =	"Invalid parameter name.";
	
	//Different modes to call the method frame from this window
	private final boolean ADD_MODE =	false; // to add n methods to a class
	private final boolean EDIT_MODE =	true; // to edit the selected method

	public EditMethodPanel(JavaMethodMapperPanel parent, final Project project, final EditMethodDialog editMethodDialog, final boolean editMode) {
		this.project = project;
		this.editMethodDialog = editMethodDialog;
		this.parentPanel = parent;
		this.editMode = editMode;
		
		javaKeywords = new String[] { "true", "false", "null", "abstract", "continue", "for", "new", "switch",
		                              "assert", "default", "goto", "package", "synchronized",
		                              "boolean", "do", "if", "private", "this",
		                              "break", "double", "implements", "protected", "throw",
		                              "byte", "else", "import", "public", "throws",
		                              "case", "enum", "instanceof", "return", "transient",
		                              "catch", "extends", "int", "short", "try",
		                              "char", "final", "interface", "static", "void",
		                              "class", "finally", "long", "strictfp", "volatile",
		                              "const", "float", "native", "super", "while"
		                            };
		
		//setBackground(new Color(255, 228, 196));
		setBackground(new Color(236, 233, 216));
		SpringLayout springLayout = new SpringLayout();
		setLayout(springLayout);
		
		JLabel returnTypeLabel = new JLabel("Return Type:");
		returnTypeLabel.setForeground(new Color(128, 128, 128));
		returnTypeLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		add(returnTypeLabel);
		
		JSeparator separator = new JSeparator();
		springLayout.putConstraint(SpringLayout.SOUTH, returnTypeLabel, -10, SpringLayout.NORTH, separator);
		springLayout.putConstraint(SpringLayout.NORTH, separator, 41, SpringLayout.NORTH, this);
		springLayout.putConstraint(SpringLayout.WEST, separator, 0, SpringLayout.WEST, this);
		springLayout.putConstraint(SpringLayout.EAST, separator, 0, SpringLayout.EAST, this);
		separator.setBackground(new Color(0, 0, 0));
		add(separator);
		
		returnTypeTextField = new JTextField();
		returnTypeTextField.setBackground(Color.WHITE);
		springLayout.putConstraint(SpringLayout.EAST, returnTypeLabel, -6, SpringLayout.WEST, returnTypeTextField);
		springLayout.putConstraint(SpringLayout.EAST, returnTypeTextField, -187, SpringLayout.EAST, this);
		springLayout.putConstraint(SpringLayout.WEST, returnTypeTextField, 82, SpringLayout.WEST, this);
		springLayout.putConstraint(SpringLayout.SOUTH, returnTypeTextField, -8, SpringLayout.NORTH, separator);
		returnTypeTextField.setFont(new Font("Dialog", Font.BOLD, 12));
		returnTypeTextField.setColumns(10);
		returnTypeTextField.setEnabled(true);
		returnTypeTextField.setEditable(false);
		returnTypeTextField.setCaretColor(Color.WHITE);
		returnTypeTextField.setHighlighter(null);
		add(returnTypeTextField);
		
		JLabel methodNameLabel = new JLabel("Method Name:");
		springLayout.putConstraint(SpringLayout.NORTH, methodNameLabel, 10, SpringLayout.SOUTH, separator);
		springLayout.putConstraint(SpringLayout.WEST, methodNameLabel, 5, SpringLayout.WEST, this);
		methodNameLabel.setForeground(Color.GRAY);
		methodNameLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		add(methodNameLabel);
		
		methodNameTextField = new JTextField();
		springLayout.putConstraint(SpringLayout.NORTH, methodNameTextField, -3, SpringLayout.NORTH, methodNameLabel);
		springLayout.putConstraint(SpringLayout.WEST, methodNameTextField, 90, SpringLayout.WEST, methodNameLabel);
		springLayout.putConstraint(SpringLayout.EAST, methodNameTextField, 0, SpringLayout.EAST, separator);
		methodNameTextField.setFont(new Font("Dialog", Font.BOLD, 12));
		methodNameTextField.setColumns(10);
		methodNameTextField.getDocument().addDocumentListener(new DocumentListener() {
		  public void changedUpdate(DocumentEvent e) {
		  	updateMethodTextArea();
		  }
		  public void removeUpdate(DocumentEvent e) {
		  	updateMethodTextArea();
		  }
		  public void insertUpdate(DocumentEvent e) {
		  	updateMethodTextArea();
		  }
		});
		add(methodNameTextField);
		
		JSeparator separator_1 = new JSeparator();
		separator_1.setBackground(new Color(0, 0, 0));
		springLayout.putConstraint(SpringLayout.NORTH, separator_1, 15, SpringLayout.SOUTH, methodNameLabel);
		springLayout.putConstraint(SpringLayout.WEST, separator_1, 0, SpringLayout.WEST, this);
		springLayout.putConstraint(SpringLayout.EAST, separator_1, 0, SpringLayout.EAST, this);
		add(separator_1);
		
		JLabel parametersLabel = new JLabel("Parameters:");
		springLayout.putConstraint(SpringLayout.NORTH, parametersLabel, 6, SpringLayout.SOUTH, separator_1);
		springLayout.putConstraint(SpringLayout.WEST, parametersLabel, 5, SpringLayout.WEST, this);
		parametersLabel.setForeground(Color.GRAY);
		parametersLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		add(parametersLabel);
		
		JButton removeParameterButton = new JButton("-");
		removeParameterButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				parametersComboBox.removeItem(parametersComboBox.getSelectedItem());
				updateMethodTextArea();
			}
		});
		springLayout.putConstraint(SpringLayout.NORTH, removeParameterButton, -4, SpringLayout.NORTH, parametersLabel);
		removeParameterButton.setBackground(Color.WHITE);
		removeParameterButton.setForeground(new Color(0, 0, 0));
		removeParameterButton.setFont(new Font("Dialog", Font.BOLD, 12));
		add(removeParameterButton);
		
		JSeparator separator_2 = new JSeparator();
		springLayout.putConstraint(SpringLayout.WEST, separator_2, 0, SpringLayout.WEST, this);
		springLayout.putConstraint(SpringLayout.EAST, separator_2, 0, SpringLayout.EAST, separator);
		separator_2.setBackground(Color.BLACK);
		add(separator_2);
		
		JLabel exceptionsLabel = new JLabel("Exceptions:");
		springLayout.putConstraint(SpringLayout.NORTH, exceptionsLabel, 6, SpringLayout.SOUTH, separator_2);
		springLayout.putConstraint(SpringLayout.WEST, exceptionsLabel, 5, SpringLayout.WEST, this);
		exceptionsLabel.setForeground(Color.GRAY);
		exceptionsLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		add(exceptionsLabel);
		
		parametersComboBox = new JComboBox();
		parametersComboBox.setBackground(Color.WHITE);
		springLayout.putConstraint(SpringLayout.WEST, parametersComboBox, 6, SpringLayout.EAST, parametersLabel);
		springLayout.putConstraint(SpringLayout.EAST, parametersComboBox, -55, SpringLayout.WEST, removeParameterButton);
		parametersComboBox.setFont(new Font("Dialog", Font.BOLD, 12));
		springLayout.putConstraint(SpringLayout.NORTH, parametersComboBox, 2, SpringLayout.SOUTH, separator_1);
		add(parametersComboBox);
		
		exceptionsComboBox = new JComboBox();
		exceptionsComboBox.setBackground(Color.WHITE);
		springLayout.putConstraint(SpringLayout.WEST, exceptionsComboBox, 11, SpringLayout.EAST, exceptionsLabel);
		springLayout.putConstraint(SpringLayout.EAST, exceptionsComboBox, 0, SpringLayout.EAST, parametersComboBox);
		exceptionsComboBox.setFont(new Font("Dialog", Font.BOLD, 12));
		springLayout.putConstraint(SpringLayout.NORTH, exceptionsComboBox, 6, SpringLayout.SOUTH, separator_2);
		add(exceptionsComboBox);
		
		JButton editParameterButton = new JButton("Edit");
		editParameterButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(parametersComboBox.getSelectedItem() != null) {
					EditParameterDialog dialog = new EditParameterDialog(editMethodPanel, (String) parametersComboBox.getSelectedItem(), project);
					dialog.setVisible(true);
					updateMethodTextArea();
				}
			}
		});
		springLayout.putConstraint(SpringLayout.EAST, removeParameterButton, -6, SpringLayout.WEST, editParameterButton);
		springLayout.putConstraint(SpringLayout.NORTH, separator_2, 6, SpringLayout.SOUTH, editParameterButton);
		springLayout.putConstraint(SpringLayout.NORTH, editParameterButton, 2, SpringLayout.SOUTH, separator_1);
		springLayout.putConstraint(SpringLayout.EAST, editParameterButton, 0, SpringLayout.EAST, separator);
		editParameterButton.setForeground(Color.BLACK);
		editParameterButton.setFont(new Font("Dialog", Font.BOLD, 12));
		editParameterButton.setBackground(Color.WHITE);
		add(editParameterButton);
		
		JButton addParameterButton = new JButton("+");
		addParameterButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				EditParameterDialog dialog = new EditParameterDialog(editMethodPanel, project);
				dialog.setVisible(true);
				updateMethodTextArea();
			}
		});
		springLayout.putConstraint(SpringLayout.SOUTH, addParameterButton, 0, SpringLayout.SOUTH, removeParameterButton);
		springLayout.putConstraint(SpringLayout.EAST, addParameterButton, -6, SpringLayout.WEST, removeParameterButton);
		addParameterButton.setForeground(Color.BLACK);
		addParameterButton.setFont(new Font("Dialog", Font.BOLD, 12));
		addParameterButton.setBackground(Color.WHITE);
		add(addParameterButton);
		
		JButton editExceptionButton = new JButton("Edit");
		editExceptionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(exceptionsComboBox.getItemCount() > 0) {
					OntologyClassSelectorDialog dialog = new OntologyClassSelectorDialog(editMethodPanel, project, ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION, true);
					dialog.setVisible(true);
					if(!editMethodPanel.selectedOntologyClass.equals("")) {
						//					exceptionsComboBox.setSelectedItem(editMethodPanel.selectedOntologyClass);
						int selectedIndex = editMethodPanel.exceptionsComboBox.getSelectedIndex();
						for (int i = 0; i < editMethodPanel.exceptionsComboBox.getItemCount(); i++) {
							if(i != selectedIndex && editMethodPanel.selectedOntologyClass.equals(editMethodPanel.exceptionsComboBox.getItemAt(i))) {
								JOptionPane.showMessageDialog(new JFrame(), DUPLICATE_EXCEPTION_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
								return;
							}
						}
						editItem(editMethodPanel.selectedOntologyClass, selectedIndex);
						editMethodPanel.exceptionsComboBox.setSelectedIndex(selectedIndex);
						updateMethodTextArea();
					}
					selectedOntologyClass = "";
				}
			}
		});
		springLayout.putConstraint(SpringLayout.NORTH, editExceptionButton, 6, SpringLayout.SOUTH, separator_2);
		springLayout.putConstraint(SpringLayout.EAST, editExceptionButton, 0, SpringLayout.EAST, separator);
		editExceptionButton.setForeground(Color.BLACK);
		editExceptionButton.setFont(new Font("Dialog", Font.BOLD, 12));
		editExceptionButton.setBackground(Color.WHITE);
		add(editExceptionButton);
		
		JButton removeExceptionButton = new JButton("-");
		removeExceptionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exceptionsComboBox.removeItem(exceptionsComboBox.getSelectedItem());
				updateMethodTextArea();
			}
		});
		springLayout.putConstraint(SpringLayout.NORTH, removeExceptionButton, 6, SpringLayout.SOUTH, separator_2);
		springLayout.putConstraint(SpringLayout.WEST, removeExceptionButton, 0, SpringLayout.WEST, removeParameterButton);
		removeExceptionButton.setForeground(Color.BLACK);
		removeExceptionButton.setFont(new Font("Dialog", Font.BOLD, 12));
		removeExceptionButton.setBackground(Color.WHITE);
		add(removeExceptionButton);
		
		JButton addExceptionButton = new JButton("+");
		addExceptionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
//				EditExceptionDialog dialog = new EditExceptionDialog(editMethodPanel, project);
//				dialog.setVisible(true);
				OntologyClassSelectorDialog dialog = new OntologyClassSelectorDialog(editMethodPanel, project, ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION, true);
				dialog.setVisible(true);
				if(!editMethodPanel.selectedOntologyClass.equals("")) {
					for (int i = 0; i < editMethodPanel.exceptionsComboBox.getItemCount(); i++) {
						if(editMethodPanel.selectedOntologyClass.equals(editMethodPanel.exceptionsComboBox.getItemAt(i))) {
							JOptionPane.showMessageDialog(new JFrame(), DUPLICATE_EXCEPTION_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
							return;
						}
					}
					exceptionsComboBox.addItem(editMethodPanel.selectedOntologyClass);
					exceptionsComboBox.setSelectedIndex(exceptionsComboBox.getItemCount() - 1);
					updateMethodTextArea();
				}
				selectedOntologyClass = "";
			}
		});
		springLayout.putConstraint(SpringLayout.NORTH, addExceptionButton, 6, SpringLayout.SOUTH, separator_2);
		springLayout.putConstraint(SpringLayout.EAST, addExceptionButton, 0, SpringLayout.EAST, addParameterButton);
		addExceptionButton.setForeground(Color.BLACK);
		addExceptionButton.setFont(new Font("Dialog", Font.BOLD, 12));
		addExceptionButton.setBackground(Color.WHITE);
		add(addExceptionButton);
		
		JSeparator separator_3 = new JSeparator();
		springLayout.putConstraint(SpringLayout.NORTH, separator_3, 3, SpringLayout.SOUTH, editExceptionButton);
		springLayout.putConstraint(SpringLayout.WEST, separator_3, 0, SpringLayout.WEST, this);
		springLayout.putConstraint(SpringLayout.EAST, separator_3, 0, SpringLayout.EAST, separator);
		separator_3.setBackground(Color.BLACK);
		add(separator_3);
		
		JSeparator separator_4 = new JSeparator();
		springLayout.putConstraint(SpringLayout.WEST, separator_4, 0, SpringLayout.WEST, this);
		springLayout.putConstraint(SpringLayout.EAST, separator_4, 0, SpringLayout.EAST, separator);
		separator_4.setBackground(Color.BLACK);
		add(separator_4);
		
		JScrollPane scrollPane = new JScrollPane();
		springLayout.putConstraint(SpringLayout.NORTH, scrollPane, 3, SpringLayout.SOUTH, separator_3);
		springLayout.putConstraint(SpringLayout.SOUTH, scrollPane, -30, SpringLayout.NORTH, separator_4);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		springLayout.putConstraint(SpringLayout.WEST, scrollPane, 0, SpringLayout.WEST, this);
		springLayout.putConstraint(SpringLayout.EAST, scrollPane, 0, SpringLayout.EAST, this);
		add(scrollPane);
		
		textArea = new JTextArea();
		textArea.setForeground(Color.WHITE);
		textArea.setBackground(Color.BLACK);
		textArea.setFont(new Font("Consolas", Font.BOLD, 14));
		textArea.setCaretColor(Color.WHITE);
		textArea.setLineWrap(true);
		textArea.setWrapStyleWord(true);
		scrollPane.setViewportView(textArea);
		
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				editMethodDialog.dispose();
			}
		});
		springLayout.putConstraint(SpringLayout.SOUTH, separator_4, -6, SpringLayout.NORTH, cancelButton);
		springLayout.putConstraint(SpringLayout.SOUTH, cancelButton, -10, SpringLayout.SOUTH, this);
		springLayout.putConstraint(SpringLayout.EAST, cancelButton, 0, SpringLayout.EAST, separator);
		cancelButton.setForeground(Color.BLACK);
		cancelButton.setFont(new Font("Dialog", Font.BOLD, 12));
		cancelButton.setBackground(Color.WHITE);
		add(cancelButton);
		
		JButton setButton = new JButton("Set");
		setButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(MethodParser.checkSyntax(textArea.getText())) {
					MethodData methodData = MethodParser.getMethodData(textArea.getText());
					
					if(Arrays.asList(javaKeywords).contains(methodData.getName()) || Character.isUpperCase(methodData.getName().charAt(0))) {
						JOptionPane.showMessageDialog(new JFrame(), INVALID_METHOD_NAME_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
						return;
					}

					if(typeCheckMethodData(methodData)) { 
						updateUpperInterface(methodData);
						updateMethodTextArea();
					}
					else
						return;
				}
				else {
					JOptionPane.showMessageDialog(new JFrame(), INVALID_METHOD_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
					return;
				}

				try {
					List<Method> methods = MethodParser.parseString(textArea.getText());
					if(!editMode) {
						if(isDuplicateSignature(methods)) {
							JOptionPane.showMessageDialog(new JFrame(), OVERLOAD_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
							return;
						}
						parentPanel.addMethods(methods);
					}
					else {
						if(!methods.get(0).getSignature().equals(editModeStartingSignature)) {
							if(isDuplicateSignature(methods)) {
								JOptionPane.showMessageDialog(new JFrame(), OVERLOAD_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
								return;
							}
						}
						parentPanel.changeMethod(methods.get(0), rowToEdit);
					}
					editMethodDialog.dispose();
				}
				catch (ParseCancellationException e1) { 
					JOptionPane.showMessageDialog(new JFrame(), SYNTAX_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		springLayout.putConstraint(SpringLayout.NORTH, setButton, 0, SpringLayout.NORTH, cancelButton);
		springLayout.putConstraint(SpringLayout.EAST, setButton, -1, SpringLayout.WEST, cancelButton);
		setButton.setForeground(Color.BLACK);
		setButton.setFont(new Font("Dialog", Font.BOLD, 12));
		setButton.setBackground(Color.WHITE);
		add(setButton);
		
			JButton ontologyButton = new JButton("Ontology");
			ontologyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				OntologyClassSelectorDialog dialog = new OntologyClassSelectorDialog(editMethodPanel, project, null, false);
				dialog.setVisible(true);
				if(!selectedOntologyClass.equals("")) {
					editMethodPanel.javaComboBox.setSelectedIndex(0);
					returnTypeTextField.setText(selectedOntologyClass);
					updateMethodTextArea();
				}
				selectedOntologyClass = "";
			}
		});
		springLayout.putConstraint(SpringLayout.SOUTH, ontologyButton, -5, SpringLayout.NORTH, separator);
		springLayout.putConstraint(SpringLayout.EAST, ontologyButton, 0, SpringLayout.EAST, separator);
		ontologyButton.setForeground(Color.BLACK);
		ontologyButton.setFont(new Font("Dialog", Font.BOLD, 12));
		ontologyButton.setBackground(Color.WHITE);
		add(ontologyButton);
		
		String[] items = new String[] {"void", "Object", "String", "byte", "short", "int", "long", "float", "double", 
                                   "boolean", "char", "Byte", "Short", "Integer", "Long", "Float", "Double", 
                                   "Boolean", "Character", "List", "Set", "Map"};
		javaComboBox = new JComboBox(items);
		javaComboBox.setBackground(Color.WHITE);
		springLayout.putConstraint(SpringLayout.WEST, javaComboBox, 6, SpringLayout.EAST, returnTypeTextField);
		springLayout.putConstraint(SpringLayout.SOUTH, javaComboBox, -6, SpringLayout.NORTH, separator);
		springLayout.putConstraint(SpringLayout.EAST, javaComboBox, -5, SpringLayout.WEST, ontologyButton);
		javaComboBox.setFont(new Font("Dialog", Font.BOLD, 12));
		javaComboBox.addActionListener (new ActionListener () {
	    public void actionPerformed(ActionEvent e) {
	    	returnTypeTextField.setText(javaComboBox.getSelectedItem().toString());
	    	updateMethodTextArea();
	    }
	  });
		add(javaComboBox);
		javaComboBox.setSelectedIndex(0);
		
		if(editMode) {
			int row = parentPanel.getCurrentTable().getSelectedRow();
			String method = (String) parentPanel.getCurrentTable().getValueAt(row, 0);
			textArea.setText(method + ";");
			updateUpperInterface(MethodParser.getMethodData(textArea.getText()));
			rowToEdit = row;
			editModeStartingSignature = MethodParser.parseString(textArea.getText()).get(0).getSignature();
			updateMethodTextArea();
		}
		
		applyButton = new JButton("Apply");
		applyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(MethodParser.checkSyntax(textArea.getText())) {
					MethodData methodData = MethodParser.getMethodData(textArea.getText());
					
					if(Arrays.asList(javaKeywords).contains(methodData.getName()) || Character.isUpperCase(methodData.getName().charAt(0))) {
						JOptionPane.showMessageDialog(new JFrame(), INVALID_METHOD_NAME_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
						return;
					}
					
					for(String p : methodData.getParameters())
						if(Arrays.asList(javaKeywords).contains(p.split(" ")[p.split(" ").length - 1])) {
							JOptionPane.showMessageDialog(new JFrame(), INVALID_PARAMETER_NAME_ERROR , "Error", JOptionPane.ERROR_MESSAGE);
							return;
						}
					
					if(typeCheckMethodData(methodData)) {
					  updateUpperInterface(methodData);
					  updateMethodTextArea();
					}
				}
				else
					JOptionPane.showMessageDialog(new JFrame(), SYNTAX_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
			}
		});
		springLayout.putConstraint(SpringLayout.NORTH, applyButton, 6, SpringLayout.SOUTH, scrollPane);
		springLayout.putConstraint(SpringLayout.SOUTH, applyButton, 29, SpringLayout.SOUTH, scrollPane);
		springLayout.putConstraint(SpringLayout.EAST, applyButton, 0, SpringLayout.EAST, separator);
		applyButton.setForeground(Color.BLACK);
		applyButton.setFont(new Font("Dialog", Font.BOLD, 12));
		applyButton.setBackground(Color.WHITE);
		add(applyButton);
		
	}
	
	private void updateUpperInterface(MethodData methodData) {
		returnTypeTextField.setText(methodData.getReturnType());
		
		parametersComboBox.removeAllItems();
		for(String par : methodData.getParameters())
			parametersComboBox.addItem(par);
		
		exceptionsComboBox.removeAllItems();
		for(String e : methodData.getExceptions())
			exceptionsComboBox.addItem(e);
		
		for (int i = 0; i < javaComboBox.getItemCount(); i++)
			if(javaComboBox.getItemAt(i).equals(methodData.getReturnType())) {
				javaComboBox.setSelectedIndex(i);
				break;
			}
		
		methodNameTextField.setText(methodData.getName());
	}
	
  private void updateMethodTextArea() {
  	String method = "";
  	method += returnTypeTextField.getText() + " ";
  	method += methodNameTextField.getText()  + " ";
  	method += "(";
  	for (int i = 0; i < editMethodPanel.parametersComboBox.getItemCount(); i++)
  		if(i + 1 < editMethodPanel.parametersComboBox.getItemCount())
  		  method += editMethodPanel.parametersComboBox.getItemAt(i) + ", ";
  		else
  			method += editMethodPanel.parametersComboBox.getItemAt(i);
  	method += ") ";
  	for (int i = 0; i < editMethodPanel.exceptionsComboBox.getItemCount(); i++) {
  		if(i == 0)
  			method += "throws ";
  		if(i + 1 < editMethodPanel.exceptionsComboBox.getItemCount())
  		  method += editMethodPanel.exceptionsComboBox.getItemAt(i) + ", ";
  		else
  			method += editMethodPanel.exceptionsComboBox.getItemAt(i);
  	}
  	method += ";";
  	textArea.setText(method);
  }
  
	private void editItem(String editedParameter, int selectedIndex) {
		List<String> items = copyItems();
		editMethodPanel.exceptionsComboBox.removeAllItems();
		for(int i = 0; i < items.size(); i++)
			if(i != selectedIndex)
				editMethodPanel.exceptionsComboBox.addItem(items.get(i));
			else
				editMethodPanel.exceptionsComboBox.addItem(editedParameter);
	}
	
	private List<String> copyItems() {
		List<String> list = new ArrayList<String>();
		for (int i = 0; i < editMethodPanel.exceptionsComboBox.getItemCount(); i++) {
			list.add((String) editMethodPanel.exceptionsComboBox.getItemAt(i));
		}
		return list;
	}
	
	private boolean isDuplicateSignature(List<Method> methods) {
		//Check this table and super-class table
		{
			TableModel model = parentPanel.getCurrentTable().getModel();
			for(int i = 0; i < model.getRowCount(); i++) {
				for(Method m : methods) {
					if(m.getSignature().equals(model.getValueAt(i, 1))) {
						if(parentPanel.getSelectedClassName().equals(model.getValueAt(i, 3)))
							JOptionPane.showMessageDialog(new JFrame(), DUPLICATE_SIGNATURE_THIS_CLASS_ERROR , "Error", JOptionPane.ERROR_MESSAGE);
						else
							JOptionPane.showMessageDialog(new JFrame(), DUPLICATE_SIGNATURE_SUPERCLASS_ERROR + model.getValueAt(i, 3) + 
							": " + model.getValueAt(i, 0), "Error", JOptionPane.ERROR_MESSAGE);
						return true;
					}
				}
			}
		}
		//Check sub-classes
		for(JTable t : parentPanel.getSubTables()) {
			TableModel model = t.getModel();
			for(int i = 0; i < model.getRowCount(); i++) {
				for(Method m : methods) {
					if(m.getSignature().equals(model.getValueAt(i, 1))) {
						JOptionPane.showMessageDialog(new JFrame(), DUPLICATE_SIGNATURE_SUBCLASS_ERROR + 
								parentPanel.reverseLookUpTableToClassName(t) + 
								": " + model.getValueAt(i, 0), "Error", 
								JOptionPane.ERROR_MESSAGE);
						return true;
					}
				}
			}
		}
		//Check inside new methods to add
		for(Method m1 : methods) {
			for(Method m2 : methods) {
				if(m1 != m2 && m1.getSignature().equals(m2.getSignature()))
					return true;
			}
		}
		return false;
	}
	
	private boolean typeCheckMethodData(MethodData methodData) {
		if(!isJavaType(methodData.getReturnType()) && !isOntologyType(methodData.getReturnType())) {
			JOptionPane.showMessageDialog(new JFrame(), RETURN_TYPE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		
		for (int i = 0; i < methodData.getParameters().size(); i++)
			if(!isJavaType(methodData.getParameters().get(i).split(" ")[0]) 
					&& !isOntologyType(methodData.getParameters().get(i).split(" ")[0])) {
				JOptionPane.showMessageDialog(new JFrame(), PARAMETER_TYPE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
				return false;
			}
		
		for (int i = 0; i < methodData.getExceptions().size(); i++)
			if(!isExceptionType(methodData.getExceptions().get(i))) {
				JOptionPane.showMessageDialog(new JFrame(), EXCEPTION_TYPE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
				return false;
			}
		
		return true;
	}
	
	private boolean isJavaType(String type) {
		for(int i = 0; i < javaComboBox.getItemCount(); i++)
			if(javaComboBox.getItemAt(i).equals(type))
			  return true;
		return false;
	}
	
	//Jade classes should be checked with no prefixes.
	private boolean isOntologyType(String type) {
//		for(Object obj : project.getKnowledgeBase().getClses())
//			if(((Cls) obj).getName().contains("#") && ((Cls) obj).getName().split("#")[1].equals(type))
//				return true;
		
		if(isJadeClassName(type))
			return true;
		
		for(Object obj : project.getKnowledgeBase().getClses()) {
			if(obj instanceof AbstractRDFSClass && ((AbstractRDFSClass) obj).getName().contains("#")) {
				AbstractRDFSClass c = (AbstractRDFSClass) obj;
				if(type.equals(getOntologyClassNameWithPrefix(c)))
				  return true;
			}
		}
		
		return false;
	}
	
	private boolean isJadeClassName(String className) {
		if(className.equals("Concept") ||
			 className.equals("AgentAction") ||
			 className.equals("Predicate") ||
			 className.equals("AID"))
			return true;
		return false;
	}
	
	private String getOntologyClassNameWithPrefix(AbstractRDFSClass cls) {
		String[] prefixArray = cls.getPrefixedName().split(":");
		if(prefixArray.length == 2) {
			//return prefixArray[0] + "_" + cls.getName().split("#")[1];
			return prefixArray[0] + "_" + prefixArray[1];
		}
		else
			//return cls.getName().split("#")[1];
			return prefixArray[0];
	}
	
	private boolean isExceptionType(String type) {

		//Forse  meglio mettere l'errore duplicate Exception class quando si avvia la finestra delle eccezioni.
		
		//Il prefisso di OntologyException  arbitrario, bisogna controllare che type  una sottoclasse
		//Ignorare il caso in cui siano presenti pi OntologyException con prefissi diversi ( un errore dell'utente).
		//Per di type so anche il prefisso, vorrei controllare
		
		if(type.equals(""))
			return true;
		
		AbstractRDFSClass ontologyExceptionCls = getOntologyExceptionClass();
		if(ontologyExceptionCls == null)
			return false;
		
	  //Exception_Base = Prefix:Name
		String exceptionBaseProtegePrefixName = ontologyExceptionCls.getPrefixedName();
		//type = Prefix:Name
		String exceptionProtegePrefixName = getProtegePrefixName(type);
		//If type has not Exception as prefix
		if(!exceptionProtegePrefixName.startsWith("Exception"))
			return false;
		
	  //If the exception is the root exception itself
		if(exceptionBaseProtegePrefixName.equals(exceptionProtegePrefixName))
			return true;
		
	  //If the exception is one of the subclasses
		for(Object obj : ontologyExceptionCls.getSubclasses())
			if(obj instanceof AbstractRDFSClass)
				if(((AbstractRDFSClass)obj).getPrefixedName().equals(exceptionProtegePrefixName))
					return true;
		
		return false;

//		//Exceptions are not allowed in OntoScene
//		if(type.startsWith(ONTOSCENE_PREFIX))
//			return false;
//
//		//Get the root exception class
//		AbstractRDFSClass ontologyExceptionCls = getOntologyExceptionClass();
//		if(ontologyExceptionCls == null)
//			return false;
//		String rootProtegePrefixName  = ontologyExceptionCls.getPrefixedName();
		
//		//Exceptions are not allowed in OntoScene
//		if(rootProtegePrefixName.startsWith(ONTOSCENE_PREFIX))
//			return false;
		
//		//Take the root prefix
//		String rootPrefix = null;
//		if(ontologyExceptionCls instanceof AbstractRDFSClass) {
//			rootPrefix = ((AbstractRDFSClass) ontologyExceptionCls).getPrefixedName().split(":")[0]);
//		}
//		else
//			return false;

//		//Take the exception name ignoring his prefix
//		String noPrefixException = null;
//		String[] typeSplitted = type.split("_");
//		if(typeSplitted.length > 1)
//			noPrefixException = typeSplitted[1];
//		else
//			noPrefixException = typeSplitted[0];
		
//		String exceptionProtegePrefixName = getProtegePrefixName(type);
//		
//		if(rootProtegePrefixName.equals(exceptionProtegePrefixName))
//			return true;

//		//If the exception is the root exception itself (not from OntoScene), return true
//		if(ontologyExceptionCls.getName().contains("#") &&
//				ontologyExceptionCls.getName().split("#")[1].equals(noPrefixException)) {
//			if(ontologyExceptionCls instanceof AbstractRDFSClass) {
//				if(!((AbstractRDFSClass) ontologyExceptionCls).getPrefixedName().split(":")[0].equals(ONTOSCENE_PREFIX))
//					return true;
//			}
//		}
//
//		//If the exception is the subclasses (and it is not from OntoScene), return true
//		for(Object o : ontologyExceptionCls.getSubclasses()) // The prefixes of the subclasses are ignored
//			if(((DefaultRDFSNamedClass) o).getURI().contains("#") &&
//					((DefaultRDFSNamedClass) o).getURI().split("#")[1].equals(noPrefixException)) {
//				if(o instanceof AbstractRDFSClass) {
//					if(!((AbstractRDFSClass) o).getPrefixedName().split(":")[0].equals(ONTOSCENE_PREFIX))
//						return true;
//				}
//			}

	
		
		//Cls ontologyExceptionCls = getClassByName(ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION); // The prefix must always be OntoScene_
		
//		Cls ontologyExceptionCls = null;
//		for(Object obj : project.getKnowledgeBase().getClses()) {
//			if(obj instanceof DefaultRDFSNamedClass) {
//				if(((DefaultRDFSNamedClass)obj).getPrefixedName().equals("OntoScene:" + ONTOLOGY_EXCEPTION)) {
//					ontologyExceptionCls = (Cls) obj;
//				}
//			}
//		}
//		if(ontologyExceptionCls == null)
//			return false;
//		
//		String noPrefixType = null;
//		String[] typeSplitted = type.split("_");
//		if(typeSplitted.length > 1)
//			noPrefixType = typeSplitted[1];
//		else
//			noPrefixType = typeSplitted[0];
//		
//		if(ontologyExceptionCls.getName().contains("#") &&
//				ontologyExceptionCls.getName().split("#")[1].equals(noPrefixType))
//			return true;
//		
//		for(Object o : ontologyExceptionCls.getSubclasses()) // The prefixes of the subclasses are ignored
//			if(((DefaultRDFSNamedClass) o).getURI().contains("#") &&
//					((DefaultRDFSNamedClass) o).getURI().split("#")[1].equals(noPrefixType))
//				return true;
//		
//		return false;
	}
	
	private Cls getClassByName(String className) {
		for(Object obj : project.getKnowledgeBase().getClses()) {
			if(((Cls) obj).getName().contains("#") && ((Cls) obj).getName().split("#")[1].equals(className))
				return (Cls) obj;
		}
		return null;
	}
	
  //Get the ontology exception class. It must be under Jade:Concept to be found.
	private AbstractRDFSClass getOntologyExceptionClass() {
		String ontologyExceptionName = getProtegePrefixName(ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION);
		for(Object obj : project.getKnowledgeBase().getClses()) {
			if(obj instanceof AbstractRDFSClass)
				if(ontologyExceptionName.equals(((AbstractRDFSClass)obj).getPrefixedName()))
					return (AbstractRDFSClass) obj;
		}
		return null;
//		for(Object obj : project.getKnowledgeBase().getClses())
//			if(((Cls) obj).getName().contains("#") && ((Cls) obj).getName().split("#")[1].equals(ONTOLOGY_EXCEPTION)) {
//				Collection c = ((Cls) obj).getDirectSuperclasses();
//				for(Object o : c) {
//					if(o instanceof AbstractRDFSClass) {
//						if(c.size() == 1 && ((AbstractRDFSClass) o).getPrefixedName().equals("Jade:Concept")) {
//						  if(obj instanceof AbstractRDFSClass)
//							  return (AbstractRDFSClass) obj;
//						  else
//						  	return null;
//						}
//					}
//				}
//			}
//		return null;
	}
	
//  //Get the ontology exception class. It must be under Jade:Concept to be found.
//	private AbstractRDFSClass getOntologyExceptionClass() {
//		for(Object obj : project.getKnowledgeBase().getClses())
//			if(((Cls) obj).getName().contains("#") && ((Cls) obj).getName().split("#")[1].equals(ONTOLOGY_EXCEPTION)) {
//				Collection c = ((Cls) obj).getDirectSuperclasses();
//				for(Object o : c) {
//					if(o instanceof AbstractRDFSClass) {
//						if(c.size() == 1 && ((AbstractRDFSClass) o).getPrefixedName().equals("Jade:Concept")) {
//						  if(obj instanceof AbstractRDFSClass)
//							  return (AbstractRDFSClass) obj;
//						  else
//						  	return null;
//						}
//					}
//				}
//			}
//		return null;
//	}
	
	private String getProtegePrefixName (String underscorePrefixName) {
		//Generate prefix:className from prefix_className
		String prefix = underscorePrefixName.split("_")[0];
		String otherTypeName = "";
		if(underscorePrefixName.split("_").length > 1) {
			for(int i = 1; i < underscorePrefixName.split("_").length; i++)
				if(i + 1 < underscorePrefixName.split("_").length)
				  otherTypeName +=  underscorePrefixName.split("_")[i] + "_";
				else
					otherTypeName +=  underscorePrefixName.split("_")[i];
		}
		String protegePrefixName; // Prefix:Name
		if(!otherTypeName.equals(""))
			protegePrefixName = prefix + ":" + otherTypeName;
		else
			protegePrefixName = prefix;
		
		return protegePrefixName;
	}
}
