package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.SpringLayout;
import javax.swing.JLabel;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JSeparator;
import javax.swing.JRadioButton;

import edu.stanford.smi.protege.model.Project;

public class EditParameterDialog extends JDialog {
	
	private EditMethodPanel editMethodPanel;
	
	private JTextField typeTextField;
	private JTextField nameTextField;
	private JComboBox javaComboBox;
	
	private boolean editMode;
	private Project project;
	
	private String[] javaKeywords;
	
	//Error messages
	private final String INVALID_NAME_ERROR =	"Invalid name.";
	private final String DUPLICATE_PARAMETER_ERROR =	"Duplicate name.";

	public EditParameterDialog(final EditMethodPanel editMethodPanel, String parameter, Project project) {
		
		init(editMethodPanel, project);
		setTitle("Edit parameter");
		editMode = true;
		typeTextField.setText(parameter.split(" ")[0]);
		javaComboBox.setSelectedItem(parameter.split(" ")[0]);
		nameTextField.setText(parameter.split(" ")[1]);
			
	}
	
	/**
	 * @wbp.parser.constructor
	 */
	public EditParameterDialog(final EditMethodPanel editMethodPanel, Project project) {
		init(editMethodPanel, project);
		setTitle("Add parameter");
		editMode = false;
	}
	
	private void init(final EditMethodPanel editMethodPanel, final Project project) {
		
		javaKeywords = new String[] { "true", "false", "null", "abstract", "continue", "for", "new", "switch",
                                  "assert", "default", "goto", "package", "synchronized",
                                  "boolean", "do", "if", "private", "this",
                                  "break", "double", "implements", "protected", "throw",
                                  "byte", "else", "import", "public", "throws",
                                  "case", "enum", "instanceof", "return", "transient",
                                  "catch", "extends", "int", "short", "try",
                                  "char", "final", "interface", "static", "void",
                                  "class", "finally", "long", "strictfp", "volatile",
                                  "const", "float", "native", "super", "while"
                                };
		
		this.editMethodPanel = editMethodPanel;
		this.project = project;
		this.setModal(true);
		setBounds(450, 100, 450, 130);
		JPanel panel = new JPanel();
		//panel.setBackground(new Color(255, 228, 196));
		panel.setBackground(new Color(236, 233, 216));
		getContentPane().add(panel, BorderLayout.CENTER);
		SpringLayout sl_panel = new SpringLayout();
		panel.setLayout(sl_panel);
		
		JLabel typeLabel = new JLabel("Type:");
		sl_panel.putConstraint(SpringLayout.NORTH, typeLabel, 14, SpringLayout.NORTH, panel);
		typeLabel.setForeground(Color.GRAY);
		typeLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		panel.add(typeLabel);
		
		JButton ontologyButton = new JButton("Ontology");
		sl_panel.putConstraint(SpringLayout.NORTH, ontologyButton, -4, SpringLayout.NORTH, typeLabel);
		ontologyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				OntologyClassSelectorDialog dialog = new OntologyClassSelectorDialog(editMethodPanel, project, null, false);
				dialog.setVisible(true);
				if(!editMethodPanel.selectedOntologyClass.equals("")) {
					javaComboBox.setSelectedIndex(0);
				  typeTextField.setText(editMethodPanel.selectedOntologyClass);
				}
				editMethodPanel.selectedOntologyClass = "";
			}
		});
		ontologyButton.setForeground(Color.BLACK);
		ontologyButton.setFont(new Font("Dialog", Font.BOLD, 12));
		ontologyButton.setBackground(Color.WHITE);
		panel.add(ontologyButton);
		
		String[] items = new String[] {"Object", "String", "byte", "short", "int", "long", "float", "double", 
				                           "boolean", "char", "Byte", "Short", "Integer", "Long", "Float", "Double", 
				                           "Boolean", "Character", "List", "Set", "Map"};
		javaComboBox = new JComboBox(items);
		javaComboBox.setBackground(Color.WHITE);
		sl_panel.putConstraint(SpringLayout.NORTH, javaComboBox, -3, SpringLayout.NORTH, typeLabel);
		sl_panel.putConstraint(SpringLayout.WEST, javaComboBox, 262, SpringLayout.WEST, panel);
		sl_panel.putConstraint(SpringLayout.EAST, javaComboBox, -6, SpringLayout.WEST, ontologyButton);
		javaComboBox.setSelectedIndex(0);
		javaComboBox.setFont(new Font("Dialog", Font.BOLD, 12));
		javaComboBox.addActionListener (new ActionListener () {
	    public void actionPerformed(ActionEvent e) {
	    	typeTextField.setText(javaComboBox.getSelectedItem().toString());
	    }
	  });
		panel.add(javaComboBox);
		
		typeTextField = new JTextField();
		typeTextField.setBackground(Color.WHITE);
		sl_panel.putConstraint(SpringLayout.EAST, typeTextField, -6, SpringLayout.WEST, javaComboBox);
		sl_panel.putConstraint(SpringLayout.WEST, typeTextField, 12, SpringLayout.EAST, typeLabel);
		typeTextField.setFont(new Font("Dialog", Font.BOLD, 12));
		typeTextField.setColumns(10);
		typeTextField.setEnabled(true);
		typeTextField.setEditable(false);
		typeTextField.setCaretColor(Color.WHITE);
		typeTextField.setHighlighter(null);
		typeTextField.setText("Object");
		panel.add(typeTextField);
		
		JSeparator separator = new JSeparator();
		sl_panel.putConstraint(SpringLayout.NORTH, separator, 1, SpringLayout.SOUTH, ontologyButton);
		sl_panel.putConstraint(SpringLayout.EAST, ontologyButton, 0, SpringLayout.EAST, separator);
		sl_panel.putConstraint(SpringLayout.SOUTH, typeTextField, -3, SpringLayout.NORTH, separator);
		sl_panel.putConstraint(SpringLayout.WEST, typeLabel, 0, SpringLayout.WEST, separator);
		sl_panel.putConstraint(SpringLayout.WEST, separator, 0, SpringLayout.WEST, panel);
		sl_panel.putConstraint(SpringLayout.EAST, separator, 0, SpringLayout.EAST, panel);
		separator.setBackground(Color.BLACK);
		panel.add(separator);
		
		nameTextField = new JTextField();
		sl_panel.putConstraint(SpringLayout.NORTH, nameTextField, 3, SpringLayout.SOUTH, separator);
		sl_panel.putConstraint(SpringLayout.EAST, nameTextField, 0, SpringLayout.EAST, panel);
		nameTextField.setFont(new Font("Dialog", Font.BOLD, 12));
		nameTextField.setColumns(10);
		panel.add(nameTextField);
		addWindowListener(new WindowAdapter() {
	    public void windowOpened( WindowEvent e ) {
	    	nameTextField.requestFocus();
	    }
	  });
		
		JLabel nameLabel = new JLabel("Name:");
		sl_panel.putConstraint(SpringLayout.NORTH, nameLabel, 6, SpringLayout.SOUTH, separator);
		sl_panel.putConstraint(SpringLayout.WEST, nameTextField, 6, SpringLayout.EAST, nameLabel);
		sl_panel.putConstraint(SpringLayout.WEST, nameLabel, 0, SpringLayout.WEST, typeLabel);
		nameLabel.setForeground(Color.GRAY);
		nameLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		panel.add(nameLabel);
		
		JSeparator separator_1 = new JSeparator();
		sl_panel.putConstraint(SpringLayout.NORTH, separator_1, 3, SpringLayout.SOUTH, nameTextField);
		sl_panel.putConstraint(SpringLayout.WEST, separator_1, 0, SpringLayout.WEST, panel);
		sl_panel.putConstraint(SpringLayout.EAST, separator_1, 0, SpringLayout.EAST, panel);
		separator_1.setBackground(Color.BLACK);
		panel.add(separator_1);
		
		JButton cancelButton = new JButton("Cancel");
		sl_panel.putConstraint(SpringLayout.NORTH, cancelButton, 6, SpringLayout.SOUTH, separator_1);
		sl_panel.putConstraint(SpringLayout.EAST, cancelButton, 0, SpringLayout.EAST, panel);
		cancelButton.setForeground(Color.BLACK);
		cancelButton.setFont(new Font("Dialog", Font.BOLD, 12));
		cancelButton.setBackground(Color.WHITE);
		panel.add(cancelButton);
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				dispose();
			}
		});
		
		JButton setButton = new JButton("Set");
		sl_panel.putConstraint(SpringLayout.NORTH, setButton, 6, SpringLayout.SOUTH, separator_1);
		sl_panel.putConstraint(SpringLayout.EAST, setButton, -6, SpringLayout.WEST, cancelButton);
		setButton.setForeground(Color.BLACK);
		setButton.setFont(new Font("Dialog", Font.BOLD, 12));
		setButton.setBackground(Color.WHITE);
		panel.add(setButton);
		
		setButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				
				if (!nameTextField.getText().matches("^[_a-zA-Z][_a-zA-Z0-9]*?$") || 
						Arrays.asList(javaKeywords).contains(nameTextField.getText())) {
					JOptionPane.showMessageDialog(new JFrame(), INVALID_NAME_ERROR , "Error", JOptionPane.ERROR_MESSAGE);
				}
				else {
					if(!editMode) {
						String newParameter = typeTextField.getText() + " " + nameTextField.getText();
						for (int i = 0; i < editMethodPanel.parametersComboBox.getItemCount(); i++) {
							if(newParameter.split(" ")[1].equals(((String) editMethodPanel.parametersComboBox.getItemAt(i)).split(" ")[1])) {
								JOptionPane.showMessageDialog(new JFrame(), DUPLICATE_PARAMETER_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
								return;
							}
						}
						editMethodPanel.parametersComboBox.addItem(newParameter);
						editMethodPanel.parametersComboBox.setSelectedIndex(editMethodPanel.parametersComboBox.getItemCount() - 1);
						dispose();
					}
					else {
						int selectedIndex = editMethodPanel.parametersComboBox.getSelectedIndex();
						String editedParameter = typeTextField.getText() + " " + nameTextField.getText();
						for (int i = 0; i < editMethodPanel.parametersComboBox.getItemCount(); i++) {
							if(i != selectedIndex && editedParameter.split(" ")[1].equals(((String) editMethodPanel.parametersComboBox.getItemAt(i)).split(" ")[1])) {
								JOptionPane.showMessageDialog(new JFrame(), DUPLICATE_PARAMETER_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
								return;
							}
						}
//						editMethodPanel.parametersComboBox.setSelectedItem(editMethodPanel.parametersComboBox.addItem();
						editItem(editedParameter, selectedIndex);
						editMethodPanel.parametersComboBox.setSelectedIndex(selectedIndex);
						dispose();
					}
				}
			}
		});
		
		setResizable(false);
		
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				dispose();
			}
		});
	}
	
	private void editItem(String editedParameter, int selectedIndex) {
		List<String> items = copyItems();
		editMethodPanel.parametersComboBox.removeAllItems();
		for(int i = 0; i < items.size(); i++)
			if(i != selectedIndex)
				editMethodPanel.parametersComboBox.addItem(items.get(i));
			else
				editMethodPanel.parametersComboBox.addItem(editedParameter);
	}
	
	private List<String> copyItems() {
		List<String> list = new ArrayList<String>();
		for (int i = 0; i < editMethodPanel.parametersComboBox.getItemCount(); i++) {
			list.add((String) editMethodPanel.parametersComboBox.getItemAt(i));
		}
		return list;
	}
}
