package gui;

import javax.swing.JPanel;
import javax.swing.JSplitPane;

import java.awt.BorderLayout;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Color;

import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.DefaultCellEditor;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.SpringLayout;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.ScrollPaneConstants;
import javax.swing.UIManager;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.tree.TreeModel;

import nl.uva.psy.swi.beangenerator.OntologyBeanGeneratorTab;

import org.antlr.v4.runtime.misc.ParseCancellationException;
import org.apache.commons.io.FilenameUtils;
import org.eclipse.jdt.core.dom.ThisExpression;

import antlr4_parser.Method;
import antlr4_parser.MethodParser;
import xml_parser.XMLParser;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.util.SelectionEvent;
import edu.stanford.smi.protege.util.SelectionListener;
import edu.stanford.smi.protege.widget.InstanceClsesPanel;
import edu.stanford.smi.protegex.owl.model.impl.AbstractRDFSClass;
import edu.stanford.smi.protegex.owl.model.impl.DefaultRDFSNamedClass;
import exceptions.NotWellFormedFileException;
import exceptions.OpenFileException;
import exceptions.UnknownClassException;

public class JavaMethodMapperPanel extends JPanel {
	
  //The class browser component
	private OntologyBeanGeneratorTab ontologyBeanGeneratorTab;
	
	//The interactive panel components
	private JTextField classNameTextField;
	private JTextField xmlFileNameTextField;
	private JTable table; // this always contains the current visible table from the gui
	private JButton addMethodButton;
	private JButton removeMethodButton;
	private JButton openFileButton;
	private JButton saveChangesButton;
	private JButton saveAsButton;
	private JSplitPane splitPane;
	private JScrollPane scrollPane;
	
	//The class browser component
	private InstanceClsesPanel classBrowserPanel;
	
	//Class name to Jtable hash table
	//private HashMap<String, JTable> classToMethodTableMap;
	private ConcurrentHashMap<String, JTable> classToMethodTableMap;
	
	//Protege project
	private Project project;
	
	//The current selected class
	private DefaultRDFSNamedClass selectedClass;
	
	//Different modes to call the method frame from this window
	private final boolean METHOD_DIALOG_ADD_MODE =	false; // to add n methods to a class
	private final boolean METHOD_DIALOG_EDIT_MODE =	true; // to edit the selected method
	
	//Error messages
	private final String 	NO_CLASS_SELECTED_ERROR =	"No class selected.";
	private final String 	NO_METHOD_SELECTED_ERROR = "No method selected.";
	private final String 	COULD_NOT_CREATE_XML_FILE_ERROR = "Could not create xml file.";
	private final String 	COULD_NOT_OPEN_XML_FILE_ERROR = "Could not open xml file.";
	private final String  COULD_NOT_FIND_XML_FILE_ERROR = "Could not find xml file.";
	private final String 	NOT_WELL_FORMED_FILE_ERROR = "This xml file is not well formed (no class or method elements found).";
	private final String 	UNKNOWN_CLASSES_ERROR = "Some unknown classes found in the file (open aborted).";
	private final String 	FILE_METHODS_PARSE_ERROR = "Some methods in the file could not be parsed (open aborted).";
	private final String 	COULD_NOT_SAVE_FILE_ERROR = "Could not save file.";
	private final String 	EDIT_INHERITED_METHOD_SUBCLASS_ERROR = "Cannot edit inherited method from sub-class.";
	private final String 	REMOVE_INHERITED_METHOD_ERROR = "Can not remove inherited method from super-class (remove aborted).";
  
	//Question messages
	private final String 	REMOVE_SINGLE_METHOD_QUESTION =	"Do you really want to remove this method?";
	private final String 	REMOVE_MULTIPLE_METHOD_QUESTION =	"Do you really want to remove these methods?";
	private final String 	OVERWRITE_FILE_QUESTION =	"will be overwritten. Are you sure?";
	
  //Information messages
	private final String 	IMPORTED_OK_INFORMATION =	"imported successfully.";
	private final String 	SAVED_OK_INFORMATION =	"saved successfully.";
	
	private boolean outerTabListenerAdded;
	private boolean stateChangedWithoutSaving;
	private boolean saveWarningFlag;

	//Initialize the panel (some code has been generated with the WindowBuilder plugin for Eclipse)
	public JavaMethodMapperPanel(Project project, final OntologyBeanGeneratorTab ontologyBeanGeneratorTab) {
		this.project = project;
		this.ontologyBeanGeneratorTab = ontologyBeanGeneratorTab;
		
		outerTabListenerAdded = false;
		stateChangedWithoutSaving = false;
		saveWarningFlag = true;
		//classToMethodTableMap = new HashMap<String, JTable>();
		classToMethodTableMap = new ConcurrentHashMap<String, JTable>(); 
		
		//To set all the information, question or error messages to the same font
		UIManager.put("OptionPane.messageFont", new Font("Dialog", Font.PLAIN, 12));
	
		//JavaMethodMapperPanel components code
		setLayout(new BorderLayout(0, 0));
		splitPane = new JSplitPane();
		splitPane.setBackground(Color.WHITE);
		add(splitPane, BorderLayout.CENTER);
		
		JPanel rightPanel = new JPanel();
		rightPanel.setBackground(Color.WHITE);
		splitPane.setRightComponent(rightPanel);
		rightPanel.setLayout(new BorderLayout(0, 0));
		
		JLabel titleLabel = new JLabel("Java Method Mapper 1.0");
		titleLabel.setForeground(Color.WHITE);
		titleLabel.setBackground(Color.BLACK);
		titleLabel.setOpaque(true);
		titleLabel.setFont(new Font("Dialog", Font.BOLD, 16));
		titleLabel.setBackground(new Color(208, 157, 40));
		titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
		rightPanel.add(titleLabel, BorderLayout.NORTH);
		
		JPanel methodPanel = new JPanel();
		//methodPanel.setBackground(new Color(255, 228, 196));
		methodPanel.setBackground(new Color(236, 233, 216));
		rightPanel.add(methodPanel, BorderLayout.CENTER);
		SpringLayout sl_methodPanel = new SpringLayout();
		methodPanel.setLayout(sl_methodPanel);
		
		JLabel classNameLabel = new JLabel("Class name:");
		classNameLabel.setForeground(new Color(128, 128, 128));
		classNameLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		sl_methodPanel.putConstraint(SpringLayout.NORTH, classNameLabel, 10, SpringLayout.NORTH, methodPanel);
		sl_methodPanel.putConstraint(SpringLayout.WEST, classNameLabel, 0, SpringLayout.WEST, methodPanel);
		methodPanel.add(classNameLabel);
		
		classNameTextField = new JTextField();
		classNameTextField.setEnabled(true);
		classNameTextField.setCaretColor(Color.WHITE);
		classNameTextField.setHighlighter(null);
		classNameTextField.setFont(new Font("Dialog", Font.PLAIN, 12));
		classNameTextField.setEditable(false);
		classNameTextField.setBackground(Color.WHITE);
		sl_methodPanel.putConstraint(SpringLayout.NORTH, classNameTextField, 7, SpringLayout.NORTH, methodPanel);
		sl_methodPanel.putConstraint(SpringLayout.WEST, classNameTextField, 6, SpringLayout.EAST, classNameLabel);
		sl_methodPanel.putConstraint(SpringLayout.EAST, classNameTextField, 104, SpringLayout.EAST, methodPanel);
		methodPanel.add(classNameTextField);
		classNameTextField.setColumns(10);
		
		JSeparator separator1 = new JSeparator();
		separator1.setBackground(Color.DARK_GRAY);
		sl_methodPanel.putConstraint(SpringLayout.NORTH, separator1, 6, SpringLayout.SOUTH, classNameLabel);
		sl_methodPanel.putConstraint(SpringLayout.WEST, separator1, 0, SpringLayout.WEST, classNameLabel);
		sl_methodPanel.putConstraint(SpringLayout.EAST, separator1, 0, SpringLayout.EAST, methodPanel);
		methodPanel.add(separator1);
		
		JLabel methodsLabel = new JLabel("Methods:");
		methodsLabel.setForeground(new Color(128, 128, 128));
		methodsLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		sl_methodPanel.putConstraint(SpringLayout.NORTH, methodsLabel, 6, SpringLayout.SOUTH, separator1);
		sl_methodPanel.putConstraint(SpringLayout.WEST, methodsLabel, 0, SpringLayout.WEST, classNameLabel);
		methodPanel.add(methodsLabel);
		
		JSeparator separator2 = new JSeparator();
		sl_methodPanel.putConstraint(SpringLayout.WEST, separator2, 0, SpringLayout.WEST, classNameLabel);
		sl_methodPanel.putConstraint(SpringLayout.SOUTH, separator2, -150, SpringLayout.SOUTH, methodPanel);
		sl_methodPanel.putConstraint(SpringLayout.EAST, separator2, 0, SpringLayout.EAST, methodPanel);
		separator2.setBackground(Color.DARK_GRAY);
		methodPanel.add(separator2);
		
		scrollPane = new JScrollPane();
		sl_methodPanel.putConstraint(SpringLayout.SOUTH, scrollPane, -32, SpringLayout.SOUTH, separator2);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		sl_methodPanel.putConstraint(SpringLayout.NORTH, scrollPane, 6, SpringLayout.SOUTH, methodsLabel);
		sl_methodPanel.putConstraint(SpringLayout.WEST, scrollPane, 0, SpringLayout.WEST, classNameLabel);
		sl_methodPanel.putConstraint(SpringLayout.EAST, scrollPane, 0, SpringLayout.EAST, methodPanel);
		methodPanel.add(scrollPane);
		
		removeMethodButton = new JButton("Remove");
		removeMethodButton.setBackground(Color.WHITE);
		removeMethodButton.setFont(new Font("Dialog", Font.BOLD, 12));
		sl_methodPanel.putConstraint(SpringLayout.NORTH, removeMethodButton, 6, SpringLayout.SOUTH, scrollPane);
		sl_methodPanel.putConstraint(SpringLayout.EAST, removeMethodButton, 0, SpringLayout.EAST, separator1);
		setRemoveMethodButtonListeners();
		methodPanel.add(removeMethodButton);
		
		addMethodButton = new JButton("Add");
		addMethodButton.setBackground(Color.WHITE);
		addMethodButton.setFont(new Font("Dialog", Font.BOLD, 12));
		setAddMethodButtonListeners();
		sl_methodPanel.putConstraint(SpringLayout.NORTH, addMethodButton, 6, SpringLayout.SOUTH, scrollPane);
		
		table = createEmptyTable();
		scrollPane.setViewportView(table);
		sl_methodPanel.putConstraint(SpringLayout.EAST, addMethodButton, -6, SpringLayout.WEST, removeMethodButton);
		methodPanel.add(addMethodButton);
		
		openFileButton = new JButton("Methods file");
		openFileButton.setBackground(Color.WHITE);
		addMethodButton.setBackground(Color.WHITE);
		openFileButton.setFont(new Font("Dialog", Font.BOLD, 12));
		sl_methodPanel.putConstraint(SpringLayout.NORTH, openFileButton, 5, SpringLayout.SOUTH, separator2);
		sl_methodPanel.putConstraint(SpringLayout.WEST, openFileButton, 0, SpringLayout.WEST, classNameLabel);
		setOpenFileButtonListeners();
		methodPanel.add(openFileButton);
		
//		JLabel methodsFileLabel = new JLabel("Methods file:");
//		methodsFileLabel.setForeground(new Color(128, 128, 128));
//		methodsFileLabel.setFont(new Font("Dialog", Font.BOLD, 12));
//		sl_methodPanel.putConstraint(SpringLayout.NORTH, methodsFileLabel, 7, SpringLayout.SOUTH, separator2);
//		sl_methodPanel.putConstraint(SpringLayout.WEST, methodsFileLabel, 0, SpringLayout.WEST, classNameLabel);
//		methodPanel.add(methodsFileLabel);
		
		xmlFileNameTextField = new JTextField();
		xmlFileNameTextField.setBackground(Color.WHITE);
		xmlFileNameTextField.setEnabled(true);
		xmlFileNameTextField.setCaretColor(Color.WHITE);
		xmlFileNameTextField.setHighlighter(null);
		sl_methodPanel.putConstraint(SpringLayout.EAST, xmlFileNameTextField, 0, SpringLayout.EAST, methodPanel);
		xmlFileNameTextField.setEditable(false);
		sl_methodPanel.putConstraint(SpringLayout.NORTH, xmlFileNameTextField, 6, SpringLayout.SOUTH, separator2);
		sl_methodPanel.putConstraint(SpringLayout.WEST, xmlFileNameTextField, 6, SpringLayout.EAST, openFileButton);
		//sl_methodPanel.putConstraint(SpringLayout.WEST, xmlFileNameTextField, 6, SpringLayout.EAST, methodsFileLabel);
		methodPanel.add(xmlFileNameTextField);
		xmlFileNameTextField.setColumns(10);
		xmlFileNameTextField.setFont(new Font("Dialog", Font.PLAIN, 12));
		xmlFileNameTextField.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
				ontologyBeanGeneratorTab.setXMLPath(xmlFileNameTextField.getText());
			}
			public void removeUpdate(DocumentEvent e) {
				ontologyBeanGeneratorTab.setXMLPath(xmlFileNameTextField.getText());
			}
			public void insertUpdate(DocumentEvent e) {
				ontologyBeanGeneratorTab.setXMLPath(xmlFileNameTextField.getText());
			}
		});
		
		saveChangesButton = new JButton("Save Changes");
		saveChangesButton.setBackground(Color.WHITE);
		saveChangesButton.setFont(new Font("Dialog", Font.BOLD, 12));
		sl_methodPanel.putConstraint(SpringLayout.NORTH, saveChangesButton, 6, SpringLayout.SOUTH, xmlFileNameTextField);
		sl_methodPanel.putConstraint(SpringLayout.EAST, saveChangesButton, 0, SpringLayout.EAST, separator1);
		setSaveChangesButtonListeners();
		methodPanel.add(saveChangesButton);
		
		saveAsButton = new JButton("Save As");
		saveAsButton.setBackground(Color.WHITE);
		saveAsButton.setFont(new Font("Dialog", Font.BOLD, 12));
		sl_methodPanel.putConstraint(SpringLayout.NORTH, saveAsButton, 6, SpringLayout.SOUTH, xmlFileNameTextField);
		sl_methodPanel.putConstraint(SpringLayout.EAST, saveAsButton, -100, SpringLayout.EAST, separator1);
		setSaveAsButtonListeners();
		methodPanel.add(saveAsButton);
		
		JPanel leftPanel = new JPanel();
		splitPane.setLeftComponent(leftPanel);
	}
	
	private void setAddMethodButtonListeners() {
		addMethodButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addOuterTabListenerIfMissing();
				if(!classNameTextField.getText().equals("")) // if a class has been selected
				  startEditMethodFrame(METHOD_DIALOG_ADD_MODE);
				else {
          JOptionPane.showMessageDialog(new JFrame(), NO_CLASS_SELECTED_ERROR, "ERROR", JOptionPane.ERROR_MESSAGE);
          return;
				}
			}
		});
	}
	
	private void setRemoveMethodButtonListeners() {
		removeMethodButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addOuterTabListenerIfMissing();
				int methodsCount = table.getSelectedRows().length;
				if(methodsCount == 0) {
					JOptionPane.showMessageDialog(new JFrame(), NO_METHOD_SELECTED_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
					return;
				}
				int n = JOptionPane.showConfirmDialog(
				    new JFrame(),
				    (methodsCount == 1) ? REMOVE_SINGLE_METHOD_QUESTION : REMOVE_MULTIPLE_METHOD_QUESTION,
				    "Warning",
				    JOptionPane.YES_NO_OPTION);
				if (n == JOptionPane.YES_OPTION) {
					DefaultTableModel model = (DefaultTableModel)table.getModel();
					int[] rows = table.getSelectedRows();
					
					if(!areRemovableRows(rows)) {
						JOptionPane.showMessageDialog(new JFrame(), REMOVE_INHERITED_METHOD_ERROR , "Error", JOptionPane.ERROR_MESSAGE);
            return;
					}
					
					List<String> methodSignatures = new ArrayList<String>();
					for(int i = 0; i < rows.length; i++)
						methodSignatures.add((String) model.getValueAt(rows[i], 1));
					
					//Remove methods from this class
					for(int i = 0; i < rows.length; i++) {
						model.removeRow(rows[i]-i);
					}
					
					//Remove inherited methods from sub-classes
					for(Object o : selectedClass.getSubclasses()) {
						if(o instanceof DefaultRDFSNamedClass) {
							JTable subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
							if(subTable == null) {
								classToMethodTableMap.put(((DefaultRDFSNamedClass) o).getURI(), createEmptyTable());
								subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
							}
							TableModel subModel = subTable.getModel();
							List<Integer> rowsToRemove = new ArrayList<Integer>();
							for(int i = 0; i < subModel.getRowCount(); i++) {
								if(methodSignatures.contains(subModel.getValueAt(i, 1)))
									rowsToRemove.add(i);
							}
							for(int i = 0; i < rows.length; i++) {
								((DefaultTableModel) subModel).removeRow(rowsToRemove.get(i)-i);
							}
						}
				  }
					stateChangedWithoutSaving = true;
				}
			}
		});
	}
	
	private boolean areRemovableRows(int[] rows) {
		DefaultTableModel model = (DefaultTableModel)table.getModel();
		for(int i = 0; i < rows.length; i++) {
			if(!selectedClass.getName().equals(model.getValueAt(rows[i]-i, 3)))
				return false;
		}
		return true;
	}
	
//	private void openDefaultXMLFile() {
//		HashMap<String, List<Method>> classToMethodHashMap;
//		
//		String protegeFolderPath = System.getProperty("user.dir");
//		String xmlFilePath = protegeFolderPath + "\\plugins\\nl.uva.psy.swi.beangenerator\\methods.xml";
//		File f = new File(xmlFilePath);
//    if(!f.exists()) {
//  		JOptionPane.showMessageDialog(new JFrame(), "Could not find " + xmlFilePath + " file.", "Error", JOptionPane.ERROR_MESSAGE);
//    	return;
//    }
//		
//		try {
//			classToMethodHashMap = XMLParser.readXMLFile(xmlFilePath);	
//			xmlFileNameTextField.setText(xmlFilePath);
//			createTables(classToMethodHashMap);
//		} 
//  	catch (OpenFileException e) {
//  		xmlFileNameTextField.setText("");
//  		JOptionPane.showMessageDialog(new JFrame(), "Could not open " + xmlFilePath + " file.", "Error", JOptionPane.ERROR_MESSAGE);
//  	} 
////  	catch (NotWellFormedFileException e) {
////  		xmlFileNameTextField.setText("");
////  		JOptionPane.showMessageDialog(new JFrame(), NOT_WELL_FORMED_FILE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
////  	}
//  	catch (UnknownClassException e) {
//  		xmlFileNameTextField.setText("");
//  		JOptionPane.showMessageDialog(new JFrame(), UNKNOWN_CLASSES_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
//  	}
//  	catch (ParseCancellationException e) {
//  		xmlFileNameTextField.setText("");
//  		classToMethodTableMap.clear();
//  		JOptionPane.showMessageDialog(new JFrame(), FILE_METHODS_PARSE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
//  	}  
//		
//		//To refresh the current visible table
//		if(selectedClass != null) {
//			table = classToMethodTableMap.get(selectedClass.getName());
//			if(table != null) {
//				scrollPane.setViewportView(table);
//				scrollPane.repaint();
//			}
//		}
//	}
	
	private void setOpenFileButtonListeners() {
		openFileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				addOuterTabListenerIfMissing();
				
			  JFileChooser chooser = new JFileChooser();
		    FileNameExtensionFilter filter = new FileNameExtensionFilter("xml files (*.xml)", "xml");
		    chooser.setFileFilter(filter);
		    int returnVal = chooser.showOpenDialog(null);
		    if(returnVal == JFileChooser.APPROVE_OPTION) {
		    	HashMap<String, List<Method>> classToMethodHashMap;
		    	try {
		    		File file = new File(chooser.getSelectedFile().getAbsolutePath());
		    		if(!file.exists()) {
		    			PrintWriter writer;
		    	    try {
		    		    writer = new PrintWriter(chooser.getSelectedFile().getAbsolutePath(), "UTF-8");
		    				writer.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		    				writer.println("<root>");
		    				writer.println("</root>");
		    				writer.close();
		    	    }
		    	    catch(Exception e) {
				    		JOptionPane.showMessageDialog(new JFrame(), COULD_NOT_CREATE_XML_FILE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
                return;
		    	    }
		    		}
		    		
		    		classToMethodHashMap = XMLParser.readXMLFile(chooser.getSelectedFile().getAbsolutePath());
		    		xmlFileNameTextField.setText(chooser.getSelectedFile().getAbsolutePath());
		    		createTables(classToMethodHashMap);
		    		JOptionPane.showMessageDialog(new JFrame(), xmlFileNameTextField.getText() + " " + IMPORTED_OK_INFORMATION , "Information", JOptionPane.INFORMATION_MESSAGE);  
		    		stateChangedWithoutSaving = false;
		    	} 
		    	catch (OpenFileException e) {
		    		xmlFileNameTextField.setText("");
		    		JOptionPane.showMessageDialog(new JFrame(), COULD_NOT_OPEN_XML_FILE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
		    	} 
//		    	catch (NotWellFormedFileException e) {
//		    		xmlFileNameTextField.setText("");
//		    		JOptionPane.showMessageDialog(new JFrame(), NOT_WELL_FORMED_FILE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
//		    	}
		    	catch (UnknownClassException e) {
		    		xmlFileNameTextField.setText("");
		    		JOptionPane.showMessageDialog(new JFrame(), UNKNOWN_CLASSES_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
		    	}
		    	catch (ParseCancellationException e) {
		    		xmlFileNameTextField.setText("");
		    		classToMethodTableMap.clear();
		    		JOptionPane.showMessageDialog(new JFrame(), FILE_METHODS_PARSE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
		    	}
		    	//To refresh the current visible table
		    	if(selectedClass != null) {
		    		table = classToMethodTableMap.get(selectedClass.getName());
		    		if(table != null) {
		    			scrollPane.setViewportView(table);
		    			scrollPane.repaint();
		    		}
		    	}
		    }
			}
		});
	}

	private void saveNewFile() {
		JFileChooser chooser = new JFileChooser();
    FileNameExtensionFilter filter = new FileNameExtensionFilter("xml files (*.xml)", "xml");
    chooser.setFileFilter(filter);
    int returnVal = chooser.showSaveDialog(null);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
    	File file = forceXMLExtension(chooser.getSelectedFile());
    	if(!XMLParser.writeXMLFile(file.getAbsolutePath(), classToMethodTableMap))
        JOptionPane.showMessageDialog(new JFrame(), COULD_NOT_SAVE_FILE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
    	else {
    		JOptionPane.showMessageDialog(new JFrame(), file.getAbsolutePath() + " " + SAVED_OK_INFORMATION, "Information", JOptionPane.INFORMATION_MESSAGE);
	    	xmlFileNameTextField.setText(file.getAbsolutePath());
				stateChangedWithoutSaving = false;
    	}
    }
	}
	
	private void setSaveAsButtonListeners() {
		saveAsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addOuterTabListenerIfMissing();
				saveNewFile();
			}
		});
	}
	
	private void saveChanges() {
		addOuterTabListenerIfMissing();
		if(xmlFileNameTextField.getText().equals(""))
			saveNewFile();
		else {
			int n = JOptionPane.showConfirmDialog(
					new JFrame(),
					xmlFileNameTextField.getText() + " " + OVERWRITE_FILE_QUESTION,
					"Warning",
					JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE);
			if (n == JOptionPane.YES_OPTION) {
				if(!XMLParser.writeXMLFile(xmlFileNameTextField.getText(), classToMethodTableMap))
					JOptionPane.showMessageDialog(new JFrame(), COULD_NOT_SAVE_FILE_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
				else {
					JOptionPane.showMessageDialog(new JFrame(), xmlFileNameTextField.getText() + " " +  SAVED_OK_INFORMATION, "Information", JOptionPane.INFORMATION_MESSAGE);
					stateChangedWithoutSaving = false;
				}

			}
		}
	}
	
	private void setSaveChangesButtonListeners() {
		saveChangesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveChanges();
			}
		});
	}
	
	private File forceXMLExtension(File file) {
		if (FilenameUtils.getExtension(file.getName()).equalsIgnoreCase("xml"))
			return file;
		else
			return new File(file.getParentFile(), FilenameUtils.getBaseName(file.getName())+".xml");
	}
	
	private void startEditMethodFrame(boolean editModeFlag) {
		try {
			if(editModeFlag == METHOD_DIALOG_EDIT_MODE) {
				int row = table.getSelectedRow();
			  boolean isMethodFromSameClass = (table.getValueAt(row, 3)).equals(selectedClass.getName());
			  //Override disabled
				if (!isMethodFromSameClass) {
					//return;
					OverrideMethodDialog dialog = new OverrideMethodDialog(this, project);
					dialog.setVisible(true);
					return;
				}
			}
			EditMethodDialog dialog = new EditMethodDialog(this, project, editModeFlag);
			dialog.setVisible(true);
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void addMethods(List<Method> methods) {
		TableModel model = table.getModel();
		for(Method m : methods) {
			Object[] newRow = new Object[]{m.getMethod(), m.getSignature(), "<html> <font color=green > this" +
		   ":: </font> " + m.getNiceFormatting().substring(7), selectedClass.getName()};
	    ((DefaultTableModel) model).addRow(newRow);
	    for(Object o : selectedClass.getSubclasses()) {
	    	if(o instanceof DefaultRDFSNamedClass) {
	    		JTable subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
	    		if(subTable == null) {
	    			classToMethodTableMap.put(((DefaultRDFSNamedClass) o).getURI(), createEmptyTable());
	    			subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
	    		}
	    		TableModel subModel = subTable.getModel();
	    		
	    		Object[] newSubRow = new Object[]{m.getMethod(), m.getSignature(), "<html> <font color=green >" +
	    				selectedClass.getURI().split("#")[1] + ":: </font> " + m.getNiceFormatting().substring(7),
	    				selectedClass.getName()};
//	    		Object[] newSubRow = new Object[]{m.getMethod(), m.getSignature(), "<html> <font color=green >" +
//	    				getOntologyClassNameWithPrefix(selectedClass) + ":: </font> " + m.getNiceFormatting().substring(7),
//	    				selectedClass.getName()};
	    		
	    		
	    		((DefaultTableModel) subModel).addRow(newSubRow);
	    	}
	    }
		}
		stateChangedWithoutSaving = true;
	}
	
	public void changeMethod(Method m, int rowToEdit) {
		//I am using the signature of a method as its own ID
		String oldSignature = (String) table.getValueAt(rowToEdit, 1);
	  table.setValueAt(m.getMethod(), rowToEdit, 0);
	  table.setValueAt(m.getSignature(), rowToEdit, 1);
	  table.setValueAt("<html> <font color=green > this" +
		":: </font> " + m.getNiceFormatting().substring(7), rowToEdit, 2);
	  table.setValueAt(selectedClass.getName(), rowToEdit, 3);
	  table.setValueAt(m.getOverrideMethod(), rowToEdit, 4);
	  
	  for(Object o : selectedClass.getSubclasses()) {
	  	if(o instanceof DefaultRDFSNamedClass) {
	  		JTable subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
	  		if(subTable == null) {
	  			classToMethodTableMap.put(((DefaultRDFSNamedClass) o).getURI(), createEmptyTable());
	  			subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
	  		}
	  		else {
	  			TableModel subModel = subTable.getModel();
	  			for(int i = 0; i < subModel.getRowCount(); i++)
	  				if(oldSignature.equals(subTable.getValueAt(i, 1))) {
	  					subTable.setValueAt(m.getMethod(), i, 0);
	  					subTable.setValueAt(m.getSignature(), i, 1);
	  					
	  					
	  					subTable.setValueAt("<html> <font color=green >" +
	  							selectedClass.getURI().split("#")[1] + ":: </font> " + m.getNiceFormatting().substring(7), i, 2);
//	  					subTable.setValueAt("<html> <font color=green >" +
//	  					getOntologyClassNameWithPrefix(selectedClass) + ":: </font> " + m.getNiceFormatting().substring(7), i, 2);
	  					
	  					
	  					subTable.setValueAt(selectedClass.getName(), i, 3);
	  					subTable.setValueAt(m.getOverrideMethod(), rowToEdit, 4);
	  				}
	  		}
	  	}
	  }
	  stateChangedWithoutSaving = true;
	}
	
	public void overrideMethod(Method m, int rowToEdit) {
		table.setValueAt(m.getMethod(), rowToEdit, 0);
		table.setValueAt(m.getSignature(), rowToEdit, 1);
		table.setValueAt(m.getNiceFormatting(), rowToEdit, 2);
		table.setValueAt(m.getInheritedFrom(), rowToEdit, 3);
		table.setValueAt(m.getOverrideMethod(), rowToEdit, 4);

		for(Object o : selectedClass.getSubclasses()) {
			if(o instanceof DefaultRDFSNamedClass) {
				JTable subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
				if(subTable == null) {
					classToMethodTableMap.put(((DefaultRDFSNamedClass) o).getURI(), createEmptyTable());
					subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
				}
				TableModel subModel = subTable.getModel();
				for(int i = 0; i < subModel.getRowCount(); i++)
					if(m.getSignature().equals(subTable.getValueAt(i, 1))) {
						subTable.setValueAt(m.getMethod(), i, 0);
						subTable.setValueAt(m.getSignature(), i, 1);
						subTable.setValueAt(m.getNiceFormatting(), i, 2);
						subTable.setValueAt(m.getInheritedFrom(), i, 3);
						subTable.setValueAt(m.getOverrideMethod(), i, 4);
					}
			}
		}
		stateChangedWithoutSaving = true;
	}
	
	private void createTables(HashMap<String, List<Method>> classToMethodHashMap) throws UnknownClassException {
		classToMethodTableMap.clear();
		for(String className : classToMethodHashMap.keySet()) {
			JTable newTable = createEmptyTable();
			classToMethodTableMap.put(className, newTable);
			TableModel model = newTable.getModel();
			for(Method m : classToMethodHashMap.get(className))
				if(className.equals(m.getInheritedFrom()))
					((DefaultTableModel) model).addRow(new Object[]{m.getMethod(), m.getSignature(), "<html> <font color=green > this" +
					":: </font> " + m.getNiceFormatting().substring(7), m.getInheritedFrom()});
				else if(m.getOverrideMethod() == null)
					((DefaultTableModel) model).addRow(new Object[]{m.getMethod(), m.getSignature(), "<html> <font color=green >" +
					 m.getInheritedFrom().split("#")[1] + ":: </font> " + m.getNiceFormatting().substring(7), m.getInheritedFrom()});
				else {
					((DefaultTableModel) model).addRow(new Object[]{m.getMethod(), m.getSignature(), m.getNiceFormatting(), m.getInheritedFrom(), m.getOverrideMethod()});
				}
		}
	}
	
	public void initClassBrowserPanel() {
		classBrowserPanel = new InstanceClsesPanel(project);
		classBrowserPanel.setPreferredSize(new Dimension(275, 1000));
		classBrowserPanel.addSelectionListener(new SelectionListener() {
			public void selectionChanged(SelectionEvent e) {
				
				try {
					//Set the selected class
					Object objClass = null;
					if(classBrowserPanel.getSelection() != null)
						if(classBrowserPanel.getSelection() != null)
							objClass = classBrowserPanel.getSelection().iterator().next();

					if(objClass != null && objClass instanceof DefaultRDFSNamedClass) {// it seems to work only with owl class
						selectedClass = ((DefaultRDFSNamedClass) classBrowserPanel.getSelection().iterator().next());

						if(selectedClass.getPrefixedName().equals("OntoScene:Exception")) {
							classNameTextField.setText("");
						  return;
						}
						for(Object o : selectedClass.getSuperclasses()) {
							if(o instanceof AbstractRDFSClass) {
							  if(((AbstractRDFSClass) o).getPrefixedName().equals("OntoScene:Exception")) {
							  	classNameTextField.setText("");
								  return;
							  }
							}
						}
						
						String[] prefixArray = selectedClass.getPrefixedName().split(":");
						if(prefixArray.length == 2)
							if(prefixArray[0].equals("owl") || prefixArray[0].equals("rdfs") ||  
									prefixArray[0].equals("protege") ||  prefixArray[0].equals("Jade") ||
									prefixArray[0].equals("Exception")) {
								classNameTextField.setText("");
								return;
							}
						//				  if(selectedClass.getName().equals(project.getKnowledgeBase().getRootCls().getName()) ||
						//				  	selectedClass.getName().split("#")[1].equals("Concept") || 
						//				  	selectedClass.getName().split("#")[1].equals("AgentAction") ||
						//				  	selectedClass.getName().split("#")[1].equals("AID") ||
						//				  	selectedClass.getName().split("#")[1].equals("Predicate")) {
						//				  	classNameTextField.setText("");
						//				  	return;
						//				  }
					}
					else
						return;
					//				if(selectedClass.getName().equals(project.getKnowledgeBase().getRootCls().getName())) {
					//					classNameTextField.setText("");
					//				}
					//				else
				} catch(Exception e1) {
					classNameTextField.setText("");
					return;
				}
				
				classNameTextField.setText(selectedClass.getName());

				String className = classNameTextField.getText();
				table = classToMethodTableMap.get(className);
				if(table == null) {
					classToMethodTableMap.put(className, createEmptyTable());
					table = classToMethodTableMap.get(className);
				}
				//Refresh visible table
				scrollPane.setViewportView(table);
				scrollPane.repaint();
			}
		});
		splitPane.setLeftComponent(classBrowserPanel);
	}
	
	public JTable createEmptyTable() {
		DefaultTableModel model = new DefaultTableModel() {
			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		}; 
		JTable newEmptyTable = new JTable(model);
		model.addColumn("Method");
		model.addColumn("Signature");
		model.addColumn("NiceFormatting");
		model.addColumn("InheritedFrom");
		model.addColumn("OverrideMethod");
		newEmptyTable.setBackground(Color.WHITE);
		newEmptyTable.getColumnModel().getColumn(0).setCellRenderer(new CustomCellRenderer());
		newEmptyTable.setFont(new Font("Consolas", 1, 14));
		newEmptyTable.setEnabled(true);
		JTextField textField = new JTextField();
		textField.setFont(new Font("Consolas", 1, 14));
		textField.setBorder(new LineBorder(Color.BLACK));
		DefaultCellEditor dce = new DefaultCellEditor(textField);
		newEmptyTable.getColumnModel().getColumn(0).setCellEditor(dce);
		newEmptyTable.setTableHeader(null);
		newEmptyTable.setRowHeight(25);
		newEmptyTable.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					startEditMethodFrame(METHOD_DIALOG_EDIT_MODE);
				}
			}
		});
		//Hide columns (only the 'NiceFormatting' column remains visible)
		newEmptyTable.getColumnModel().getColumn(0).setMinWidth(0);
		newEmptyTable.getColumnModel().getColumn(0).setMaxWidth(0);
		newEmptyTable.getColumnModel().getColumn(1).setMinWidth(0);
		newEmptyTable.getColumnModel().getColumn(1).setMaxWidth(0);
		newEmptyTable.getColumnModel().getColumn(3).setMinWidth(0);
		newEmptyTable.getColumnModel().getColumn(3).setMaxWidth(0);
		newEmptyTable.getColumnModel().getColumn(4).setMinWidth(0);
		newEmptyTable.getColumnModel().getColumn(4).setMaxWidth(0);
		return newEmptyTable;
	}
	
	public String getSelectedClassName() {
		return selectedClass.getName();
	}
	
	public JTable getCurrentTable() {
		return table;
	}
	
	public List<JTable> getSuperTables() {
		List<JTable> tables = new ArrayList<JTable>();
		for(Object o : selectedClass.getSuperclasses()) {
			if(o instanceof DefaultRDFSNamedClass) {
				JTable t = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
				if(t != null && t.getRowCount() > 0)
					tables.add(t);
			}
		}
		return tables;
	}
	
	public List<JTable> getSubTables() {
		List<JTable> tables = new ArrayList<JTable>();
		for(Object o : selectedClass.getSubclasses()) {
			if(o instanceof DefaultRDFSNamedClass) {
				JTable t = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
				if(t != null && t.getRowCount() > 0)
					tables.add(t);
			}
		}
		return tables;
	}
	
	public String reverseLookUpTableToClassName(JTable t) {
		for(String key : classToMethodTableMap.keySet())
			if(classToMethodTableMap.get(key) == t)
				return key;
		return null;
	}
	
	/*If a new class is added below an existing one already containing some methods,
  these methods must be inherited by the new subclass. On the contrary, if a class is
  deleted, his methods must not be exported on the xml file*/
	public void refreshOnClassHierarchyChange() {
		removeDeletedClasses();
		updateNewSubClasses();
	}
	
	private void updateNewSubClasses() {
		for(String cls : classToMethodTableMap.keySet()) {
			JTable table = classToMethodTableMap.get(cls);
			TableModel model = table.getModel();

			List<Method> methods = new ArrayList<Method>();
			for(int i = 0; i < model.getRowCount(); i++) {
				if(model.getValueAt(i, 3).equals(cls)) {//this:: methods
					Method m = MethodParser.parseString(model.getValueAt(i, 0) + ";").get(0);
					m.setInheritedFrom(cls);
				  methods.add(m);
				}
				else if(model.getValueAt(i, 4) != null) {//overridden methods
					Method m = MethodParser.parseString((String) model.getValueAt(i, 4)).get(0);
					m.setInheritedFrom((String) model.getValueAt(i, 3));
					m.setOverrideMethod((String) model.getValueAt(i, 4)); 
				  methods.add(m);
				}
			}
			
			DefaultRDFSNamedClass selectedClass = getClassByName(cls);
			if(selectedClass == null)
				continue;
			
			for(Method m : methods) {
				for(Object o : selectedClass.getSubclasses()) {
					if(o instanceof DefaultRDFSNamedClass) {
						JTable subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
						if(subTable == null) {
							classToMethodTableMap.put(((DefaultRDFSNamedClass) o).getURI(), createEmptyTable());
							subTable = classToMethodTableMap.get(((DefaultRDFSNamedClass) o).getURI());
						}
						TableModel subModel = subTable.getModel();
						boolean duplicateSignature = false;
						for(int i = 0; i < subModel.getRowCount(); i++) {
							if(subModel.getValueAt(i, 1).equals(m.getSignature())) {
								duplicateSignature = true;
								break;
							}
						}
						if(!duplicateSignature) {
							if(m.getOverrideMethod() == null) {
								Object[] newSubRow = new Object[]{m.getMethod(), m.getSignature(), "<html> <font color=green >" +
										selectedClass.getURI().split("#")[1] + ":: </font> " + m.getNiceFormatting().substring(7),
										selectedClass.getName()};
								((DefaultTableModel) subModel).addRow(newSubRow);
							}
							else {
								Object[] newSubRow = new Object[]{m.getMethod(), m.getSignature(), "<html> <font color=gray >@Override </font> <font color=green >" +
										m.getInheritedFrom().split("#")[1] + ":: </font> " + m.getNiceFormatting().substring(7), m.getInheritedFrom(), m.getOverrideMethod()};
								((DefaultTableModel) subModel).addRow(newSubRow);
							}
						}
					}
				}
			}
		}
	}
	
	private DefaultRDFSNamedClass getClassByName(String className) {
		for(Object o : project.getKnowledgeBase().getClses()) {
			if(o instanceof DefaultRDFSNamedClass) {
				DefaultRDFSNamedClass cls = (DefaultRDFSNamedClass) o;
				if(cls.getName().equals(className))
				  return cls;
			}
		}
		return null;
	}
	
	private void removeDeletedClasses() {
		for(String cls : classToMethodTableMap.keySet()) {
			if(getClassByName(cls) == null)
				classToMethodTableMap.remove(cls);
		}
	}
	
	private void addOuterTabListenerIfMissing() {
		if(!outerTabListenerAdded) {
			final JTabbedPane outerTabbedPane = ((JTabbedPane) ontologyBeanGeneratorTab.getParent());
			outerTabbedPane.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					if(outerTabbedPane.getSelectedComponent() != null && outerTabbedPane.getSelectedComponent().getName().equals("OntologyBeanGeneratorTab")) {
						refreshOnClassHierarchyChange();
						saveWarningFlag = true;
						stateChangedWithoutSaving = false;
					}
					if(stateChangedWithoutSaving && outerTabbedPane.getSelectedComponent() != null &&
						!outerTabbedPane.getSelectedComponent().getName().equals("OntologyBeanGeneratorTab")) {
						if(saveWarningFlag) {
							int n = JOptionPane.showConfirmDialog(
									new JFrame(),
									"Ontology Bean Generator 5.0: do you want to save changes?",
									"Warning",
									JOptionPane.YES_NO_OPTION);
							if (n == JOptionPane.YES_OPTION) 
								saveChanges();
							
							saveWarningFlag = false;
						}
					}
				}
			});
			outerTabListenerAdded = true;
		}
	}
	
	private boolean isJadeClassName(String className) {
		if(className.equals("Concept") ||
			 className.equals("AgentAction") ||
			 className.equals("Predicate") ||
			 className.equals("AID"))
			return true;
		return false;
	}
	
	private String getOntologyClassNameWithPrefix(DefaultRDFSNamedClass cls) {
		String[] prefixArray = cls.getPrefixedName().split(":");
		if(prefixArray.length == 2) {
			
			if(prefixArray[0].equals("Jade") && isJadeClassName(prefixArray[1])) 
				return cls.getName().split("#")[1];
			
			return prefixArray[0] + "_" + cls.getName().split("#")[1];
		}
		else
			return cls.getName().split("#")[1];
	}
}

