package gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.widget.InstanceClsesPanel;
import edu.stanford.smi.protegex.owl.model.impl.AbstractRDFSClass;
import edu.stanford.smi.protegex.owl.model.impl.DefaultRDFSNamedClass;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;

public class OntologyClassSelectorDialog extends JDialog {

	private Project project;
	private CustomInstanceClsesPanel classBrowserPanel;
	private boolean error = false;
//	private int NO_CLASS_FOUND = 1;
//	private int DUPLICATE_ONTOOGY_EXCEPTION = 2;
//	private int error = 0;
	private final JPanel contentPanel = new JPanel();
	
	private String ONTOLOGY_EXCEPTION_PREFIX = "Exception_";
	private String ONTOLOGY_EXCEPTION = "Base";

	public OntologyClassSelectorDialog(final Object callerPanel, final Project project, final String className, final boolean exceptions) {
		this.project = project;
		setModal(true);
		setTitle("Choose class");
		setBounds(450, 100, 450, 300);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(new BorderLayout(0, 0));
		{
      if(className == null)
			  classBrowserPanel = new CustomInstanceClsesPanel(project, project.getKnowledgeBase().getRootCls());
//      else if (className.equals(ONTOLOGY_EXCEPTION)) {
//      	if(!checkOnlyOneOntologyExceptionClass()) {
//      		error = DUPLICATE_ONTOOGY_EXCEPTION;
//      	}
//      }
      else {
      	Cls root = getClassByPrefixAndName(className);
      	if(root == null)
      		error = true;
      		//error = NO_CLASS_FOUND;
      	else
      		classBrowserPanel = new CustomInstanceClsesPanel(project, root);
      }
      if(classBrowserPanel != null)
			  contentPanel.add(classBrowserPanel, BorderLayout.CENTER);
		}
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("Ok");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						DefaultRDFSNamedClass selectedClass = ((DefaultRDFSNamedClass) classBrowserPanel.getSelection().iterator().next());
						
						if(isValidPrefixClassName(selectedClass, exceptions)) {
							if(callerPanel instanceof EditMethodPanel) {
								//((EditMethodPanel) callerPanel).selectedOntologyClass = selectedClass.getName().split("#")[1];
								((EditMethodPanel) callerPanel).selectedOntologyClass = getOntologyClassNameWithPrefix(selectedClass);
							}
							if(callerPanel instanceof OverrideMethodDialog) {
								//((OverrideMethodDialog) callerPanel).selectedOntologyClass = selectedClass.getName().split("#")[1];
								((OverrideMethodDialog) callerPanel).selectedOntologyClass = getOntologyClassNameWithPrefix(selectedClass);
							}
							dispose();
						}
						else { // Cancel
							JOptionPane.showMessageDialog(new JFrame(), "Invalid class.", "ERROR", JOptionPane.ERROR_MESSAGE);
							if(callerPanel instanceof EditMethodPanel)
								((EditMethodPanel) callerPanel).selectedOntologyClass = "";
							if(callerPanel instanceof OverrideMethodDialog)
								((OverrideMethodDialog) callerPanel).selectedOntologyClass = "";
							dispose();
						}
						
					}
				});
				okButton.setActionCommand("Ok");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.setActionCommand("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						if(callerPanel instanceof EditMethodPanel)
							((EditMethodPanel) callerPanel).selectedOntologyClass = "";
						if(callerPanel instanceof OverrideMethodDialog)
							((OverrideMethodDialog) callerPanel).selectedOntologyClass = "";
						dispose();
					}
				});
				buttonPane.add(cancelButton);
			}
		}
		
		addWindowListener(new WindowAdapter() {
			public void windowOpened( WindowEvent e ) {
				if(error) {
					JOptionPane.showMessageDialog(new JFrame(), "Cannot find " + className + " class.", "Error", JOptionPane.ERROR_MESSAGE);
					dispose();
				}
//				if(error == NO_CLASS_FOUND) {
//					JOptionPane.showMessageDialog(new JFrame(), "Cannot find " + className + " class.", "Error", JOptionPane.ERROR_MESSAGE);
//					dispose();
//				}
//				else if(error == DUPLICATE_ONTOOGY_EXCEPTION) {
//					JOptionPane.showMessageDialog(new JFrame(), "Duplicate " + ONTOLOGY_EXCEPTION + " class.", "Error", JOptionPane.ERROR_MESSAGE);
//					dispose();
//				}
			}
		});
	}
	
	private Cls getClassByPrefixAndName(String className /*Prefix_Name*/) {
		
//  	//Treat the OntologyException differently (because his prefix can be arbitrary
//  	if(className.equals(ONTOLOGY_EXCEPTION))
//  		return getOntologyExceptionClass();
		
		//Generate prefix:className from prefix_className
//		String prefix = className.split("_")[0];
//		String otherTypeName = "";
//		if(className.split("_").length > 1) {
//			for(int i = 1; i < className.split("_").length; i++)
//				otherTypeName +=  className.split("_")[i];
//		}
//		String ontologyClassName; // Prefix:Name
//		if(!otherTypeName.equals(""))
//			ontologyClassName = prefix + ":" + otherTypeName;
//		else
//			ontologyClassName = prefix;
		
	  //Generate prefix:className from prefix_className
		String ontologyClassName = getProtegePrefixName (className);
		
		//Put the prefix for Jade again
		if(isJadeClassName(ontologyClassName))
			ontologyClassName = "Jade:" + ontologyClassName;
		
	  //Get class from prefix:className
		for(Object obj : project.getKnowledgeBase().getClses()) {
//			if(((Cls) obj).getName().contains("#") && ((Cls) obj).getName().split("#")[1].equals(className))
//				return (Cls) obj;
			
			if(obj instanceof DefaultRDFSNamedClass) {
				if(ontologyClassName.equals(((DefaultRDFSNamedClass)obj).getPrefixedName()))
				  return (Cls) obj;
			}
		}
		return null;
	}
	
	private boolean isValidPrefixClassName (DefaultRDFSNamedClass selectedClass, boolean exceptionsMode) {
		String[] prefixArray = selectedClass.getPrefixedName().split(":");
		if(prefixArray.length == 2) {
			if(prefixArray[0].equals("owl") ||
				 prefixArray[0].equals("rdfs") ||
				 prefixArray[0].equals("protege"))
				return false;
		}
		
		if(exceptionsMode) {
			if(!prefixArray[0].equals("Exception"))
				return false;
		}
		
//		if(!exceptionsAllowed) {
//			//Exceptions
//			if(selectedClass.getPrefixedName().equals("OntoScene:Exception")) {
//				return false;
//			}
//			for(Object o : selectedClass.getSuperclasses()) {
//				if(o instanceof AbstractRDFSClass) {
//					if(((AbstractRDFSClass) o).getPrefixedName().equals("OntoScene:Exception")) {
//						return false;
//					}
//				}
//			}
//		}
		
		return true;
	}
	
	private boolean isJadeClassName(String className) {
		if(className.equals("Concept") ||
			 className.equals("AgentAction") ||
			 className.equals("Predicate") ||
			 className.equals("AID"))
			return true;
		return false;
	}
	
	//The user should remember to not use the Jade interface names in an ontology with no prefix
	private String getOntologyClassNameWithPrefix(DefaultRDFSNamedClass selectedClass) {
		String[] prefixArray = selectedClass.getPrefixedName().split(":");
		if(prefixArray.length == 2) {
			
			if(prefixArray[0].equals("Jade") && isJadeClassName(prefixArray[1]))
				return prefixArray[1];
				//return selectedClass.getName().split("#")[1];
			
			//return prefixArray[0] + "_" + selectedClass.getName().split("#")[1];
			return prefixArray[0] + "_" + prefixArray[1];
		}
		else
			//return selectedClass.getName().split("#")[1];
			return prefixArray[0];
	}
	
	//Get the ontology exception class. It must be under Jade:Concept to be found.
	private Cls getOntologyExceptionClass() {
		for(Object obj : project.getKnowledgeBase().getClses())
			if(((Cls) obj).getName().contains("#") && ((Cls) obj).getName().split("#")[1].equals(ONTOLOGY_EXCEPTION)) {
				Collection c = ((Cls) obj).getDirectSuperclasses();
				for(Object o : c) {
					if(o instanceof AbstractRDFSClass) {
						if(c.size() == 1 && ((AbstractRDFSClass) o).getPrefixedName().equals("Jade:Concept")) {
							return (Cls) obj;
						}
					}
				}
			}
		return null;
	}
	
	private boolean checkOnlyOneOntologyExceptionClass() {
		int count = 0;
		for(Object obj : project.getKnowledgeBase().getClses())
			if(((Cls) obj).getName().contains("#") && ((Cls) obj).getName().split("#")[1].equals(ONTOLOGY_EXCEPTION))
				count++;
		if(count == 1)
			return true;
		return false;
	}
	
	private String getProtegePrefixName (String underscorePrefixName) {
		//Generate prefix:className from prefix_className
		String prefix = underscorePrefixName.split("_")[0];
		String otherTypeName = "";
		if(underscorePrefixName.split("_").length > 1) {
			for(int i = 1; i < underscorePrefixName.split("_").length; i++)
				if(i + 1 < underscorePrefixName.split("_").length)
				  otherTypeName +=  underscorePrefixName.split("_")[i] + "_";
				else
					otherTypeName +=  underscorePrefixName.split("_")[i];
		}
		String protegePrefixName; // Prefix:Name
		if(!otherTypeName.equals(""))
			protegePrefixName = prefix + ":" + otherTypeName;
		else
			protegePrefixName = prefix;
		
		return protegePrefixName;
	}
}
