package gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.SpringLayout;
import javax.swing.JLabel;

import java.awt.Font;
import java.awt.Color;

import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JSeparator;

import antlr4_parser.Method;
import antlr4_parser.MethodData;
import antlr4_parser.MethodParser;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protegex.owl.model.impl.DefaultRDFSNamedClass;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

public class OverrideMethodDialog extends JDialog {
	private OverrideMethodDialog overrideMethodDialog = this;
	private JComboBox javaComboBox;
	private JComboBox exceptionsComboBox;
	private JTextField returnTypeTextField;
	private JavaMethodMapperPanel parentPanel;
	private Project project;
	
	private int rowToEdit;
	private MethodData methodData;
	
	public String selectedOntologyClass;
	
	private final String DUPLICATE_EXCEPTION_ERROR =	"Duplicate exception.";
	
	public OverrideMethodDialog(final JavaMethodMapperPanel parentPanel, final Project project) {
		this.parentPanel = parentPanel;
		this.project = project;
		
		setTitle("Override method");
		setBounds(450, 100, 450, 135);
		setResizable(false);
		setModal(true);
		getContentPane().setLayout(new BorderLayout(0, 0));
		
		JPanel panel = new JPanel();
		//panel.setBackground(new Color(255, 228, 196));
		setBackground(new Color(236, 233, 216));
		getContentPane().add(panel, BorderLayout.CENTER);
		SpringLayout sl_panel = new SpringLayout();
		panel.setLayout(sl_panel);
		
		JLabel returnTypeLabel = new JLabel("Return Type:");
		sl_panel.putConstraint(SpringLayout.NORTH, returnTypeLabel, 5, SpringLayout.NORTH, panel);
		sl_panel.putConstraint(SpringLayout.WEST, returnTypeLabel, 0, SpringLayout.WEST, panel);
		returnTypeLabel.setForeground(Color.GRAY);
		returnTypeLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		panel.add(returnTypeLabel);
		
		returnTypeTextField = new JTextField();
		returnTypeTextField.setBackground(Color.WHITE);
		sl_panel.putConstraint(SpringLayout.NORTH, returnTypeTextField, 0, SpringLayout.NORTH, returnTypeLabel);
		sl_panel.putConstraint(SpringLayout.WEST, returnTypeTextField, 6, SpringLayout.EAST, returnTypeLabel);
		sl_panel.putConstraint(SpringLayout.EAST, returnTypeTextField, -172, SpringLayout.EAST, panel);
		returnTypeTextField.setFont(new Font("Dialog", Font.BOLD, 12));
		returnTypeTextField.setEnabled(true);
		returnTypeTextField.setEditable(false);
		returnTypeTextField.setColumns(10);
		returnTypeTextField.setCaretColor(Color.WHITE);
		panel.add(returnTypeTextField);
		
		JButton ontologyButton = new JButton("Ontology");
		ontologyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String root = methodData.getReturnType();
				if(isOntologyType(root)) {
					OntologyClassSelectorDialog dialog = new OntologyClassSelectorDialog(overrideMethodDialog, project, root, false);
					dialog.setVisible(true);
					if(selectedOntologyClass != null && !selectedOntologyClass.equals("")) {
						returnTypeTextField.setText(selectedOntologyClass);
					}
				}
				selectedOntologyClass = "";
			}
		});
		sl_panel.putConstraint(SpringLayout.NORTH, ontologyButton, -1, SpringLayout.NORTH, returnTypeLabel);
		sl_panel.putConstraint(SpringLayout.EAST, ontologyButton, 0, SpringLayout.EAST, panel);
		ontologyButton.setForeground(Color.BLACK);
		ontologyButton.setFont(new Font("Dialog", Font.BOLD, 12));
		ontologyButton.setBackground(Color.WHITE);
		panel.add(ontologyButton);
		
		JSeparator separator = new JSeparator();
		sl_panel.putConstraint(SpringLayout.NORTH, separator, 4, SpringLayout.SOUTH, ontologyButton);
		sl_panel.putConstraint(SpringLayout.WEST, separator, 0, SpringLayout.WEST, panel);
		sl_panel.putConstraint(SpringLayout.EAST, separator, 0, SpringLayout.EAST, panel);
		separator.setBackground(Color.BLACK);
		panel.add(separator);
		
		JButton editButton = new JButton("Edit");
		editButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(methodData.getExceptions().isEmpty())
					return;
				String root = methodData.getExceptions().get(exceptionsComboBox.getSelectedIndex());
				OntologyClassSelectorDialog dialog = new OntologyClassSelectorDialog(overrideMethodDialog, project, root, true);
				dialog.setVisible(true);
				if(selectedOntologyClass != null && !selectedOntologyClass.equals("")) {
					int selectedIndex = overrideMethodDialog.exceptionsComboBox.getSelectedIndex();
					for (int i = 0; i < overrideMethodDialog.exceptionsComboBox.getItemCount(); i++) {
						if(i != selectedIndex && overrideMethodDialog.selectedOntologyClass.equals(overrideMethodDialog.exceptionsComboBox.getItemAt(i))) {
							JOptionPane.showMessageDialog(new JFrame(), DUPLICATE_EXCEPTION_ERROR, "Error", JOptionPane.ERROR_MESSAGE);
							return;
						}
					}
					editItem(selectedOntologyClass, selectedIndex);
				}
				selectedOntologyClass = "";
			}
		});
		sl_panel.putConstraint(SpringLayout.NORTH, editButton, 7, SpringLayout.SOUTH, separator);
		sl_panel.putConstraint(SpringLayout.EAST, editButton, 0, SpringLayout.EAST, ontologyButton);
		editButton.setForeground(Color.BLACK);
		editButton.setFont(new Font("Dialog", Font.BOLD, 12));
		editButton.setBackground(Color.WHITE);
		panel.add(editButton);
		
		exceptionsComboBox = new JComboBox();
		exceptionsComboBox.setBackground(Color.WHITE);
		sl_panel.putConstraint(SpringLayout.NORTH, exceptionsComboBox, 8, SpringLayout.SOUTH, separator);
		sl_panel.putConstraint(SpringLayout.EAST, exceptionsComboBox, -64, SpringLayout.EAST, panel);
		sl_panel.putConstraint(SpringLayout.WEST, editButton, 6, SpringLayout.EAST, exceptionsComboBox);
		exceptionsComboBox.setFont(new Font("Dialog", Font.BOLD, 12));
		panel.add(exceptionsComboBox);
		
		JLabel exceptionsLabel = new JLabel("Exceptions:");
		sl_panel.putConstraint(SpringLayout.NORTH, exceptionsLabel, 8, SpringLayout.SOUTH, separator);
		sl_panel.putConstraint(SpringLayout.WEST, exceptionsComboBox, 6, SpringLayout.EAST, exceptionsLabel);
		sl_panel.putConstraint(SpringLayout.WEST, exceptionsLabel, 0, SpringLayout.WEST, returnTypeLabel);
		exceptionsLabel.setForeground(Color.GRAY);
		exceptionsLabel.setFont(new Font("Dialog", Font.BOLD, 12));
		panel.add(exceptionsLabel);
		
		JSeparator separator_1 = new JSeparator();
		sl_panel.putConstraint(SpringLayout.NORTH, separator_1, 6, SpringLayout.SOUTH, editButton);
		sl_panel.putConstraint(SpringLayout.WEST, separator_1, 0, SpringLayout.WEST, panel);
		sl_panel.putConstraint(SpringLayout.EAST, separator_1, 0, SpringLayout.EAST, panel);
		separator_1.setBackground(Color.BLACK);
		panel.add(separator_1);
		
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		sl_panel.putConstraint(SpringLayout.NORTH, cancelButton, 7, SpringLayout.SOUTH, separator_1);
		sl_panel.putConstraint(SpringLayout.EAST, cancelButton, 0, SpringLayout.EAST, ontologyButton);
		cancelButton.setForeground(Color.BLACK);
		cancelButton.setFont(new Font("Dialog", Font.BOLD, 12));
		cancelButton.setBackground(Color.WHITE);
		panel.add(cancelButton);
		
		JButton overrideButton = new JButton("Override");
		sl_panel.putConstraint(SpringLayout.NORTH, overrideButton, 0, SpringLayout.NORTH, cancelButton);
		sl_panel.putConstraint(SpringLayout.EAST, overrideButton, 0, SpringLayout.WEST, cancelButton);
		overrideButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				methodData.setReturnType(returnTypeTextField.getText());
				List exceptions = new ArrayList<String>();
				for(int i = 0; i < exceptionsComboBox.getItemCount(); i++)
					exceptions.add(exceptionsComboBox.getItemAt(i));
				methodData.setExceptions(exceptions);
				String overrideMethodString = methodData.toString();
				
				//The method visible in the table
				Method method = MethodParser.parseString(((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 0)) + ";").get(0);
			  //If this is the first time that this method is overridden (it is null), then set the overridden method.
				if(method.getOverrideMethod() == null) {
					parentPanel.getCurrentTable().setValueAt(overrideMethodString, rowToEdit, 4);
					method.setOverrideMethod(overrideMethodString);
				}
				//To generate the new nice formatting string
				Method overrideMethod = MethodParser.parseString(overrideMethodString).get(0);

				String newNiceFormatting = "<html> <font color=gray >@Override </font> <font color=green >" + 
						((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 3)).split("#")[1] + ":: </font>" + 
						overrideMethod.getNiceFormatting();     
				method.setNiceFormatting(newNiceFormatting);
				method.setInheritedFrom((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 3));
				
				parentPanel.overrideMethod(method, rowToEdit);
				dispose();
			}
		});
		overrideButton.setForeground(Color.BLACK);
		overrideButton.setFont(new Font("Dialog", Font.BOLD, 12));
		overrideButton.setBackground(Color.WHITE);
		panel.add(overrideButton);
		
		rowToEdit = parentPanel.getCurrentTable().getSelectedRow();
		
		if((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 4) == null)
		  methodData = MethodParser.getMethodData((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 0) + ";");
		else
		  methodData = MethodParser.getMethodData((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 4));

//		methodData = MethodParser.getMethodData((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 0) + ";");

		returnTypeTextField.setText(methodData.getReturnType());
		for(String e : methodData.getExceptions())
			exceptionsComboBox.addItem(e);
		
		if(methodData.getReturnType().equals("Object")) {
			String[] items = new String[] {"Object", "String", "Byte", "Short", "Integer", "Long", "Float", "Double", 
					                           "Boolean", "Character", "List", "Set", "Map"};
			javaComboBox = new JComboBox(items);
		}
		else
	    javaComboBox = new JComboBox();
		
		javaComboBox.setBackground(Color.WHITE);
		sl_panel.putConstraint(SpringLayout.WEST, javaComboBox, 6, SpringLayout.EAST, returnTypeTextField);
		sl_panel.putConstraint(SpringLayout.NORTH, javaComboBox, 5, SpringLayout.NORTH, panel);
		sl_panel.putConstraint(SpringLayout.EAST, javaComboBox, -2, SpringLayout.WEST, ontologyButton);
		javaComboBox.setFont(new Font("Dialog", Font.BOLD, 12));
		panel.add(javaComboBox);
		
		/*You can make a reset by pressing 'override' in the super-class method or 'set' if the method
		  was a plain one*/
		
//		JButton resetButton = new JButton("Reset");
//		sl_panel.putConstraint(SpringLayout.NORTH, resetButton, 0, SpringLayout.NORTH, cancelButton);
//		sl_panel.putConstraint(SpringLayout.WEST, resetButton, 0, SpringLayout.WEST, returnTypeLabel);
//		resetButton.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent arg0) {
//				Method method = MethodParser.parseString(((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 0)) + ";").get(0);
//				method.setOverrideMethod(null);
//				method.setInheritedFrom((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 3));
//				String newNiceFormatting = "<html> <font color=green >" + 
//						((String) parentPanel.getCurrentTable().getValueAt(rowToEdit, 3)).split("#")[1] + ":: </font>" + 
//						method.getNiceFormatting();     
//				method.setNiceFormatting(newNiceFormatting);
//				parentPanel.overrideMethod(method, rowToEdit);
//				dispose();
//			}
//		});
//		resetButton.setForeground(Color.BLACK);
//		resetButton.setFont(new Font("Dialog", Font.BOLD, 12));
//		resetButton.setBackground(Color.WHITE);
//		panel.add(resetButton);
		
		javaComboBox.addActionListener (new ActionListener () {
	    public void actionPerformed(ActionEvent e) {
	    	returnTypeTextField.setText(javaComboBox.getSelectedItem().toString());
	    }
	  });
		
	}
	
	private void editItem(String editedParameter, int selectedIndex) {
		List<String> items = copyItems();
		exceptionsComboBox.removeAllItems();
		for(int i = 0; i < items.size(); i++)
			if(i != selectedIndex)
				exceptionsComboBox.addItem(items.get(i));
			else
				exceptionsComboBox.addItem(editedParameter);
	}
	
	
	private List<String> copyItems() {
		List<String> list = new ArrayList<String>();
		for (int i = 0; i < exceptionsComboBox.getItemCount(); i++) {
			list.add((String) exceptionsComboBox.getItemAt(i));
		}
		return list;
	}
	
	private boolean isJadeClassName(String className) {
		if(className.equals("Concept") ||
			 className.equals("AgentAction") ||
			 className.equals("Predicate") ||
			 className.equals("AID"))
			return true;
		return false;
	}
	
	private String getProtegePrefixName (String underscorePrefixName) {
		//Generate prefix:className from prefix_className
		String prefix = underscorePrefixName.split("_")[0];
		String otherTypeName = "";
		if(underscorePrefixName.split("_").length > 1) {
			for(int i = 1; i < underscorePrefixName.split("_").length; i++)
				if(i + 1 < underscorePrefixName.split("_").length)
				  otherTypeName +=  underscorePrefixName.split("_")[i] + "_";
				else
					otherTypeName +=  underscorePrefixName.split("_")[i];
		}
		String protegePrefixName; // Prefix:Name
		if(!otherTypeName.equals(""))
			protegePrefixName = prefix + ":" + otherTypeName;
		else
			protegePrefixName = prefix;
		
		return protegePrefixName;
	}
	
	private boolean isOntologyType(String type) {
		String[] items = new String[] {"Object", "String", "Byte", "Short", "Integer", "Long", "Float", "Double", 
        "Boolean", "Character", "List", "Set", "Map"};
		for(int i = 0; i < items.length; i++)
			if(items[i].equals(type))
				return false;
		
//		String prefix = type.split("_")[0];
//		String otherTypeName = "";
//		if(type.split("_").length > 1) {
//			for(int i = 1; i < type.split("_").length; i++)
//				otherTypeName +=  type.split("_")[i];
//		}
//		String ontologyType;
//		if(!otherTypeName.equals(""))
//		  ontologyType = prefix + ":" + otherTypeName;
//		else
//			ontologyType = prefix;
		
		String ontologyType = getProtegePrefixName(type);
		
		if(isJadeClassName(ontologyType))
			ontologyType = "Jade:" + ontologyType;
		
		for(Object obj : project.getKnowledgeBase().getClses()) {
//			if(((Cls) obj).getName().contains("#") && ((Cls) obj).getName().split("#")[1].equals(type))
//				return true;
			

			if(obj instanceof DefaultRDFSNamedClass) {
				if(ontologyType.equals(((DefaultRDFSNamedClass)obj).getPrefixedName()))
				  return true;
			}
		}
		return false;
	}

}
