  package nl.uva.psy.swi.beangenerator;
  
  import edu.stanford.smi.protege.model.Cls;
  import edu.stanford.smi.protege.model.KnowledgeBase;
  import edu.stanford.smi.protege.model.Slot;
  import edu.stanford.smi.protegex.owl.model.NamespaceManager;
  import edu.stanford.smi.protegex.owl.model.OWLClass;
  import edu.stanford.smi.protegex.owl.model.OWLModel;
  import edu.stanford.smi.protegex.owl.model.OWLProperty;
  import java.util.Collection;
  import java.util.Iterator;
  
  public class NameConverter
  {
    private Slot jadeNameSlot;
    private boolean useJADENames;
    
    public String getClassName(Cls theCls)
    {
      String className = (theCls instanceof OWLClass) ? ((OWLClass)theCls).getPrefixedName() : theCls.getName();
      if (!this.useJADENames) {
        return className;
      }
      if (this.jadeNameSlot != null)
      {
        Object temp = theCls.getOwnSlotValue(this.jadeNameSlot);
        if (temp != null) {
          className = (String)temp;
        }
      }
      return className;
    }
    
    public String getSlotName(Slot theSlot)
    {
      String slotName = (theSlot instanceof OWLProperty) ? ((OWLProperty)theSlot).getPrefixedName() : theSlot.getName();
      if (!this.useJADENames) {
        return slotName;
      }
      if (this.jadeNameSlot != null)
      {
        Object temp = theSlot.getOwnSlotValue(this.jadeNameSlot);
        if (temp != null) {
          slotName = (String)temp;
        }
      }
      return slotName;
    }
    
    public String toAddMethod(Slot theSlot)
    {
      String slotName = ProtegeTools.toJavaStyleString(getSlotName(theSlot));
      return "add" + ProtegeTools.firstUpper(slotName);
    }
    
    public String toClearAllMethod(Slot theSlot)
    {
      String slotName = ProtegeTools.toJavaStyleString(getSlotName(theSlot));
      return "clearAll" + ProtegeTools.firstUpper(slotName);
    }
    
    public String toDir(String packageName, String srcDir)
    {
    	//EDIT: change slash in backslash for windows
//      StringBuffer buff = new StringBuffer(srcDir);
//      buff.append("/");
//      buff.append(packageName.replace('.', '/'));
//      return buff.toString();
    	
    	StringBuffer buff = new StringBuffer(srcDir);
    	buff.append("\\");
    	buff.append(packageName.replace('.', '\\'));
    	return buff.toString();
    }
    
    public String toFullJavaClassName(String packageName, Cls theCls)
    {
      return packageName + "." + toJavaTypeName(theCls);
    }
    
    public String toGetAllMethod(Slot theSlot)
    {
      String slotName = ProtegeTools.toJavaStyleString(getSlotName(theSlot));
      return "getAll" + ProtegeTools.firstUpper(slotName);
    }
    
    public String toGetMethod(Slot theSlot)
    {
      String slotName = ProtegeTools.toJavaStyleString(getSlotName(theSlot));
      return "get" + ProtegeTools.firstUpper(slotName);
    }
    
    public String toJavaTypeName(Cls theCls)
    {
      String className = getClassName(theCls);
      return ProtegeTools.firstUpper(ProtegeTools.toJavaStyleString(className));
    }
    
    public String toJavaTypeName(String className)
    {
      String buff = ProtegeTools.firstUpper(ProtegeTools.toJavaStyleString(className));
      buff = buff.replace(' ', '_');
      buff = buff.replace('-', '_');
      buff = buff.replace(':', '_');
      return buff;
    }
    
    public String toOntologyClassName(String ontologyName)
    {
      StringBuffer buff = new StringBuffer(ontologyName);
      buff.append("Ontology");
      ProtegeTools.toJavaStyleString(buff);
      ProtegeTools.firstUpper(buff);
      return buff.toString();
    }
    
    public String toRemoveMethod(Slot theSlot)
    {
      String slotName = ProtegeTools.toJavaStyleString(getSlotName(theSlot));
      return "remove" + ProtegeTools.firstUpper(slotName);
    }
    
    public String toRoleConstant(Cls theCls, Slot theSlot)
    {
      String className = getClassName(theCls);
      
      StringBuffer buff = new StringBuffer(className);
      ProtegeTools.toJavaString(buff);
      if (theSlot != null)
      {
        String slotName = getSlotName(theSlot);
        buff.append("_");
        buff.append(ProtegeTools.toJavaString(slotName));
      }
      return buff.toString().toUpperCase();
    }
    
    public String toSchemaVariableName(Cls theCls, Slot theSlot)
    {
      String className = getClassName(theCls);
      
      StringBuffer buff = new StringBuffer(className);
      if (theSlot != null)
      {
        String slotName = getSlotName(theSlot);
        buff.append("_");
        buff.append(slotName);
      }
      buff.append("Schema");
      
      ProtegeTools.toJavaVariableStyleString(buff);
      
      return buff.toString();
    }
    
    public String toSetMethod(Slot theSlot)
    {
      String slotName = ProtegeTools.toJavaStyleString(getSlotName(theSlot));
      return "set" + ProtegeTools.firstUpper(slotName);
    }
    
    public NameConverter(KnowledgeBase itsKB, boolean useJADENames)
    {
      this.useJADENames = useJADENames;
      
      updateConstantsNames(itsKB);
      
      Cls jadeClass = itsKB.getCls(this.JADE_CLASS);
      if (jadeClass == null) {
        return;
      }
      Collection<?> jadeClassSlots = jadeClass.getDirectTemplateSlots();
      Iterator<?> iter = jadeClassSlots.iterator();
      while (iter.hasNext())
      {
        Slot slot = (Slot)iter.next();
        if (slot.getName().equals(this.JADE_NAME_SLOT))
        {
          this.jadeNameSlot = slot;
          break;
        }
      }
    }
    
    private void updateConstantsNames(KnowledgeBase kb)
    {
      if (!(kb instanceof OWLModel)) {
        return;
      }
      String prefix = ((OWLModel)kb).getNamespaceManager().getPrefix("http://jade.cselt.it/beangenerator#");
      if (prefix == null) {
        return;
      }
      this.JADE_CLASS = "http://jade.cselt.it/beangenerator#JADE-CLASS";
      this.JADE_NAME_SLOT = "http://jade.cselt.it/beangenerator#JADE-NAME";
    }
    
    public String toFactoryGetMethod(String javaClassName)
    {
      return "get" + javaClassName;
    }
    
    public String toFactoryGetMethod(Cls theCls)
    {
      return toFactoryGetMethod(toJavaTypeName(theCls));
    }
    
    private String JADE_CLASS = ":JADE-CLASS";
    private String JADE_NAME_SLOT = ":JADE-NAME";
  }



 