package nl.uva.psy.swi.beangenerator;

import defaults.PrimitiveDefaults;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Frame;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.model.ValueType;
import edu.stanford.smi.protege.util.CollectionUtilities;
import edu.stanford.smi.protege.util.Log;
import edu.stanford.smi.protege.widget.AbstractTabWidget;
import edu.stanford.smi.protegex.owl.model.NamespaceManager;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.OWLProperty;
import edu.stanford.smi.protegex.owl.model.RDFResource;
import edu.stanford.smi.protegex.owl.model.RDFSClass;
import edu.stanford.smi.protegex.owl.model.impl.AbstractRDFSClass;
import edu.stanford.smi.protegex.owl.model.impl.DefaultRDFSNamedClass;
import gui.JavaMethodMapperPanel;

import java.awt.Font;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import cmd.CMD;
import xml_parser.XMLParser;
import antlr4_parser.Method;
import antlr4_parser.MethodData;
import antlr4_parser.MethodParser;

//Ctrl + f for 'EDITED' to find the modified code areas 
public class OntologyBeanGeneratorTab extends AbstractTabWidget
{
	private static final long serialVersionUID = 1L;
	
	private boolean xmlFileExists = true;
	private boolean ontologyExceptionExists = false;
	private boolean exceptionCountMoreThanZero = false;
//	private String ONTOLOGY_EXCEPTION_PREFIX = "OntoScene_";
//	private String ONTOLOGY_EXCEPTION = "Exception";
	private String ONTOLOGY_EXCEPTION_PREFIX_NO_UNDERSCORE = "Exception";
	private String ONTOLOGY_EXCEPTION_PREFIX = "Exception_";
	private String ONTOLOGY_EXCEPTION = "Base";
	private String xmlFilePath = null;
	private HashMap<String, List<Method>> classNameToMethodsHashMap;
	private String protegeFolderPath;
	private String jadePath;
	
	public static void main(String[] args) {
		edu.stanford.smi.protege.Application.main(args);
	}
	
	public void setXMLPath(String xmlFilePath) {
		this.xmlFilePath = xmlFilePath;
	}

	private String getCardinality(Slot theSlot)
	{
		String optionality = null;
		String maxString = null;
		int min = theSlot.getMinimumCardinality();
		int max = theSlot.getMaximumCardinality();
		if (min == 0) {
			optionality = "ObjectSchema.OPTIONAL";
		} else {
			optionality = "ObjectSchema.MANDATORY";
		}
		if (max >= 1) {
			maxString = Integer.toString(max);
		} else {
			maxString = "ObjectSchema.UNLIMITED";
		}
		StringBuffer schemaParams = new StringBuffer();
		if (max == 1)
		{
			schemaParams.append(optionality);
		}
		else
		{
			schemaParams.append(min);
			schemaParams.append(", ");
			schemaParams.append(maxString);
		}
		return schemaParams.toString();
	}

	private String getDocumentation(Frame frame, String indent)
	{
		StringBuffer documentation = new StringBuffer();
		Iterator<?> iter = frame.getDocumentation().iterator();
		while (iter.hasNext())
		{
			documentation.append(indent + "* ");
			documentation.append(iter.next().toString());
			documentation.append("\n");
		}
		documentation.append(indent + "* Protege name: " + frame.getName());
		return documentation.toString();
	}

	private String getJavaSuperClass(Cls theCls)
	{
		String javaSuperClassName = null;
		if (this.javaSuperClassSlot != null)
		{
			Object temp = theCls.getOwnSlotValue(this.javaSuperClassSlot);
			if (temp != null) {
				javaSuperClassName = (String)temp;
			}
		}
		return javaSuperClassName;
	}

	private Cls getRealSuperClass(Cls theCls)
	{
		Collection<?> superClses = getDirectSuperclasses(theCls);

		Cls superClass = theCls;
		while (superClses.size() > 0)
		{
			superClass = (Cls)CollectionUtilities.getFirstItem(superClses);
			for (Iterator<?> iter = superClses.iterator(); iter.hasNext();)
			{
				superClass = (Cls)iter.next();
				if ((isJADEPrimitive(superClass)) || (!isIgnored(superClass))) {
					return superClass;
				}
			}
			superClses = getDirectSuperclasses(superClass);
		}
		return superClass;
	}

	private void getRealSuperInterfaces(Cls theCls, Collection<Cls> res)
	{
		Collection<?> superClses = getDirectSuperclasses(theCls);
		for (Iterator<?> iter = superClses.iterator(); iter.hasNext();)
		{
			Cls superClass = (Cls)iter.next();
			if (isJADEClass(superClass)) {
				if ((isJADEPrimitive(superClass)) || (!isIgnored(superClass))) {
					res.add(superClass);
				} else {
					getDirectSuperclasses(superClass);
				}
			}
		}
	}

	private Collection<?> getDirectSuperclasses(Cls theCls)
	{
		if ((theCls instanceof RDFSClass)) {
			return ((RDFSClass)theCls).getNamedSuperclasses();
		}
		return theCls.getDirectSuperclasses();
	}

	/*private String getSuperClass(Cls theCls)
    {
      StringBuffer result = new StringBuffer("");

      Cls superClass = getRealSuperClass(theCls);

      String superClassName = fixNamespacePrefix(this.converter.getClassName(superClass));

      superClassName = this.converter.toJavaTypeName(superClassName);
      if ((superClassName.equals(this.JAVA_CONCEPT_CLASS_NAME)) || 
        (superClassName.equals(this.JAVA_AGENTACTION_CLASS_NAME)) || 
        (superClassName.equals(this.JAVA_PREDICATE_CLASS_NAME)))
      {
        String javaBaseClass = getJavaSuperClass(theCls);
        if (javaBaseClass != null)
        {
          result.append(" extends ");
          result.append(javaBaseClass);
        }
        result.append(" implements ");
        result.append(superClassName);
      }
      else
      {
        result.append(" extends ");
        result.append(superClassName);
      }
      if (superClassName.charAt(0) == ':') {
        result = new StringBuffer("");
      }
      return result.toString();
    }*/

	private String getSuperClasses(Cls theCls)
	{
		StringBuffer result = new StringBuffer("");

		ArrayList<Cls> supClasses = new ArrayList();
		getRealSuperInterfaces(theCls, supClasses);

		boolean first = true;
		for (Iterator<?> iter = supClasses.iterator(); iter.hasNext();)
		{
			Cls superClass = (Cls)iter.next();
			String superClassName = fixNamespacePrefix(this.converter.getClassName(superClass));
			superClassName = this.converter.toJavaTypeName(superClassName);
			if (first)
			{
				result.append(" extends ");
				first = false;
			}
			else
			{
				result.append(", ");
			}
			if (isJADEPrimitive(superClass))
			{
				if (superClassName.equals(this.JAVA_AID_CLASS_NAME)) {
					result.append("jade.core." + superClassName);
				} else {
					result.append("jade.content." + superClassName);
				}
			}
			else {
				result.append(superClassName);
			}
		}
		return result.toString();
	}

	private String fixNamespacePrefix(String clsName)
	{
		if (!(getKnowledgeBase() instanceof OWLModel)) {
			return clsName;
		}
		String prefix = ((OWLModel)getKnowledgeBase()).getNamespaceManager().getPrefix("http://jade.cselt.it/beangenerator#");
		if (prefix == null) {
			return clsName;
		}
		if (clsName.startsWith(prefix))
		{
			String shortSuperclsName = clsName.substring(prefix.length() + 1);
			return shortSuperclsName;
		}
		return clsName;
	}

	private boolean isIgnored(Cls theCls)
	{
		Boolean ignored = new Boolean(false);
		if (this.jadeIgnoredSlot != null)
		{
			Object temp = theCls.getOwnSlotValue(this.jadeIgnoredSlot);
			if (temp != null) {
				ignored = (Boolean)temp;
			}
		}
		return ignored.booleanValue();
	}

	private boolean isJADEPrimitive(Cls theCls)
	{
		String className = theCls.getName();
		boolean primitive = false;
		if ((className.equals(this.AID)) || (className.equals(this.CONCEPT)) || 
				(className.equals(this.AGENTACTION)) || 
				(className.equals(this.PREDICATE))) {
			primitive = true;
		}
		if ((getKnowledgeBase() instanceof OWLModel)) {
			try
			{
				if (((RDFResource)theCls).getNamespace().equals("http://jade.cselt.it/beangenerator#")) {
					primitive = true;
				}
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		return primitive;
	}

	private boolean isJADEClass(Cls theCls)
	{
		if (isJADEPrimitive(theCls)) {
			return true;
		}
		Collection<?> allSuperclasses = theCls.getSuperclasses();

		return (allSuperclasses.contains(this.itsKB.getCls(this.CONCEPT))) || (allSuperclasses.contains(this.itsKB.getCls(this.PREDICATE)));
	}

	private void addClass(Cls theCls, String collectionName)
	{
		this.ontologyHeader.insert(0, "    public static final String " + 
				this.converter.toRoleConstant(theCls, null) + "=\"" + 
				this.converter.toJavaTypeName(this.converter.getClassName(theCls)) + "\";\n");
		if (collectionName.equals(this.JAVA_AID_CLASS_NAME)) {
			collectionName = this.JAVA_CONCEPT_CLASS_NAME;
		}
		this.ontologyHeader.append("    " + collectionName + "Schema ");
		this.ontologyHeader.append(this.converter.toSchemaVariableName(theCls, null));
		this.ontologyHeader.append(" = new ");
		this.ontologyHeader.append(collectionName);
		this.ontologyHeader.append("Schema(" + this.converter.toRoleConstant(theCls, null) + 
				");\n");
	}

	private void addInheritance(Cls theCls)
			throws Exception
	{
		ArrayList<Cls> supClasses = new ArrayList();
		getRealSuperInterfaces(theCls, supClasses);
		for (Cls superClass : supClasses) {
			if (!isJADEPrimitive(superClass))
			{
				String superClassSchema = this.converter.toSchemaVariableName(superClass, null);
				String classSchema = this.converter.toSchemaVariableName(theCls, null);


				this.ontologyInheritance.append("    " + classSchema + ".addSuperSchema(" + 
						superClassSchema + 
						");\n");
			}
		}
	}

	private void addJADEMapping(Cls theCls, Slot theSlot)
	{
		String fullClassName = this.converter.toFullJavaClassName(this.packageName, theCls);
		String slotName = this.converter.getSlotName(theSlot);

		slotName = "\"" + slotName + "\"";

		this.nameMapping.append("    storeSlotName(" + slotName + ", \"" + fullClassName + 
				"\", \"" + theSlot.getName() + "\");  \n");
	}

	private void addRole(Cls theCls, Slot theSlot)
	{
		try
		{
			addRoleNameConstant(theCls, theSlot);
			addRoleToSchema(theCls, theSlot);
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(this, 
					"Something went wrong with Slot: " + 
							theSlot.getName() + " of class: " + 
							theCls.getName(), "error", 
							0);
			return;
		}
	}

	private void addRoleNameConstant(Cls theCls, Slot theSlot)
			throws Exception
	{
		String slotName = this.converter.getSlotName(theSlot);
		slotName = ProtegeTools.toJavaString(slotName);
		if ((slotName.length() < 1) || (slotName.charAt(0) == ':')) {
			throw new Exception("Invalid slot name");
		}
		String roleName = this.converter.toRoleConstant(theCls, theSlot);

		this.ontologyHeader.insert(0, 
				"    public static final String " + roleName + "=\"" + 
						slotName + "\";\n");
	}

	private void addRoleToSchema(Cls theCls, Slot theSlot)
	{
		String roleName = this.converter.toRoleConstant(theCls, theSlot);
		String classSchemaVariableName = this.converter.toSchemaVariableName(theCls, null);
		String termSchemaString = null;
		String termString = null;
		ValueType valueType = theSlot.getValueType();

		String ontologyType = OntologyBeanGeneratorUtil.getOntologyType(valueType);
		if (valueType.toString().charAt(0) == ':') {
			return;
		}
		Collection<?> c = new ArrayList();
		if (theSlot.getValueType().equals(ValueType.INSTANCE)) {
			c = theCls.getTemplateSlotAllowedClses(theSlot);
		} else if (theSlot.getValueType().equals(ValueType.CLS)) {
			c = theCls.getTemplateSlotAllowedParents(theSlot);
		}
		if (c.isEmpty()) {
			termString = "(TermSchema)";
		}
		if (c.size() == 1)
		{
			Cls allowedCls = (Cls)c.iterator().next();
			ontologyType = this.converter.getClassName(allowedCls);

			KnowledgeBase kb = getKnowledgeBase();
			if (allowedCls.equals(kb.getCls(this.CONCEPT)))
			{
				ontologyType = "BasicOntology.STRING";
				termString = "(TermSchema)";
				termSchemaString = "new ConceptSchema(\"Concept\")";
			}
			else if (allowedCls.equals(kb.getCls(this.AGENTACTION)))
			{
				ontologyType = "BasicOntology.STRING";
				termString = "(TermSchema)";
				termSchemaString = "new AgentActionSchema(\"AgentAction\")";
			}
			else if (allowedCls.equals(kb.getCls(this.PREDICATE)))
			{
				ontologyType = "BasicOntology.STRING";
				termString = "(TermSchema)";
				termSchemaString = "new PredicateSchema(\"Predicate\")";
			}
			else if (allowedCls.equals(kb.getCls(this.AID)))
			{
				termString = "(ConceptSchema)";
				ontologyType = fixNamespacePrefix(ontologyType);
				ontologyType = "BasicOntology." + ontologyType.toUpperCase();
			}
			else
			{
				if ((allowedCls.isSystem()) || (allowedCls.getName().startsWith(":")))
				{
					System.out.println("Ignore ontology type: " + allowedCls + "... Returning");
					return;
				}
				termString = "(TermSchema)";

				ontologyType = this.converter.toSchemaVariableName(allowedCls, null);
				termSchemaString = ontologyType;
			}
		}
		if (c.size() > 1)
		{
			ontologyType = "BasicOntology.STRING";
			termString = "(TermSchema)";
			termSchemaString = "new ConceptSchema(\"Concept\")";
		}
		if ((ontologyType.length() > 0) && (ontologyType.toString().charAt(0) == ':'))
		{
			System.out.println("ingnore ontology type!, returning");
			return;
		}
		if (termSchemaString == null) {
			if (termString == null) {
				termSchemaString = ontologyType;
			} else {
				termSchemaString = termString + "getSchema(" + ontologyType + ")";
			}
		}
		if (((theSlot.getValueType().equals(ValueType.INSTANCE)) || (theSlot.getValueType().equals(ValueType.INSTANCE))) && (ontologyType.length() < 1))
		{
			ontologyType = "BasicOntology.STRING";
			termString = "(TermSchema)";
			termSchemaString = "new ConceptSchema(\"Concept\")";
		}
		String roleParams = getCardinality(theSlot);

		this.ontologyBody.append("    " + classSchemaVariableName);
		this.ontologyBody.append(".add(" + roleName + ", " + termSchemaString + ", " + roleParams + ");");
		this.ontologyBody.append("\n");
		if (this.jadeUnnamedSlot != null)
		{
			Object temp2 = theSlot.getOwnSlotValue(this.jadeUnnamedSlot);
			if (temp2 != null)
			{
				Boolean anonymous = (Boolean)temp2;
				if (anonymous.booleanValue()) {
					this.ontologyBody.append("\n   " + classSchemaVariableName + ".setEncodingByOrder(true);");
				}
			}
		}
	}
	
	//EDITED: write to exception files
	private PrintStream createException(Cls theCls, File f, String superClass)
	{
		String theName = this.converter.toJavaTypeName(this.converter.getClassName(theCls));
		try
		{
			FileOutputStream fos = new FileOutputStream(f);
			OutputStream bos = new BufferedOutputStream(fos);
			PrintStream ps = new PrintStream(bos);
			
			ps.println("package " + this.packageName + ".exceptions" + ";");
			ps.println();
			ps.println("import " + this.packageName + ".*;");
			ps.println();
			ps.println("/**");
			ps.println(getDocumentation(theCls, ""));
			ps.println("* @author " + OntologyBeanGeneratorUtil.getVersionInformation());
			ps.println("* @version " + OntologyBeanGeneratorUtil.getDate());
			ps.println("*/");
			
			ps.print("public class " + theName + superClass);
			ps.println(" { public " +  theName + "() {} }");
			
			return ps;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return null;
	}
	
	private PrintStream createBean(Cls theCls, File f, boolean isSuperBean)
	{
		String superClass = "";
		String theName = this.converter.toJavaTypeName(this.converter.getClassName(theCls));

		String javaBaseClass = getJavaSuperClass(theCls);
		if (javaBaseClass != null) {
			superClass = " extends " + javaBaseClass;
		}
		//EDIT: to avoid the Character problem (I added this.packageName + ".")

		superClass = superClass + " implements " + this.packageName + "." + theName;
		theName = "Default" + theName;
		try
		{
			FileOutputStream fos = new FileOutputStream(f);
			OutputStream bos = new BufferedOutputStream(fos);
			PrintStream ps = new PrintStream(bos);

			ps.println("package " + this.packageName + ".impl" + ";");
			ps.println();
			if (isSuperBean)
			{
				if (this.fullBeanSupport)
				{
					ps.println("import java.io.Serializable;");
					ps.println("import java.beans.PropertyChangeSupport;");
					ps.println("import java.beans.PropertyChangeListener;");
				}
				ps.println();
			}
			if (this.microSupport)
			{
				ps.println("import jade.content.abs.*;");
				ps.println("import jade.content.onto.*;");
			}
		
			ps.println("import " + this.packageName + ".*;");
			
			//EDITED: import exceptions if they exist
			if(ontologyExceptionExists)
				ps.println("import " + this.packageName + ".exceptions.*;");
			
			ps.println("import jade.content.Concept;");
			ps.println("import jade.content.AgentAction;");
			ps.println("import jade.content.Predicate;");
			ps.println("import jade.core.AID;");
			
			//EDITED: uncomment this and comment ps.println("import java.util.*;"); if you want to go back
			//ps.println("import jade.util.leap.*;");
		  ps.println("import java.util.*;");
			
//			ps.println("import jade.util.leap.List;");
//		  ps.println("import jade.util.leap.Set;");
//		  ps.println("import jade.util.leap.Map;");
			

			ps.println();
			ps.println("/**");
			ps.println(getDocumentation(theCls, ""));
			ps.println("* @author " + OntologyBeanGeneratorUtil.getVersionInformation());
			ps.println("* @version " + OntologyBeanGeneratorUtil.getDate());
			ps.println("*/");
			if (isSuperBean)
			{
				ps.print("public class " + theName + superClass);
				if (this.fullBeanSupport)
				{
					ps.println(", Serializable {");
					ps.println("   // bean stuff");
					ps.println(
							"   protected PropertyChangeSupport pcs = new PropertyChangeSupport(this);\n");
					ps.println(
							"   public void addPropertyChangeListener(PropertyChangeListener pcl) {");
					ps.println("     pcs.addPropertyChangeListener(pcl);\n   }\n");
					ps.println(
							"   public void removePropertyChangeListener(PropertyChangeListener pcl) {");
					ps.println("     pcs.removePropertyChangeListener(pcl);\n   }\n");
				}
				else if (this.microSupport)
				{
					ps.println(", Introspectable {");
				}
				else
				{
					ps.println(" {");
				}
			}
			else
			{
				ps.println("public class " + theName + superClass + "{ ");
			}
			ps.println();

			ps.println("  private static final long serialVersionUID = " + this.currentSerialVersionUID + "L;");
			ps.println();

			ps.println("  private String _internalInstanceName = null;");
			ps.println();
			ps.println("  public " + theName + "() {");
			ps.println("    this._internalInstanceName = \"\";");
			ps.println("  }");
			ps.println();
			ps.println("  public " + theName + "(String instance_name) {");
			ps.println("    this._internalInstanceName = instance_name;");
			ps.println("  }");
			ps.println();
			ps.println("  public String toString() {");
			ps.println("    return _internalInstanceName;");
			ps.println("  }");
			ps.println();
			if (this.javaJavaCodeSlot != null)
			{
				Object codeVal = theCls.getOwnSlotValue(this.javaJavaCodeSlot);
				if (codeVal != null)
				{
					ps.println("//////////////////////////// User code");
					ps.println((String)codeVal);
					ps.println();
				}
			}
			return ps;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return null;
	}

	private PrintStream createBeanInterface(Cls theCls, File f, String superClass, boolean isSuperBean)
	{
		String theName = this.converter.toJavaTypeName(this.converter.getClassName(theCls));
		try
		{
			FileOutputStream fos = new FileOutputStream(f);
			OutputStream bos = new BufferedOutputStream(fos);
			PrintStream ps = new PrintStream(bos);

			ps.println("package " + this.packageName + ";");
			
		  //EDITED: import exceptions if they exist
			if(ontologyExceptionExists)
				ps.println("import " + this.packageName + ".exceptions.*;");
			
			ps.println("import jade.content.Concept;");
			ps.println("import jade.content.AgentAction;");
			ps.println("import jade.content.Predicate;");
			//ps.println("import jade.core.AID;");
			
//			ps.println("import jade.util.leap.List;");
//			ps.println("import jade.util.leap.Set;");
//			ps.println("import jade.util.leap.Map;");
			//ps.println("import java.util.*;");
		  //EDITED: uncomment this and comment ps.println("import java.util.*;"); if you want to go back
			//ps.println("import jade.util.leap.*;");
			ps.println("import java.util.*;");

			ps.println();
			if (isSuperBean)
			{
				if (this.fullBeanSupport)
				{
					ps.println("import java.io.Serializable;");
					ps.println("import java.beans.PropertyChangeSupport;");
					ps.println("import java.beans.PropertyChangeListener;");
				}
				ps.println();
			}
			if (this.microSupport)
			{
				ps.println("import jade.content.abs.*;");
				ps.println("import jade.content.onto.*;");
			}
		  
			ps.println();
			ps.println("/**");
			ps.println(getDocumentation(theCls, ""));
			ps.println("* @author " + OntologyBeanGeneratorUtil.getVersionInformation());
			ps.println("* @version " + OntologyBeanGeneratorUtil.getDate());
			ps.println("*/");
			if (isSuperBean)
			{
				ps.print("public interface " + theName + superClass);
				if (this.fullBeanSupport)
				{
					ps.println(", Serializable {");
					ps.println("   // bean stuff");
					ps.println(


							"   public void addPropertyChangeListener(PropertyChangeListener pcl);");


					ps.println(
							"   public void removePropertyChangeListener(PropertyChangeListener pcl);");
				}
				else if (this.microSupport)
				{
					ps.println(", Introspectable {");
				}
				else
				{
					ps.println(" {");
				}
			}
			ps.println();
			return ps;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		return null;
	}

	//EDITED: to change the ontology file generation  
//	 private PrintStream createOntologyFile()
//   {
//     try
//     {
//       StringBuffer fileName = new StringBuffer(this.converter
//         .toDir(this.packageName, this.dirName));
//       fileName.append("/");
//       fileName.append(this.converter.toOntologyClassName(this.ontologyName));
//       fileName.append(".java");
//       
//       File f = new File(fileName.toString());
//       this.ontologyBeanGeneratorPanel.statusTextArea.append(
//         "created ontology file: " + 
//         fileName.toString() + 
//         "\n");
//       
//       FileOutputStream fos = new FileOutputStream(f);
//       OutputStream bos = new BufferedOutputStream(fos);
//       PrintStream ps = new PrintStream(bos);
//       ps.println("// file: " + f.getName() + 
//         " generated by ontology bean generator.  DO NOT EDIT, UNLESS YOU ARE REALLY SURE WHAT YOU ARE DOING!");
//       
// 
//       ps.println("package " + this.packageName + ";");
//       ps.println();
//       
// 
//       ps.println("import jade.content.onto.*;");
//       ps.println("import jade.content.schema.*;");
//       if ((!this.microSupport) && (this.useJADENames))
//       {
//         ps.println("import jade.util.leap.HashMap;");
//         ps.println("import nl.uva.psy.swi.beangenerator.ProtegeIntrospector;");
//         ps.println("import nl.uva.psy.swi.beangenerator.SlotHolder;");
//         ps.println("import nl.uva.psy.swi.beangenerator.ProtegeTools;");
//       }
//       ps.println();
//       ps.println("/** file: " + f.getName());
//       ps.println(" * @author " + OntologyBeanGeneratorUtil.getVersionInformation());
//       ps.println(" * @version " + OntologyBeanGeneratorUtil.getDate());
//       ps.println(" */");
//       
//       ps.print("public class " + this.converter.toOntologyClassName(this.ontologyName) + 
//         " extends jade.content.onto.Ontology ");
//       
//       
//       if ((!this.microSupport) && (this.useJADENames))
//       {
//         ps.println("implements ProtegeTools.ProtegeOntology {");
//         ps.println("   /**");
//         ps.println("    * These hashmap store a mapping from jade names to either protege names of SlotHolder ");
//         ps.println("    * containing the protege names.  And vice versa");
//         ps.println("    */  ");
//         ps.println("   private HashMap jadeToProtege;");
//       }
//       else
//       {
//         ps.println(" {");
//       }
//       ps.println();
//       ps.println("  private static final long serialVersionUID = " + this.currentSerialVersionUID + "L;");
//       ps.println();
//       
//       ps.println("  //NAME");
//       ps.println("  public static final String ONTOLOGY_NAME = \"" + 
//         this.ontologyName + "\";");
//       if (!this.microSupport) {
//         if (this.useJADENames) {
//           ps.println(
//             "  private static ProtegeIntrospector introspect = new ProtegeIntrospector();");
//         }
//       }
//       ps.println("  // The singleton instance of this ontology");
//       ps.println("  private static Ontology theInstance = new " + 
//         this.converter.toOntologyClassName(this.ontologyName) + "();");
//       ps.println("  public static Ontology getInstance() {");
//       ps.println("     return theInstance;");
//       ps.println("  }");
//       ps.println();
//       if ((!this.microSupport) && (this.useJADENames))
//       {
//         ps.println("   // ProtegeOntology methods");
//         
//         ps.println(
//           "   public SlotHolder getSlotNameFromJADEName(SlotHolder jadeSlot) {");
//         ps.println("     return (SlotHolder) jadeToProtege.get(jadeSlot);");
//         ps.println("   }");
//         ps.println();
//         
//         ps.println();
//         ps.println("   // storing the information");
//         ps.println("   private void storeSlotName(String jadeName, String javaClassName, String slotName){");
//         
//         ps.println("       jadeToProtege.put(new SlotHolder(javaClassName, jadeName), new SlotHolder(javaClassName, slotName));");
//         ps.println("   }");
//         ps.println();
//       }
//       ps.println();
//       
//       ps.println("   // VOCABULARY");
//       this.ontologyHeader.append("\n  /**\n   * Constructor\n  */");
//       this.ontologyHeader.append("\n  private " + 
//         this.converter.toOntologyClassName(this.ontologyName) + 
//         "(){ \n");
//       
// 
//       this.ontologyHeader.append(
//         "    super(ONTOLOGY_NAME, BasicOntology.getInstance());\n");
//       if ((!this.microSupport) && (this.useJADENames))
//       {
//         this.ontologyHeader.append("    introspect.setOntology(this);\n");
//         this.ontologyHeader.append("    jadeToProtege = new HashMap();\n");
//       }
//       this.ontologyHeader.append("    try { \n");
//       
//       return ps;
//     }
//     catch (Exception e)
//     {
//       JOptionPane.showMessageDialog(null, e.getMessage(), "error", 
//         0);
//       this.ontologyBeanGeneratorPanel.statusTextArea.append(e.getMessage() + "\n");
//     }
//     return null;
//   }
	
	// DANY
	// recursive dir delete
	private void recursiveDelete(File file){
		if(!file.exists()) return;
		
		if(file.isDirectory()){
			for(File f: file.listFiles()) recursiveDelete(f);
		}
		
		file.delete();
	}
	
	
	private PrintStream createOntologyFile()
	{
		try
		{
			StringBuffer fileName = new StringBuffer(this.converter
					.toDir(this.packageName, this.dirName));
			fileName.append("/");
			
			//fileName.append(this.converter.toOntologyClassName(this.ontologyName));
			fileName.append(this.converter.toOntologyClassName(this.ontologyName));
			fileName.append(".java");

			File f = new File(fileName.toString());
			this.ontologyBeanGeneratorPanel.statusTextArea.append(
					"created ontology file: " + 
							fileName.toString() + 
					"\n");

			FileOutputStream fos = new FileOutputStream(f);
			OutputStream bos = new BufferedOutputStream(fos);
			PrintStream ps = new PrintStream(bos);
			/*ps.println("// file: " + f.getName() + 
          " generated by ontology bean generator.  DO NOT EDIT, UNLESS YOU ARE REALLY SURE WHAT YOU ARE DOING!");*/

			ps.println("package " + this.packageName + ";");
			ps.println();
			ps.println("import jade.content.onto.*;");
			ps.println("import jade.content.schema.*;");

			if ((!this.microSupport) && (this.useJADENames))
			{
				ps.println("import jade.util.leap.HashMap;");
				ps.println("import nl.uva.psy.swi.beangenerator.ProtegeIntrospector;");
				ps.println("import nl.uva.psy.swi.beangenerator.SlotHolder;");
				ps.println("import nl.uva.psy.swi.beangenerator.ProtegeTools;");
			}
			ps.println();
			ps.println("/** file: " + f.getName());
			ps.println(" * @author " + OntologyBeanGeneratorUtil.getVersionInformation());
			ps.println(" * @version " + OntologyBeanGeneratorUtil.getDate());
			ps.println(" */");

			/*ps.print("public class " + this.converter.toOntologyClassName(this.ontologyName) + 
          " extends jade.content.onto.Ontology ");*/
			
			
			ps.print("public class " + this.converter.toOntologyClassName(this.ontologyName) + 
					" extends jade.content.onto.BeanOntology ");

			if ((!this.microSupport) && (this.useJADENames))
			{
				ps.println("implements ProtegeTools.ProtegeOntology {");
				ps.println("   /**");
				ps.println("    * These hashmap store a mapping from jade names to either protege names of SlotHolder ");
				ps.println("    * containing the protege names.  And vice versa");
				ps.println("    */  ");
				ps.println("   private HashMap jadeToProtege;");
			}
			else
			{
				ps.println(" {");
			}
			ps.println();
			ps.println("  private static final long serialVersionUID = " + this.currentSerialVersionUID + "L;");
			ps.println();

			//ps.println("  //NAME");
			ps.println("  public static final String ONTOLOGY_NAME = \"" + 
					this.ontologyName + "\";");

			if (!this.microSupport) {
				if (this.useJADENames) {
					ps.println(
							"  private static ProtegeIntrospector introspect = new ProtegeIntrospector();");
				}
			}
			ps.println();
			//ps.println("  // The singleton instance of this ontology");
			/*ps.println("  private static Ontology theInstance = new " + 
          this.converter.toOntologyClassName(this.ontologyName) + "();");*/
			ps.println("  private static BeanOntology theInstance = new " + 
					this.converter.toOntologyClassName(this.ontologyName) + "();");
			//ps.println("  public static Ontology getInstance() {");
			ps.println();
			ps.println("  public static BeanOntology getInstance() {");
			ps.println("     return theInstance;");
			ps.println("  }");
			//ps.println();
			if ((!this.microSupport) && (this.useJADENames))
			{
				ps.println("   // ProtegeOntology methods");

				ps.println(
						"   public SlotHolder getSlotNameFromJADEName(SlotHolder jadeSlot) {");
				ps.println("     return (SlotHolder) jadeToProtege.get(jadeSlot);");
				ps.println("   }");
				ps.println();
				ps.println();
				ps.println("   // storing the information");
				ps.println("   private void storeSlotName(String jadeName, String javaClassName, String slotName){");
				ps.println("       jadeToProtege.put(new SlotHolder(javaClassName, jadeName), new SlotHolder(javaClassName, slotName));");
				ps.println("   }");
				ps.println();
			}
			//ps.println();

			//ps.println("   // VOCABULARY");
			this.ontologyHeader.append("\n  /**\n   * Constructor\n  */");
			this.ontologyHeader.append("\n  private " + 
					this.converter.toOntologyClassName(this.ontologyName) + 
					"(){ \n");
			this.ontologyHeader.append(
					"    super(ONTOLOGY_NAME, BasicOntology.getInstance());\n");

			if ((!this.microSupport) && (this.useJADENames))
			{
				this.ontologyHeader.append("    introspect.setOntology(this);\n");
				this.ontologyHeader.append("    jadeToProtege = new HashMap();\n");
			}
			this.ontologyHeader.append("    try { \n");

			return ps;
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(null, e.getMessage(), "error", 
					0);
			this.ontologyBeanGeneratorPanel.statusTextArea.append(e.getMessage() + "\n");
		}
		return null;
	}

	private PrintStream createFactoryFile()
	{
		try
		{
			StringBuffer fileName = new StringBuffer(this.converter
					.toDir(this.packageName, this.dirName));
			fileName.append("/");
			fileName.append(this.factoryClassName);
			fileName.append(".java");

			File f = new File(fileName.toString());
			this.ontologyBeanGeneratorPanel.statusTextArea.append(
					"created factory file: " + 
							fileName.toString() + 
					"\n");

			FileOutputStream fos = new FileOutputStream(f);
			OutputStream bos = new BufferedOutputStream(fos);
			PrintStream ps = new PrintStream(bos);
			ps.println("// file: " + f.getName() + 
					" generated by ontology bean generator.  DO NOT EDIT, UNLESS YOU ARE REALLY SURE WHAT YOU ARE DOING!");


			ps.println("package " + this.packageName + ";");
			ps.println();

			ps.println("import " + this.packageName + ".impl.*;");
			ps.println();
			ps.println("import java.lang.reflect.InvocationTargetException;");
			ps.println("import java.lang.reflect.Method;");
			ps.println();

			ps.println("import jade.util.leap.ArrayList;");

			ps.println();
			if ((this.itsKB instanceof OWLModel))
			{
				ps.println("import edu.stanford.smi.protegex.owl.model.OWLModel;");
				ps.println("import edu.stanford.smi.protegex.owl.model.OWLIndividual;");
				ps.println("import edu.stanford.smi.protegex.owl.model.RDFProperty;");
			}
			else
			{
				ps.println("import edu.stanford.smi.protege.model.KnowledgeBase;");
				ps.println("import edu.stanford.smi.protege.model.Instance;");
				ps.println("import edu.stanford.smi.protege.model.Slot;");
			}
			ps.println();
			ps.println("/** file: " + f.getName());
			ps.println(" * @author " + OntologyBeanGeneratorUtil.getVersionInformation());
			ps.println(" * @version " + OntologyBeanGeneratorUtil.getDate());
			ps.println(" */");


			ps.print("public class " + this.factoryClassName + " {");
			ps.println();
			ps.println("  private static final long serialVersionUID = " + this.currentSerialVersionUID + "L;");
			ps.println();
			if ((this.itsKB instanceof OWLModel)) {
				ps.println("  private OWLModel owlModel = null;");
			} else {
				ps.println("  private KnowledgeBase kb = null;");
			}
			ps.println("  private java.util.HashMap<String, Object> beanInstances = null;");
			ps.println("  public static java.util.HashMap<String, String> typeToMethodName = new java.util.HashMap<String, String> ();");
			ps.println();
			ps.println("  private Object tempAnyValue;");
			ps.println("  private Boolean tempBooleanValue;");
			ps.println("  private Class tempClsValue;");
			ps.println("  private Float tempFloatValue;");
			if ((this.itsKB instanceof OWLModel)) {
				ps.println("  private OWLIndividual tempInstanceValue;");
			} else {
				ps.println("  private Instance tempInstanceValue;");
			}
			ps.println("  private Integer tempIntegerValue;");
			ps.println("  private String tempStringValue;");
			ps.println("  private String tempSymbolValue;");
			ps.println();

			ps.println("  private java.util.ArrayList  tempAnyValues;");
			ps.println("  private java.util.ArrayList <Boolean> tempBooleanValues;");
			ps.println("  private java.util.ArrayList <Class> tempClsValues;");
			ps.println("  private java.util.ArrayList <Float> tempFloatValues;");
			if ((this.itsKB instanceof OWLModel)) {
				ps.println("  private java.util.ArrayList <OWLIndividual> tempInstanceValues;");
			} else {
				ps.println("  private java.util.ArrayList <Instance> tempInstanceValues;");
			}
			ps.println("  private java.util.ArrayList <Integer> tempIntegerValues;");
			ps.println("  private java.util.ArrayList <String> tempStringValues;");
			ps.println("  private java.util.ArrayList <String> tempSymbolValues;");
			ps.println();
			ps.println();
			ps.println("  //Constructor(s)");
			if ((this.itsKB instanceof OWLModel))
			{
				ps.println("  public " + this.factoryClassName + "(OWLModel owlModel) {");
				ps.println("    this.owlModel = owlModel;");
			}
			else
			{
				ps.println("  public " + this.factoryClassName + "(KnowledgeBase kb) {");
				ps.println("    this.kb = kb;");
			}
			ps.println("    beanInstances = new java.util.HashMap<String, Object>();");
			ps.println("  }");
			ps.println();

			return ps;
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(null, e.getMessage(), "error", 
					0);
			this.ontologyBeanGeneratorPanel.statusTextArea.append(e.getMessage() + "\n");
		}
		return null;
	}

	private void createMethodNameMapper()
	{
		this.factoryGlobal.append("\n");
		if ((this.itsKB instanceof OWLModel))
		{
			this.factoryGlobal.append("  public Object getJadeInstanceFor(OWLIndividual tempInstanceValue) throws " + this.factoryExceptionClassName + " {\n");
			this.factoryGlobal.append("    //TODO test for multiple types\n");
			this.factoryGlobal.append("    String methodName = typeToMethodName.get(tempInstanceValue.getRDFType().getName());\n");
		}
		else
		{
			this.factoryGlobal.append("  public Object getJadeInstanceFor(Instance tempInstanceValue) throws " + this.factoryExceptionClassName + " {\n");
			this.factoryGlobal.append("    //TODO test for multiple types\n");
			this.factoryGlobal.append("    String methodName = typeToMethodName.get(tempInstanceValue.getDirectType().getName());\n");
		}
		this.factoryGlobal.append("    if (methodName != null) {\n");

		this.factoryGlobal.append("      try {\n");
		this.factoryGlobal.append("        Method method = this.getClass().getMethod(methodName, new Class[] {Class.forName(\"java.lang.String\")});\n");
		if ((this.itsKB instanceof OWLModel)) {
			this.factoryGlobal.append("        return method.invoke(this, new Object[] {tempInstanceValue.getPrefixedName()});\n");
		} else {
			this.factoryGlobal.append("        return method.invoke(this, new Object[] {tempInstanceValue.getName()});\n");
		}
		this.factoryGlobal.append("      }\n");
		this.factoryGlobal.append("      catch (SecurityException e) {\n");
		this.factoryGlobal.append("         throw new " + this.factoryExceptionClassName + "(e);\n");
		this.factoryGlobal.append("      } catch (NoSuchMethodException e) {\n");
		this.factoryGlobal.append("         throw new " + this.factoryExceptionClassName + "(e);\n");
		this.factoryGlobal.append("      } catch (ClassNotFoundException e) {\n");
		this.factoryGlobal.append("         throw new " + this.factoryExceptionClassName + "(e);\n");
		this.factoryGlobal.append("      }\n");
		this.factoryGlobal.append("      catch (IllegalArgumentException e) {\n");
		this.factoryGlobal.append("         throw new " + this.factoryExceptionClassName + "(\"Exception while invoking method \" + methodName + \" for instance \" + tempInstanceValue.getName(), e);\n");
		this.factoryGlobal.append("      } catch (IllegalAccessException e) {\n");
		this.factoryGlobal.append("         throw new " + this.factoryExceptionClassName + "(\"Exception while invoking method \" + methodName + \" for instance \" + tempInstanceValue.getName(), e);\n");
		this.factoryGlobal.append("      } catch (InvocationTargetException e) {\n");
		this.factoryGlobal.append("         throw new " + this.factoryExceptionClassName + "(\"Exception while invoking method \" + methodName + \" for instance \" + tempInstanceValue.getName(), e);\n");
		this.factoryGlobal.append("      }\n");
		this.factoryGlobal.append("    }\n");
		this.factoryGlobal.append("    return null;\n");
		this.factoryGlobal.append("  }\n");
		this.factoryGlobal.append("\n");
		this.factoryGlobal.append("  static {\n");
	}

	private static void writeln(StringWriter sw)
	{
		writeln(sw, "");
	}

	private static void writeln(StringWriter sw, String s)
	{
		sw.write(s);
		sw.write("\n");
	}

	private static void write(StringBuffer sb, String s)
	{
		sb.append(s);
	}

	private static void writeln(StringBuffer sb)
	{
		writeln(sb, "");
	}

	private static void writeln(StringBuffer sb, String s)
	{
		sb.append(s);
		sb.append("\n");
	}

	private void createFactoryExceptionFile()
	{
		try
		{
			StringBuffer fileName = new StringBuffer(this.converter
					.toDir(this.packageName, this.dirName));
			fileName.append("/");
			fileName.append(this.factoryExceptionClassName);
			fileName.append(".java");

			File f = new File(fileName.toString());
			this.ontologyBeanGeneratorPanel.statusTextArea.append(
					"created factory exception file: " + 
							fileName.toString() + 
					"\n");

			FileOutputStream fos = new FileOutputStream(f);
			OutputStream bos = new BufferedOutputStream(fos);
			PrintStream ps = new PrintStream(bos);
			ps.println("// file: " + f.getName() + 
					" generated by ontology bean generator.  DO NOT EDIT, UNLESS YOU ARE REALLY SURE WHAT YOU ARE DOING!");


			ps.println("package " + this.packageName + ";");
			ps.println();
			ps.println();
			ps.println("/** file: " + f.getName());
			ps.println(" * @author " + OntologyBeanGeneratorUtil.getVersionInformation());
			ps.println(" * @version " + OntologyBeanGeneratorUtil.getDate());
			ps.println(" */");

			ps.println("public class " + this.factoryExceptionClassName + " extends Exception { ");
			ps.println();
			ps.println("  private static final long serialVersionUID = " + this.currentSerialVersionUID + "L;");
			ps.println();
			ps.println("  public " + this.factoryExceptionClassName + "(String msg) {");
			ps.println("    super(msg);");
			ps.println("  }");
			ps.println();
			ps.println("  public " + this.factoryExceptionClassName + "(String msg, Throwable thr) {");
			ps.println("    super(msg, thr);");
			ps.println("  }");
			ps.println();
			ps.println("  public " + this.factoryExceptionClassName + "(Throwable thr) {");
			ps.println("    super(thr);");
			ps.println("  }");
			ps.println();
			ps.println("}");
			ps.println();


			ps.close();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(null, e.getMessage(), "error", 
					0);
			this.ontologyBeanGeneratorPanel.statusTextArea.append(e.getMessage() + "\n");
		}
	}

	private void createSlot(PrintStream ps, PrintStream ps2, PrintStream psFact, Cls theCls, Slot theSlot, StringBuffer extern, StringBuffer intern, boolean isDirectSlot)
	{
		boolean multipleSetters = false;
		String theSlotName = this.converter.getSlotName(theSlot);
		String theVarName = ProtegeTools.toJavaVariableStyleString(this.converter.getSlotName(theSlot));
		String absSlotType = null;
		if (theSlot.isSystem())
		{
			this.ontologyBeanGeneratorPanel.statusTextArea.append("     skipping " + 
					theVarName + "\n");
			return;
		}
		if (((this.itsKB instanceof OWLModel)) && (
				(theSlotName.startsWith("swrl")) || (
						((theSlot instanceof OWLProperty)) && 
						(((OWLProperty)theSlot).isAnnotationProperty())))) {
			return;
		}
		String theType = "String";
		ValueType valueType = theSlot.getValueType();
		theType = OntologyBeanGeneratorUtil.getJavaSlotType(valueType);

		Cls theValueCls = null;
		if ((valueType.equals(ValueType.CLS)) || (valueType.equals(ValueType.INSTANCE)))
		{
			theValueCls = getInstanceValueCls(theCls, theSlot);
			theType = getInstanceValueType(theValueCls);
			if (theType == null) {
				return;
			}
		}
		if (hasCardinalitySingle(theSlot))
		{
			setUsedPropertyType(valueType, TypeCardinality.Single);
			if ((theType.equalsIgnoreCase("boolean")) || 
					(theType.equalsIgnoreCase("int")) || 
					(theType.equalsIgnoreCase("float")))
			{
				theType = theType.toLowerCase();
				absSlotType = theVarName;
			}
			if (isDirectSlot)
			{
				ps.println("   /**");
				ps.println(getDocumentation(theSlot, "   "));
				ps.println("   */");
			}
			ps2.println("   /**");
			ps2.println(getDocumentation(theSlot, "   "));
			ps2.println("   */");


			ps2.println("   private " + theType + " " + theVarName + ";");
			this.ontologyBeanGeneratorPanel.statusTextArea.append("     " + theType + " " + 
					theVarName + "\n");
			if (!multipleSetters)
			{
				if (isDirectSlot) {
					ps.println("   public void " + this.converter.toSetMethod(theSlot) + "(" + 
							theType + " value);");
				}
				ps2.println("   public void " + this.converter.toSetMethod(theSlot) + "(" + 
						theType + " value) { ");
				if (this.fullBeanSupport)
				{
					ps2.print("     pcs.firePropertyChange(\"" + theVarName + "\", ");
					if ((theType.equals("int")) || 
							(theType.equals("boolean")))
					{
						ps2.print("this." + theVarName);
						ps2.println(", value);");
					}
					else if (theType.equals("float"))
					{
						ps2.print("\"\"+this." + theVarName);
						ps2.println(", \"\"+value);");
					}
					else
					{
						ps2.print("(this." + theVarName + "==null?new " + theType + 
								"():this." + theVarName + ")");
						ps2.println(", value);");
					}
				}
				ps2.println("    this." + theVarName + "=value;\n" + "   }");
			}
			if (isDirectSlot)
			{
				ps.println("   public " + theType + " " + this.converter.toGetMethod(theSlot) + "();");
				ps.println();
			}
			ps2.println("   public " + theType + " " + this.converter.toGetMethod(theSlot) + "() {");
			ps2.println("     return this." + theVarName + ";\n" + "   }");
			ps2.println();
			if (this.factoryGeneration)
			{
				String tempVarName = "temp" + valueType + "Value";
				psFact.println("    " + tempVarName + " = " + 
						getTypedAccessorName(valueType, TypeCardinality.Single) + 
						"( protegeInst, " + (
								(this.itsKB instanceof OWLModel) ? "owlModel.getRDFProperty" : "kb.getSlot") + 
								"( \"" + theSlot.getName() + "\" ) );");
				psFact.println("    if (" + tempVarName + " != null) {");
				if (valueType == ValueType.INSTANCE)
				{
					if (!theType.equals("Object")) {
						if ((this.itsKB instanceof OWLModel)) {
							psFact.println("        if (tempInstanceValue.hasRDFType(owlModel.getOWLNamedClass(\"" + theValueCls.getName() + "\"), true )) {");
						} else {
							psFact.println("        if (tempInstanceValue.hasType(kb.getCls(\"" + theValueCls.getName() + "\") )) {");
						}
					}
					psFact.println("          try {");
					psFact.println("            Object referredBean = getJadeInstanceFor(tempInstanceValue);");
					psFact.println();
					psFact.println("            if (referredBean != null) {");


					psFact.println("              javaInst." + this.converter.toSetMethod(theSlot) + "(" + 
							" (" + theType + ") referredBean);");
					psFact.println("            }");
					psFact.println("          } catch (" + this.factoryExceptionClassName + " e) {");
					psFact.println("            e.printStackTrace();");
					psFact.println("          }");
					if (!theType.equals("Object"))
					{
						psFact.println("        }");
						psFact.println("        else {");
						psFact.println("          throw new " + this.factoryExceptionClassName + 
								"(\"Invalid type for instance \" + tempInstanceValue + \" referred by slot \\\"" + theSlot.getName() + "\\\"" + 
								" in instance \" + instanceName);");
						psFact.println("        }");
					}
				}
				else
				{
					psFact.println("      javaInst." + this.converter.toSetMethod(theSlot) + "(" + tempVarName + ");");
				}
				psFact.println("    }");
				psFact.println();
			}
		}
		else
		{
			setUsedPropertyType(valueType, TypeCardinality.Multiple);

			this.hasSlotsWithMultipleCardinality = true;
			if (isDirectSlot) {
				this.hasDirectSlotsWithMultipleCardinality = true;
			}
			String propertyStr = "";
			if (this.fullBeanSupport) {
				propertyStr = 
						"     pcs.firePropertyChange(\"" + theVarName + "\", oldList, this." + theVarName + ");";
			}
			this.ontologyBeanGeneratorPanel.statusTextArea.append("Collection " + 
					theVarName + "\n");
			if (isDirectSlot)
			{
				ps.println("   /**");
				ps.println(getDocumentation(theSlot, "   "));
				ps.println("   */");
			}
			ps2.println("   /**");
			ps2.println(getDocumentation(theSlot, "   "));
			ps2.println("   */");

			ps2.println("   private List " + theVarName + " = new ArrayList();");
			if (isDirectSlot) {
				ps.println("   public void " + this.converter.toAddMethod(theSlot) + "(" + 
						theType + " elem);");
			}
			ps2.println("   public void " + this.converter.toAddMethod(theSlot) + "(" + 
					theType + " elem) { ");

			ps2.println("     " + theVarName + ".add(elem);");
			if (propertyStr != "") {
				ps2.println(propertyStr);
			}
			ps2.println("   }");
			if (isDirectSlot) {
				ps.println("   public boolean " + this.converter.toRemoveMethod(theSlot) + "(" + 
						theType + " elem);");
			}
			ps2.println("   public boolean " + this.converter.toRemoveMethod(theSlot) + "(" + 
					theType + " elem) {");

			ps2.println("     boolean result = " + theVarName + ".remove(elem);");
			if (propertyStr != "") {
				ps2.println(propertyStr);
			}
			ps2.println("     return result;");
			ps2.println("   }");
			if (isDirectSlot) {
				ps.println("   public void " + this.converter.toClearAllMethod(theSlot) + "();");
			}
			ps2.println("   public void " + this.converter.toClearAllMethod(theSlot) + 
					"() {");

			ps2.println("     " + theVarName + ".clear();");
			if (propertyStr != "") {
				ps2.println(propertyStr);
			}
			ps2.println("   }");
			if (isDirectSlot)
			{
				ps.println("   public Iterator " + this.converter.toGetAllMethod(theSlot) + "();");
				ps.println("   public List " + this.converter.toGetMethod(theSlot) + "();");
				ps.println("   public void " + this.converter.toSetMethod(theSlot) + "(List l);");
				ps.println();
			}
			ps2.println("   public Iterator " + this.converter.toGetAllMethod(theSlot) + 
					"() {return " + theVarName + ".iterator(); }");
			ps2.println("   public List " + this.converter.toGetMethod(theSlot) + 
					"() {return " + theVarName + "; }");
			ps2.println("   public void " + this.converter.toSetMethod(theSlot) + 
					"(List l) {" + theVarName + " = l; }");
			ps2.println();
			theType = "List";
			if (this.factoryGeneration)
			{
				String tempVarName = "temp" + valueType + "Values";
				psFact.println("    " + tempVarName + " = " + 
						getTypedAccessorName(valueType, TypeCardinality.Multiple) + 
						"( protegeInst, " + (
								(this.itsKB instanceof OWLModel) ? "owlModel.getRDFProperty" : "kb.getSlot") + 
								"( \"" + theSlot.getName() + "\" ) );");
				psFact.println("    if (" + tempVarName + " != null) {");
				if (valueType == ValueType.INSTANCE)
				{
					Cls theRefValueCls = null;
					String theRefType = null;

					theRefValueCls = getInstanceValueCls(theCls, theSlot);
					theRefType = getInstanceValueType(theRefValueCls);
					if (theRefType == null)
					{
						psFact.println("    }");
						return;
				}
				if ((this.itsKB instanceof OWLModel))
				{
					psFact.println("      for (OWLIndividual instValue : tempInstanceValues) {");
					psFact.println("        tempInstanceValue = instValue;");
					if (!theRefType.equals("Object")) {
						psFact.println("        if (tempInstanceValue.hasRDFType(owlModel.getOWLNamedClass(\"" + theRefValueCls.getName() + "\"), true )) {");
					}
				}
				else
				{
					psFact.println("      for (Instance instValue : tempInstanceValues) {");
					psFact.println("        tempInstanceValue = instValue;");
					if (!theRefType.equals("Object")) {
						psFact.println("        if (tempInstanceValue.hasType(kb.getCls(\"" + theRefValueCls.getName() + "\") )) {");
					}
				}
				psFact.println("          try {");
				psFact.println("            Object referredBean = getJadeInstanceFor(tempInstanceValue);");
				psFact.println();
				psFact.println("            if (referredBean != null) {");


				psFact.println("              javaInst." + this.converter.toAddMethod(theSlot) + "(" + 
						" (" + theRefType + ") referredBean);");

				psFact.println("            }");
				psFact.println("          } catch (" + this.factoryExceptionClassName + " e) {");
				psFact.println("            e.printStackTrace();");
				psFact.println("          }");
				if (!theRefType.equals("Object"))
				{
					psFact.println("        }");
					psFact.println("        else {");
					psFact.println("          throw new " + this.factoryExceptionClassName + 
							"(\"Invalid type for instance \" + tempInstanceValue + \" referred by slot \\\"" + theSlot.getName() + "\\\"" + 
							" in instance \" + instanceName);");
					psFact.println("        }");
				}
				psFact.println("      }");
				}
				else
				{
					psFact.println("      javaInst." + this.converter.toSetMethod(theSlot) + "( new ArrayList(" + tempVarName + ") );");
				}
				psFact.println("    }");
				psFact.println();
			}
		}
		extern.append("      abs.set(");
		extern.append(this.converter.toOntologyClassName(this.ontologyName));
		extern.append(".");
		extern.append(this.converter.toRoleConstant(theCls, theSlot));
		extern.append(", ");
		if (absSlotType == null)
		{
			extern.append("(AbsTerm) ");
			extern.append("onto.fromObject(");
			extern.append(this.converter.toGetMethod(theSlot));
			extern.append("())");
		}
		else
		{
			extern.append(absSlotType);
		}
		extern.append(");\n");


		intern.append("      " + theVarName);
		if ((theType.equalsIgnoreCase("boolean")) || (theType.equalsIgnoreCase("int")) || 
				(theType.equalsIgnoreCase("float")))
		{
			intern.append(" = abs.get");
			if (theType.equalsIgnoreCase("int")) {
				theType = "integer";
			}
			intern.append(ProtegeTools.firstUpper(theType));
			intern.append("(");
			intern.append(this.converter.toOntologyClassName(this.ontologyName));
			intern.append(".");
			intern.append(this.converter.toRoleConstant(theCls, theSlot));
			intern.append(");\n");
		}
		else
		{
			intern.append(" = (");
			intern.append(theType);
			intern.append(")");
			intern.append("onto.toObject(abs.getAbsObject");
			intern.append("(");
			intern.append(this.converter.toOntologyClassName(this.ontologyName));
			intern.append(".");
			intern.append(this.converter.toRoleConstant(theCls, theSlot));
			intern.append("));\n");
		}
	}

	private boolean hasCardinalitySingle(Slot theSlot)
	{
		if ((theSlot instanceof OWLProperty)) {
			return ((OWLProperty)theSlot).isFunctional();
		}
		return !theSlot.getOwnSlotAllowsMultipleValues(theSlot);
	}

	private Cls getInstanceValueCls(Cls theCls, Slot theSlot)
	{
		Cls theValueCls = null;

		ValueType valueType = theSlot.getValueType();


		assert ((valueType.equals(ValueType.CLS)) || (valueType.equals(ValueType.INSTANCE)));


		Collection<?> c = new ArrayList();
		if (valueType.equals(ValueType.INSTANCE))
		{
			Cls superCls = getFirstDeclaringSuperclass(theCls, theSlot);
			c = superCls.getTemplateSlotAllowedClses(theSlot);
		}
		else if (valueType.equals(ValueType.CLS))
		{
			c = theCls.getTemplateSlotAllowedParents(theSlot);
		}
		if (c.isEmpty()) {
			theValueCls = this.itsKB.getRootCls();
		} else if (c.size() > 1) {
			theValueCls = null;
		}
		if (c.size() == 1) {
			theValueCls = (Cls)c.iterator().next();
		}
		return theValueCls;
	}

	private String getInstanceValueType(Cls theValueCls)
	{
		String theType = "Object";
		if (theValueCls != null)
		{
			theType = this.converter.getClassName(theValueCls);
			theType = fixNamespacePrefix(theType);
			if ((theValueCls.isSystem()) || (!isJADEClass(theValueCls)))
			{
				this.ontologyBeanGeneratorPanel.statusTextArea.append("     skipping " + 
						theType + "\n");
				return null;
			}
			theType = this.converter.toJavaTypeName(theType);
			if (isJADEPrimitive(theValueCls)) {
				if (theType.equals(this.JAVA_AID_CLASS_NAME)) {
					theType = "jade.core." + theType;
				} else {
					theType = "jade.content." + theType;
				}
			}
		}
		return theType;
	}

	private void setUsedPropertyType(ValueType valueType, TypeCardinality cardinality)
	{
		Set<TypeCardinality> valueTypeCardinalities = (Set)this.usedPropertyTypes.get(valueType);
		if (valueTypeCardinalities == null) {
			valueTypeCardinalities = new HashSet();
		}
		valueTypeCardinalities.add(cardinality);
		this.usedPropertyTypes.put(valueType, valueTypeCardinalities);
	}

	private Cls getFirstDeclaringSuperclass(Cls theCls, Slot theSlot)
	{
		Cls res = theCls;
		Cls superCls = null;
		for (Iterator<?> it = getDirectSuperclasses(theCls).iterator(); it.hasNext();)
		{
			superCls = (Cls)it.next();
			if (superCls.hasTemplateSlot(theSlot))
			{
				res = getFirstDeclaringSuperclass(superCls, theSlot);

				break;
			}
		}
		return res;
	}

	void doIt()
	{
		/*EDITED: The package name ontoScene can be used (but his variations OntoScene ecc.. can not).
		 * This is due to fact that windows does not differentiate between lower case and capital letters and
		 * 2 folders named 'ontoscene' can not be created.*/
		String ontoSceneString= ((OWLModel) getKnowledgeBase()).getNamespaceManager().getPrefix("http://www.owl-ontologies.com/OntoScene.owl#");
		boolean ontoSceneOntologyLoaded = ontoSceneString == null ? false : true;
		
		String packageNameCheck = (String) this.ontologyBeanGeneratorPanel.packageComboBox.getSelectedItem();
		if(ontoSceneOntologyLoaded && packageNameCheck.toLowerCase().equals("ontoscene") && !packageNameCheck.equals("ontoScene")) {
			JOptionPane.showMessageDialog(new JFrame(), packageNameCheck + " is not a valid package name. Did you mean 'ontoScene'?" , "Error", JOptionPane.ERROR_MESSAGE);
		  return;
		}
		
		try
		{
			reset();
		}
		catch (Exception e)
		{
			JOptionPane.showMessageDialog(null, e.getMessage(), "error", 
					0); return;
		}
		PrintStream ontologyFile;
		if (this.ontologyGeneration)
		{
			ontologyFile = createOntologyFile();
			if (ontologyFile == null) {
				this.ontologyBeanGeneratorPanel.statusTextArea.append("Ontology file COULD NOT be created!");
			}
		}
		else
		{
			ontologyFile = null;
		}
		PrintStream factoryFile;
		if (this.factoryGeneration)
		{
			factoryFile = createFactoryFile();
			if (factoryFile == null)
			{
				this.ontologyBeanGeneratorPanel.statusTextArea.append("Factory file COULD NOT be created!");
				return;
			}
			createFactoryExceptionFile();
			createMethodNameMapper();
		}
		else
		{
			factoryFile = null;
		}
		
		Cls conceptCls = this.itsKB.getCls(this.CONCEPT);
		Cls AIDCls = this.itsKB.getCls(this.AID);
		Cls agtactCls = this.itsKB.getCls(this.AGENTACTION);
		Cls predClS = this.itsKB.getCls(this.PREDICATE);

		Collection<?> conceptCol = new HashSet(conceptCls.getSubclasses());
		Collection<?> aidCol = new HashSet(AIDCls.getSubclasses());
		Collection<?> agtactClsCol = new HashSet(agtactCls.getSubclasses());
		Collection<?> predCol = new HashSet(predClS.getSubclasses());
		
	  //EDITED: look for the exception class named 'Exception:Base' and save his subclasses
		Cls exceptionClS = null;
		Collection<?> classes =  this.itsKB.getClses();
		Iterator<?> iter = classes.iterator();
		while (iter.hasNext())
		{
			Cls theCls = (Cls)iter.next();

//			if(theCls instanceof AbstractRDFSClass) {
//				if(ontoSceneOntologyLoaded && ((AbstractRDFSClass) theCls).getPrefixedName().equals("OntoScene:Exception")) {
//					exceptionClS = theCls; 
//					break;
//				}
//			}
			
			if(theCls instanceof AbstractRDFSClass) {
				if(((AbstractRDFSClass) theCls).getPrefixedName().equals(ONTOLOGY_EXCEPTION_PREFIX_NO_UNDERSCORE + ":" + ONTOLOGY_EXCEPTION)) {
					exceptionClS = theCls; 
					break;
				}
			}

//			if(theCls.getName().contains("#") && theCls.getName().split("#")[1].equals(ONTOLOGY_EXCEPTION)) {
//				exceptionClS = theCls; 
//				break;
//			}
		}
		
	  //EDITED: Check that all the exceptions have prefix = Exception
		if(exceptionClS != null) {
			
			boolean notValidSubExceptionFound = false;
			
			if(exceptionClS instanceof AbstractRDFSClass) {
				for(Object o : ((AbstractRDFSClass)exceptionClS).getSubclasses()) {
					//If there is at least an exception that is not from Exception
					if(!((DefaultRDFSNamedClass) o).getPrefixedName().split(":")[0].equals("Exception")) {
						notValidSubExceptionFound = true;
						break;
					}
				}
			}
			//Do not export exceptions
			if(notValidSubExceptionFound)
			  exceptionClS = null;
		}
		
		
		Collection<?> exceptionCol = null;
		if(exceptionClS != null) {
			
//			if(exceptionClS instanceof AbstractRDFSClass) {
//				for(Object o : ((AbstractRDFSClass)exceptionClS).getSubclasses()) {
//					//If there is at least an exception that is not from OntoScene
//					if(!((DefaultRDFSNamedClass) o).getPrefixedName().split(":")[0].equals("OntoScene")) {
//						exceptionCountMoreThanZero = true;
//					  break;
//					}
//				}
//			}
			
			ontologyExceptionExists = true;
			exceptionCol = new HashSet(exceptionClS.getSubclasses());
			generateExceptionsByCollection(exceptionCol);
			
			//Remove exceptions to avoid generating their interfaces
			conceptCol.removeAll(exceptionCol);
			aidCol.removeAll(exceptionCol);
			agtactClsCol.removeAll(exceptionCol);
			predCol.removeAll(exceptionCol);
		}
		else
			ontologyExceptionExists = false;

		filterClasses(conceptCol, aidCol, agtactClsCol, predCol);
		int max = conceptCol.size() + aidCol.size() + agtactClsCol.size() + 
				predCol.size();
		this.ontologyBeanGeneratorPanel.progress.setMaximum(max);
		this.ontologyBeanGeneratorPanel.progress.setMinimum(0);
		
		//EDITED: to load methods from the xml file before generating the bean interfaces
		try {
			classNameToMethodsHashMap = XMLParser.readXMLFile(xmlFilePath);
		} 
		catch (Exception e) {
			//No methods to add if the file could not be read
			classNameToMethodsHashMap = new HashMap<String, List<Method>>();
		} 
		
		generateByCollection(conceptCol, this.JAVA_CONCEPT_CLASS_NAME, factoryFile);
		generateByCollection(agtactClsCol, this.JAVA_AGENTACTION_CLASS_NAME, factoryFile);
		generateByCollection(aidCol, this.JAVA_AID_CLASS_NAME, factoryFile);
		generateByCollection(predCol, this.JAVA_PREDICATE_CLASS_NAME, factoryFile);
		
		if (this.ontologyGeneration)
		{
			/*ontologyFile.println(this.ontologyHeader);
        ontologyFile.println(this.ontologyInit);
        ontologyFile.println("    // adding fields");
        ontologyFile.println(this.ontologyBody);
        ontologyFile.println("    // adding name mappings");
        ontologyFile.println(this.nameMapping);
        ontologyFile.println("    // adding inheritance");
        ontologyFile.println(this.ontologyInheritance);*/
			ontologyFile.print("\n  private " + this.converter.toOntologyClassName(this.ontologyName) + "(){ \n");
			ontologyFile.print("    super(ONTOLOGY_NAME, BasicOntology.getInstance());\n");
			ontologyFile.print("    try { \n");
			//ontologyFile.println("      add(" + "\"" + this.packageName + "ontology\"" + ");");
			//EDITED AGAIN
			//ontologyFile.println("      add(" + "\"" + this.packageName + "\"" + ");");
			
		  //EDITED
			//ontologyFile.println("    add(\"" + this.packageName + ".impl\");");
			boolean ontoSceneMode = false;
			if(ontoSceneOntologyLoaded && !packageName.equals("ontoScene"))
			  ontoSceneMode = true;
			
				ontologyFile.print("        //Concepts\n");
				addClassesToOntology(ontoSceneMode, ontologyFile, conceptCol);
				ontologyFile.print("        //AgentActions\n");
				addClassesToOntology(ontoSceneMode, ontologyFile, agtactClsCol);
				ontologyFile.print("        //AIDs\n");
				addClassesToOntology(ontoSceneMode, ontologyFile, aidCol);
				ontologyFile.print("        //Predicates\n");
				addClassesToOntology(ontoSceneMode, ontologyFile, predCol);
			
				// DANY
				// 2020 This seems not to work with some java/jade versions...
				//ontologyFile.print("        add(\"" +  this.packageName + ".impl\");\n" );
				// added directly all the classes in impl
				
//			ontologyFile.println(
//					"    }catch (java.lang.Exception e) {e.printStackTrace();}");
  		ontologyFile.println(
				"    }catch (BeanOntologyException e) {e.printStackTrace();}");
			ontologyFile.println("  }");
			ontologyFile.println("}");
			ontologyFile.flush();
			ontologyFile.close();
		}
//		if (this.ontologyGeneration)
//    {
//		  
//      ontologyFile.println(this.ontologyHeader);
//      ontologyFile.println(this.ontologyInit);
//      ontologyFile.println("    // adding fields");
//      ontologyFile.println(this.ontologyBody);
//      ontologyFile.println("    // adding name mappings");
//      ontologyFile.println(this.nameMapping);
//      ontologyFile.println("    // adding inheritance");
//      ontologyFile.println(this.ontologyInheritance);
//      
//      
//      ontologyFile.println(
//        "   }catch (java.lang.Exception e) {e.printStackTrace();}");
//      ontologyFile.println("  }");
//      ontologyFile.println("}");
//      ontologyFile.flush();
//      ontologyFile.close();
//    }
		if (this.factoryGeneration)
		{
			generateAllAccessors(factoryFile);

			this.factoryGlobal.append("  }\n");
			factoryFile.println(this.factoryGlobal.toString());
			factoryFile.println("}");
			factoryFile.flush();
			factoryFile.close();
		}
		saveProjectAnnotation();
		this.ontologyBeanGeneratorPanel.saveProperties();
		
		String locationName = (String) this.ontologyBeanGeneratorPanel.locationComboBox.getSelectedItem();
		
	  //EDITED: export the ontoScene.jar if the template ontology OntoScene.owl is loaded and if the package name is not already named ontoScene
		boolean ontoSceneToExportAsWell = false;
		if(ontoSceneOntologyLoaded && !packageName.equals("ontoScene")) {
			ontoSceneToExportAsWell = true;
			try {
				File dir = new File(locationName + "\\ontoScene");
				dir.mkdir();
				File implDir = new File(locationName + "\\ontoScene\\impl");
				implDir.mkdir();
//				File exceptionsDir = new File(locationName + "\\ontoScene\\exceptions");
//				exceptionsDir.mkdir();

//				final File packageFolder = new File(locationName + "\\" + packageName);
//				//Move OntoScene_ interfaces from 'locationName\packageName' folder to 'locationName\ontoScene' folder 
//				moveOntoSceneInterfaces(packageFolder, locationName);
//				//Make the interfaces in 'locationName\packageName' folder import the ontoScene package
//				changeDomainFiles(packageFolder);
				
				final File packageFolder = new File(locationName + "\\" + packageName);
				//Move OntoScene_ interfaces from 'locationName\packageName' folder to 'locationName\ontoScene' folder 
				moveOntoSceneInterfaces(packageFolder, locationName);
				//Make the interfaces in 'locationName\packageName' folder import the ontoScene package
				changeDomainInterfaces(packageFolder);
				
				final File packageImplFolder = new File(locationName + "\\" + packageName + "\\impl");
				//Make the implementations in 'locationName\packageName\impl' folder import the ontoScene package
				changeDomainImplementations(packageImplFolder);
				//Move OntoScene_ implementations from 'locationName\packageName\impl' folder to 'locationName\ontoScene\impl' folder 
				moveOntoSceneImplementations(packageImplFolder, locationName);

				
//				final File packageImplFolder = new File(locationName + "\\" + packageName + "\\impl");
//				//Make the implementations in 'locationName\packageName\impl' folder import the ontoScene package
//				changeDomainFiles(packageImplFolder);
//				//Move OntoScene_ implementations from 'locationName\packageName\impl' folder to 'locationName\ontoScene\impl' folder 
//				moveOntoSceneImplementations(packageImplFolder, locationName);
//
//				final File packageExceptionsFolder = new File(locationName + "\\" + packageName + "\\exceptions");
//				//Move OntoScene_ exceptions from 'locationName\packageName\exceptions' folder to 'locationName\ontoScene\exceptions' folder 
//				moveOntoSceneExceptions(packageExceptionsFolder, locationName);
//				//Make the exceptions in 'locationName\packageName\exceptions' folder import the ontoScene package
//				changeDomainFiles(packageExceptionsFolder);
			}
			catch (Exception e) {
			}
		}
		
	  //EDITED: try to compile java files and to create the jar
//		if(ontoSceneToExportAsWell)
//			CMD.exec(new String[] {"cmd",}, locationName, packageName, jadePath, ontologyExceptionExists && exceptionCountMoreThanZero, true);
//		else
//			CMD.exec(new String[] {"cmd",}, locationName, packageName, jadePath, ontologyExceptionExists, false);
		CMD.exec(new String[] {"cmd",}, locationName, packageName, jadePath, ontologyExceptionExists, ontoSceneToExportAsWell);
		// DANY
		// questo sopra  il comando che crea il jar, ma non credo di doverci mettere mano
		
		//CMD.exec(new String[] {"cmd",}, locationName, packageName, jadePath, ontologyExceptionExists && exceptionCountMoreThanZero, ontoSceneToExportAsWell);
		
	  //EDITED: try to generate prolog subclass rules from a given list of classes
		if(this.prologGeneration) {
			
			String classesToExport = (String) this.ontologyBeanGeneratorPanel.factoryComboBox.getSelectedItem();
			if(classesToExport != null) {
				String[] classArray = classesToExport.split(";");

				if(classArray.length > 0) {

					List<String> classList = new ArrayList<String>();
					for(String c : classArray)
						classList.add(c);

					generatePrologRulesFromClasses(classList, locationName);
				}
			}
		}
		
	}
	
	//EDITED
	private void addClassesToOntology(boolean ontoSceneMode, PrintStream ontologyFile, Collection<?> classes) {
		Iterator<?> classIter = classes.iterator();
		while (classIter.hasNext())
		{
			Cls theCls = (Cls)classIter.next();
			if(theCls instanceof AbstractRDFSClass) {
			  String prefixName = ((AbstractRDFSClass) theCls).getPrefixedName();
			  String[] preficNameSplitted = prefixName.split(":");
			  String clsName;
			  if(preficNameSplitted.length > 1) 
			  	clsName = preficNameSplitted[0] + "_" + preficNameSplitted[1];
			  else
			  	clsName = preficNameSplitted[0];
			  
			  //EDITED
			  //Bug Lac_Sainte-Marie should become Lac_Sainte_Marie
			  // DANY
			  // classes names are created in Java style, that is, with the first letter that is Upper case,
			  // so here I have to be sure that the first letter is uppercase
			  clsName =  ProtegeTools.firstUpper(clsName.replace('-', '_'));
			  
			  if(ontoSceneMode && clsName.startsWith("OntoScene_")) { //If it is a class from OntoScene
			    ontologyFile.println("        add(ontoScene." + clsName + ".class);");
	  			ontologyFile.println("        add(ontoScene.impl." + "Default" + clsName + ".class);" );
			  }
			  else {
			  	if(clsName.startsWith("Exception_")) { //If it is an exception
			  		ontologyFile.println("        add(" + this.packageName + ".exceptions." + clsName + ".class);");
			  	}
			  	else { //Normal case
			  		// DANY
			  		ontologyFile.println("        add(" + this.packageName + "." + clsName + ".class);");
			  		
			  		// DANY 2020
			  		// before I removed... now here again
			  		ontologyFile.println("        add(" + this.packageName + ".impl." + "Default" + clsName + ".class);" );
			  	}
			  }
			  
			}
			else
				continue;
    }
	}
	
	private void moveOntoSceneInterfaces(final File folder, String locationName) throws Exception {
		for (final File fileEntry : folder.listFiles()) {
			if(fileEntry.isDirectory()) {
				//listFilesForFolder(fileEntry);
			} 
			else {
				if(fileEntry.getName().startsWith("OntoScene_")) {
					BufferedReader br = new BufferedReader(new FileReader(fileEntry));
					String line;
					String copiedFileContent = "";
					while ((line = br.readLine()) != null) {
						if(line.startsWith("package"))
							copiedFileContent += "package ontoScene;\n";
						else if(line.startsWith("import " + this.packageName)) { //Do not import the domain package in OntoScene
						}
						else
							copiedFileContent += line + "\n";
					}
					br.close();
					PrintWriter out = new PrintWriter(locationName + "\\ontoScene\\" + fileEntry.getName());
					out.println(copiedFileContent);
					out.close();
					
					if(fileEntry.exists()) {
						fileEntry.delete();
					}
				}
			}
		}
	}
	
	private void changeDomainImplementations(final File folder) throws Exception {
		for (final File fileEntry : folder.listFiles()) {
			if(fileEntry.isDirectory()) {
				//listFilesForFolder(fileEntry);
			} 
			else {
				BufferedReader br = new BufferedReader(new FileReader(fileEntry));
				String line;
				String copiedFileContent = "";
				while ((line = br.readLine()) != null) {
					if(line.startsWith("package"))
						copiedFileContent += line + "\nimport ontoScene.*;\n"; //import ontoScene.exceptions.*;\n";
					else if(line.startsWith("public class DefaultOntoScene_")) {// error if someone calls his class DefaultOntoScene_something?
						String[] parts = line.split("implements");
						String name = fileEntry.getName().substring(7); // remove Default
						name = name.substring(0, name.length() - 5); // remove .java
						String newLine = parts[0] + "implements " + name + " {\n";
						copiedFileContent += newLine;
					}
					else if(line.endsWith(".exceptions.*;") && !ontologyExceptionExists) {// skip the line if the package has no exceptions
						
					}
					else
						copiedFileContent += line + "\n";
				}
				br.close();
				String fileName = fileEntry.getName();
				if(fileEntry.exists()) {
					fileEntry.delete();
				}
				PrintWriter out = new PrintWriter(folder.getAbsolutePath() + "\\" + fileName);
				out.println(copiedFileContent);
				out.close();

			}
		}
	}
	
	private void changeDomainInterfaces(final File folder) throws Exception {
		for (final File fileEntry : folder.listFiles()) {
			if(fileEntry.isDirectory()) {
				//listFilesForFolder(fileEntry);
			} 
			else {
				BufferedReader br = new BufferedReader(new FileReader(fileEntry));
				String line;
				String copiedFileContent = "";
				while ((line = br.readLine()) != null) {
					if(line.startsWith("package"))
						copiedFileContent += line + "\nimport ontoScene.*;\n"; //import ontoScene.exceptions.*;\n";
					else if(line.endsWith(".exceptions.*;") && !ontologyExceptionExists) {// skip the line if the package has no exceptions
					}
					else
						copiedFileContent += line + "\n";
				}
				br.close();
				String fileName = fileEntry.getName();
				if(fileEntry.exists()) {
					fileEntry.delete();
				}
				PrintWriter out = new PrintWriter(folder.getAbsolutePath() + "\\" + fileName);
				out.println(copiedFileContent);
				out.close();

			}
		}
	}
	
//	private void changeDomainFiles(final File folder) throws Exception {
//		for (final File fileEntry : folder.listFiles()) {
//			if(fileEntry.isDirectory()) {
//				//listFilesForFolder(fileEntry);
//			} 
//			else {
//				BufferedReader br = new BufferedReader(new FileReader(fileEntry));
//				String line;
//				String copiedFileContent = "";
//				while ((line = br.readLine()) != null) {
//					if(line.startsWith("package"))
//						copiedFileContent += line + "\nimport ontoScene.*;\n"; //import ontoScene.exceptions.*;\n";
//					else if(line.startsWith("public class DefaultOntoScene_")) {// error if someone calls his class DefaultOntoScene_something
//						String[] parts = line.split("implements");
//						String name = fileEntry.getName().substring(7); // remove Default
//						name = name.substring(0, name.length() - 5); // remove .java
//						String newLine = parts[0] + "implements " + name + " {\n";
//						copiedFileContent += newLine;
//					}
//					else if(line.endsWith(".exceptions.*;") && !ontologyExceptionExists) {// skip the line if the package has no exceptions
//						
//					}
//					else
//						copiedFileContent += line + "\n";
//				}
//				br.close();
//				String fileName = fileEntry.getName();
//				if(fileEntry.exists()) {
//					fileEntry.delete();
//				}
//				PrintWriter out = new PrintWriter(folder.getAbsolutePath() + "\\" + fileName);
//				out.println(copiedFileContent);
//				out.close();
//
//			}
//		}
//	}
	
	private void moveOntoSceneImplementations(final File folder, String locationName) throws Exception {
		for (final File fileEntry : folder.listFiles()) {
			if(fileEntry.isDirectory()) {
				//listFilesForFolder(fileEntry);
			} 
			else {
				if(fileEntry.getName().startsWith("DefaultOntoScene_")) {
					BufferedReader br = new BufferedReader(new FileReader(fileEntry));
					String line;
					String copiedFileContent = "";
					while ((line = br.readLine()) != null) {
						if(line.startsWith("package"))
							copiedFileContent += "package ontoScene.impl;\n";
						else if(line.startsWith("import " + this.packageName)) { //Do not import the domain package in OntoScene
						}
						else
							copiedFileContent += line + "\n";
					}
					br.close();
					PrintWriter out = new PrintWriter(locationName + "\\ontoScene\\impl\\" + fileEntry.getName());
					out.println(copiedFileContent);
					out.close();
					
					if(fileEntry.exists()) {
						fileEntry.delete();
					}
				}
			}
		}
	}
	
//	private void moveOntoSceneExceptions(final File folder, String locationName) throws Exception {
//		for (final File fileEntry : folder.listFiles()) {
//			if(fileEntry.isDirectory()) {
//				//listFilesForFolder(fileEntry);
//			} 
//			else {
//				if(fileEntry.getName().startsWith("OntoScene_")) {
//					BufferedReader br = new BufferedReader(new FileReader(fileEntry));
//					String line;
//					String copiedFileContent = "";
//					while ((line = br.readLine()) != null) {
//						if(line.startsWith("package"))
//							copiedFileContent += "package ontoScene.exceptions;\n";
//						else if(line.startsWith("import")) {
//							
//						}
//						else
//							copiedFileContent += line + "\n";
//					}
//					br.close();
//					PrintWriter out = new PrintWriter(locationName + "\\ontoScene\\exceptions\\" + fileEntry.getName());
//					out.println(copiedFileContent);
//					out.close();
//					
//					if(fileEntry.exists()) {
//						fileEntry.delete();
//					}
//				}
//			}
//		}
//	}
	
	private void generatePrologRulesFromClasses(List<String> classList, String locationName) {
		
		PrintWriter writer;
    try {
    	String path = locationName + "\\subClasses.pl";
    	System.out.println("Prolog sub-classes exported: " + path);
	    writer = new PrintWriter(path, "UTF-8");
	    writer.println("% Generated with 'Ontology Bean Generator 5.0'\n");
	    
	    writer.println("subclass_of(X,Y) :-\n" + 
	                   "  subclass(X,Y).\n" +
	                   "\nsubclass_of(X,Y) :-\n" +
	                   "  subclass(X,Z),\n" +
	                   "  subclass(Z,Y).\n");
    } 
    catch (FileNotFoundException | UnsupportedEncodingException e) {
	    return;
    }
		
		for(String chosenClass : classList) {
			Cls chosenClS = null;
			Collection<?> classes =  this.itsKB.getClses();
			Iterator<?> iter = classes.iterator();
			while (iter.hasNext())
			{
				Cls theCls = (Cls)iter.next();
				if(theCls.getName().contains("#") && theCls.getName().split("#")[1].equals(chosenClass)) {
					chosenClS = theCls;
					break;
				}
			}
			Collection<Cls> col = null;

			col = new HashSet();
			col.add(chosenClS);

			if(chosenClS != null) {
				//col = new HashSet(chosenClS.getSubclasses());
				
				Collection<Cls> subClassesCol = new HashSet(chosenClS.getSubclasses());
				col.addAll(subClassesCol);
				
				writer.println("% " + chosenClass + " sub-classes:\n");

				Iterator<?> classIter = col.iterator();
				while (classIter.hasNext())
				{
					Cls theCls = (Cls)classIter.next();
					Collection<?> subClasses = theCls.getDirectSubclasses();
					
					if(!subClasses.isEmpty()) {
						//writer.println(getCleanName(theCls.getName().toLowerCase()) + "(X) :-");
						
						Iterator<?> it = subClasses.iterator();
						while (it.hasNext())
						{
							Cls cls = (Cls)it.next();
							
							//writer.println("subclass("+ getCleanName(theCls.getName().toLowerCase()) + ", " + getCleanName(cls.getName().toLowerCase()) + ").\n");

							writer.println("subclass('"+ getCleanName(theCls.getName()) + "', '" + getCleanName(cls.getName()) + "').\n");

//							Collection<?> subSubClasses = cls.getDirectSubclasses();
//							String endLine = it.hasNext() ? ";" : ".\n";
//							if(subSubClasses.isEmpty()) {
//								writer.println("  " + "X = " + getCleanName(cls.getName().toLowerCase()) + endLine);
//							}
//							else {
//								writer.println("  " + getCleanName(cls.getName().toLowerCase()) + "(X)" + endLine);
//							}
								
						}
					}
				}
			}
		}

		writer.close();
	}
	
	private String getCleanName(String ontologyName) {
		
		if(ontologyName.contains("#"))
			return ontologyName.split("#")[1];
		
		return ontologyName;
	}

	private static enum TypeCardinality
	{
		Single,  Multiple;
	}

	private HashMap<ValueType, Set<TypeCardinality>> usedPropertyTypes = new HashMap();
	private boolean applyFormatting;
	private boolean beanGeneration;
	private boolean factoryGeneration;
	private boolean prologGeneration;
	private NameConverter converter;
	private String dirName;
	private boolean fullBeanSupport;
	private KnowledgeBase itsKB;
	private Slot jadeIgnoredSlot;
	private Slot jadeUnnamedSlot;
	private Slot javaJavaCodeSlot;
	private Slot javaSuperClassSlot;
	private boolean microSupport;
	private StringBuffer nameMapping;
	private OntologyBeanGeneratorPanel ontologyBeanGeneratorPanel;
	private StringBuffer ontologyBody;
	private boolean ontologyGeneration;
	private StringBuffer ontologyHeader;
	private StringBuffer ontologyInheritance;
	private StringBuffer ontologyInit;
	private StringBuffer factoryGlobal;
	private HashMap<Cls, Collection<Slot>> jadeTemplateSlots;
	private String ontologyName;
	private String factoryClassName;
	private String factoryExceptionClassName;

	private void generateAllAccessors(PrintStream factoryFile)
	{
		factoryFile.println("  //Typed access methods to slot values");
		for (ValueType valueType : this.usedPropertyTypes.keySet())
		{
			Set<TypeCardinality> valueTypeCardinalities = (Set)this.usedPropertyTypes.get(valueType);
			if (valueTypeCardinalities.contains(TypeCardinality.Single)) {
				generateTypedAccessor(factoryFile, valueType, TypeCardinality.Single);
			}
			if (valueTypeCardinalities.contains(TypeCardinality.Multiple)) {
				generateTypedAccessor(factoryFile, valueType, TypeCardinality.Multiple);
			}
		}
	}

	private void generateTypedAccessor(PrintStream factoryFile, ValueType valueType, TypeCardinality cardinality)
	{
		if (cardinality == TypeCardinality.Single)
		{
			if ((valueType.equals(ValueType.SYMBOL)) || (valueType.equals(ValueType.STRING))) {
				createTypedAccessorFromTemplate(factoryFile, "String");
			} else if (valueType.equals(ValueType.INTEGER)) {
				createTypedAccessorFromTemplate(factoryFile, "Integer");
			} else if (valueType.equals(ValueType.FLOAT)) {
				createTypedAccessorFromTemplate(factoryFile, "Float");
			} else if (valueType.equals(ValueType.BOOLEAN)) {
				createTypedAccessorFromTemplate(factoryFile, "Boolean");
			} else if (valueType.equals(ValueType.ANY))
			{
				if ((this.itsKB instanceof OWLModel))
				{
					factoryFile.println("  private static Object getObjectValue(OWLIndividual theInst, RDFProperty theSlot) {");
					factoryFile.println("    return theInst.getPropertyValue(theSlot);");
					factoryFile.println("  }");
				}
				else
				{
					factoryFile.println("  private static Object getObjectValue(Instance theInst, Slot theSlot) {");
					factoryFile.println("    return theInst.getOwnSlotValue(theSlot);");
					factoryFile.println("  }");
				}
			}
			else if (valueType.equals(ValueType.INSTANCE))
			{
				if ((this.itsKB instanceof OWLModel))
				{
					factoryFile.println("  private static OWLIndividual getInstanceValue(OWLIndividual theInst, RDFProperty theSlot) {");
					factoryFile.println("    return (OWLIndividual)theInst.getPropertyValue(theSlot);");
					factoryFile.println("  }");
				}
				else
				{
					factoryFile.println("  private static Instance getInstanceValue(Instance theInst, Slot theSlot) {");
					factoryFile.println("    return (Instance)theInst.getOwnSlotValue(theSlot);");
					factoryFile.println("  }");
				}
			}
			else if (valueType.equals(ValueType.CLS)) {
				if ((this.itsKB instanceof OWLModel))
				{
					factoryFile.println("  private static Class getClsValue(OWLIndividual theInst, RDFProperty theSlot) {");

					factoryFile.println("    return theInst.getPropertyValue(theSlot);");
					factoryFile.println("  }");
				}
				else
				{
					factoryFile.println("  private static Class getClsValue(Instance theInst, Slot theSlot) {");

					factoryFile.println("    return theInst.getOwnSlotValue(theSlot);");
					factoryFile.println("  }");
				}
			}
			factoryFile.println();
		}
		if (cardinality == TypeCardinality.Multiple)
		{
			if ((valueType.equals(ValueType.SYMBOL)) || (valueType.equals(ValueType.STRING))) {
				createTypedCollectionAccessorFromTemplate(factoryFile, "String");
			} else if (valueType.equals(ValueType.INTEGER)) {
				createTypedCollectionAccessorFromTemplate(factoryFile, "Integer");
			} else if (valueType.equals(ValueType.FLOAT)) {
				createTypedCollectionAccessorFromTemplate(factoryFile, "Float");
			} else if (valueType.equals(ValueType.BOOLEAN)) {
				createTypedCollectionAccessorFromTemplate(factoryFile, "Boolean");
			} else if (valueType.equals(ValueType.ANY))
			{
				if ((this.itsKB instanceof OWLModel))
				{
					factoryFile.println("  private static java.util.ArrayList<Object> getObjectValues(OWLIndividual theInst, RDFProperty theSlot) {");
					factoryFile.println("    return new java.util.ArrayList<Object>(theInst.getPropertyValues(theSlot));");
					factoryFile.println("  }");
				}
				else
				{
					factoryFile.println("  private static java.util.ArrayList<Object> getObjectValues(Instance theInst, Slot theSlot) {");
					factoryFile.println("    return new java.util.ArrayList<Object>(theInst.getOwnSlotValues(theSlot));");
					factoryFile.println("  }");
				}
			}
			else if (valueType.equals(ValueType.INSTANCE))
			{
				if ((this.itsKB instanceof OWLModel))
				{
					factoryFile.println("  private static java.util.ArrayList<OWLIndividual> getInstanceValues(OWLIndividual theInst, RDFProperty theSlot) {");
					factoryFile.println("    return new java.util.ArrayList<OWLIndividual>(theInst.getPropertyValues(theSlot));");
					factoryFile.println("  }");
				}
				else
				{
					factoryFile.println("  private static java.util.ArrayList<Instance> getInstanceValues(Instance theInst, Slot theSlot) {");
					factoryFile.println("    return new java.util.ArrayList<Instance>(theInst.getOwnSlotValues(theSlot));");
					factoryFile.println("  }");
				}
			}
			else if (valueType.equals(ValueType.CLS)) {
				if ((this.itsKB instanceof OWLModel))
				{
					factoryFile.println("  private static java.util.ArrayList<Class> getClsValues(OWLIndividual theInst, RDFProperty theSlot) {");

					factoryFile.println("    return new java.util.ArrayList(theInst.getPropertyValues(theSlot));");
					factoryFile.println("  }");
				}
				else
				{
					factoryFile.println("  private static java.util.ArrayList<Class> getClsValues(Instance theInst, Slot theSlot) {");

					factoryFile.println("    return new java.util.ArrayList(theInst.getOwnSlotValues(theSlot));");
					factoryFile.println("  }");
				}
			}
			factoryFile.println();
		}
	}

	private void createTypedAccessorFromTemplate(PrintStream factoryFile, String javaTypeName)
	{
		if ((this.itsKB instanceof OWLModel))
		{
			factoryFile.println("  private static " + javaTypeName + " get" + javaTypeName + "Value(OWLIndividual theInst, RDFProperty theSlot) {");
			factoryFile.println("    return (" + javaTypeName + ")theInst.getPropertyValue(theSlot);");
			factoryFile.println("  }");
		}
		else
		{
			factoryFile.println("  private static " + javaTypeName + " get" + javaTypeName + "Value(Instance theInst, Slot theSlot) {");
			factoryFile.println("    return (" + javaTypeName + ")theInst.getOwnSlotValue(theSlot);");
			factoryFile.println("  }");
		}
	}

	private void createTypedCollectionAccessorFromTemplate(PrintStream factoryFile, String javaTypeName)
	{
		if ((this.itsKB instanceof OWLModel)) {
			factoryFile.println("  java.util.ArrayList <" + javaTypeName + "> get" + javaTypeName + "Values(OWLIndividual theInst, RDFProperty theSlot) { ");
		} else {
			factoryFile.println("  java.util.ArrayList <" + javaTypeName + "> get" + javaTypeName + "Values(Instance theInst, Slot theSlot) { ");
		}
		factoryFile.println("    java.util.ArrayList <" + javaTypeName + "> res = new java.util.ArrayList<" + javaTypeName + ">();");
		if ((this.itsKB instanceof OWLModel)) {
			factoryFile.println("    java.util.Collection slotValues = theInst.getPropertyValues(theSlot);");
		} else {
			factoryFile.println("    java.util.Collection slotValues = theInst.getOwnSlotValues(theSlot);");
		}
		factoryFile.println("");
		factoryFile.println("    if (slotValues == null)");
		factoryFile.println("      return null;");
		factoryFile.println("");
		factoryFile.println("    for (Object slotValue : slotValues) {");
		factoryFile.println("      res.add( (" + javaTypeName + ") slotValue );");
		factoryFile.println("    }");
		factoryFile.println("    return res;");
		factoryFile.println("  }");
	}

	private String getTypedAccessorName(ValueType valueType, TypeCardinality cardinality)
	{
		if (cardinality == TypeCardinality.Single)
		{
			if ((valueType.equals(ValueType.SYMBOL)) || (valueType.equals(ValueType.STRING))) {
				return "getStringValue";
			}
			if (valueType.equals(ValueType.INTEGER)) {
				return "getIntegerValue";
			}
			if (valueType.equals(ValueType.FLOAT)) {
				return "getFloatValue";
			}
			if (valueType.equals(ValueType.BOOLEAN)) {
				return "getBooleanValue";
			}
			if (valueType.equals(ValueType.ANY)) {
				return "getObjectValue";
			}
			if (valueType.equals(ValueType.INSTANCE)) {
				return "getInstanceValue";
			}
			if (valueType.equals(ValueType.CLS)) {
				return "getClsValue";
			}
		}
		if (cardinality == TypeCardinality.Multiple)
		{
			if ((valueType.equals(ValueType.SYMBOL)) || (valueType.equals(ValueType.STRING))) {
				return "getStringValues";
			}
			if (valueType.equals(ValueType.INTEGER)) {
				return "getIntegerValues";
			}
			if (valueType.equals(ValueType.FLOAT)) {
				return "getFloatValues";
			}
			if (valueType.equals(ValueType.BOOLEAN)) {
				return "getBooleanValues";
			}
			if (valueType.equals(ValueType.ANY)) {
				return "getObjectValues";
			}
			if (valueType.equals(ValueType.INSTANCE)) {
				return "getInstanceValues";
			}
			if (valueType.equals(ValueType.CLS)) {
				return "getClsValues";
			}
		}
		return "";
	}

	private void filterClasses(Collection<?> conceptCol, Collection<?> aidCol, Collection<?> agtactClsCol, Collection<?> predCol)
	{
		if (this.itsKB.getCls(this.CONCEPT) == null) {
			System.out.println("no Concepts found");
		} else {
			conceptCol.remove(this.itsKB.getCls(this.CONCEPT));
		}
		if (this.itsKB.getCls(this.AID) != null) {
			conceptCol.remove(this.itsKB.getCls(this.AID));
		}
		if (this.itsKB.getCls(this.AGENTACTION) != null) {
			conceptCol.remove(this.itsKB.getCls(this.AGENTACTION));
		}
		Iterator<?> agtactClsItor = agtactClsCol.iterator();
		while (agtactClsItor.hasNext()) {
			conceptCol.remove(agtactClsItor.next());
		}
		Iterator<?> aidColItor = aidCol.iterator();
		while (aidColItor.hasNext()) {
			conceptCol.remove(aidColItor.next());
		}
		removeIgnoredClasses(conceptCol);
		removeIgnoredClasses(aidCol);
		removeIgnoredClasses(agtactClsCol);
		removeIgnoredClasses(predCol);
	}

	private void finishUpRole(Cls theCls)
	{
		if (!this.microSupport) {
			this.ontologyHeader.append("    add(" + this.converter.toSchemaVariableName(theCls, null) + 
					", " + this.packageName + "." + 
					this.converter.toJavaTypeName(theCls) + ".class);\n");
		} else {
			this.ontologyHeader.append("    add(" + this.converter.toSchemaVariableName(theCls, null) + ", Class.forName(\"" + this.packageName + "." + this.converter.toJavaTypeName(theCls) + "\"));\n");
		}
	}

	private void format(File source) {}

	private void formatImports(File source)
	{ FileWriter destWriter;
	File dest = new File(source.getPath() + ".tmp");
	try
	{
		destWriter = new FileWriter(dest);
	}
	catch (IOException e)
	{

		e.printStackTrace();
		return;
	}
	try
	{

		BufferedReader sourceReader = new BufferedReader(new FileReader(source));
		String line;
		while ((line = sourceReader.readLine()) != null)
		{

			if (!line.contains("import jade.util.leap.*;"))
			{
				destWriter.append(line);
				destWriter.append(System.getProperty("line.separator"));
			}
		}
		destWriter.flush();
		destWriter.close();
		sourceReader.close();
	}
	catch (FileNotFoundException e)
	{
		e.printStackTrace();
		return;
	}
	catch (IOException e)
	{
		e.printStackTrace();
		return;
	}
	source.delete();
	dest.renameTo(source);
	}
	
	//EDITED: to generate the exception files
	private void generateExceptionsByCollection(Collection<?> theClasses) {
		//Generate OntologyException class (extends Exception from java)
		try {
			File file = null;
			StringBuffer fileName = new StringBuffer(this.converter.toDir(this.packageName, this.dirName));
			fileName.append("\\exceptions\\");
			fileName.append(ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION);
			fileName.append(".java");
			file = new File(fileName.toString());
			file.getParentFile().mkdirs();
			file.createNewFile();
			try
			{
				FileOutputStream fos = new FileOutputStream(file);
				OutputStream bos = new BufferedOutputStream(fos);
				PrintStream ps = new PrintStream(bos);

				ps.println("package " + this.packageName + ".exceptions" + ";");
				ps.println();
				ps.println("import " + this.packageName + ".*;");
				ps.println();
				ps.println("/**");
				ps.println("* " +  ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION);
				ps.println("* @author " + OntologyBeanGeneratorUtil.getVersionInformation());
				ps.println("* @version " + OntologyBeanGeneratorUtil.getDate());
				ps.println("*/");

				//EDITED: make the base exception class implement Concept
				//ps.print("public class " + ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION + " extends Exception implements jade.content.Concept");
				ps.print("public class " + ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION + " extends Exception");
				ps.println(" { public " +  ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION + "() {} }");
				ps.flush();
				ps.close();
				
				this.ontologyBeanGeneratorPanel.statusTextArea.append(" created file: " + ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION + ".java\n");
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		
		//Generate other exceptions (extends OntologyException)
		Iterator<?> classIter = theClasses.iterator();
		while (classIter.hasNext())
		{
			Cls theCls = (Cls)classIter.next();
			
			//EDITED: it did not take the superclasses anymore
			Cls directSuperCls = (Cls) theCls.getDirectSuperclasses().toArray()[0];
			String prefix = "";
			String name = "";
			String superClsName = "";
			if(directSuperCls instanceof AbstractRDFSClass) {
				String[] splittedName = ((AbstractRDFSClass)directSuperCls).getPrefixedName().split(":");
				if(splittedName.length > 1) {
					prefix = splittedName[0];
					name = splittedName[1];
					superClsName = prefix + "_" + name;
				}
				else
					superClsName = splittedName[0];
			}
			String superClsNameWithExtends = " extends " + superClsName;
			
			//String superClassString = getSuperClasses(theCls);
			String superClassString = superClsNameWithExtends;
			
			try {
				String javaClassName = this.converter.toJavaTypeName(theCls);

				File file = null;
				StringBuffer fileName = new StringBuffer(this.converter.toDir(this.packageName, this.dirName));
				fileName.append("\\exceptions\\");
				fileName.append(javaClassName);
				fileName.append(".java");
				file = new File(fileName.toString());
				file.getParentFile().mkdirs();
				file.createNewFile();

				PrintStream ps = createException(theCls, file, superClassString);
				ps.flush();
				ps.close();
				this.ontologyBeanGeneratorPanel.statusTextArea.append(" created file: " + javaClassName + ".java\n");
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/*EDITED: PrintStream ps is the interface, ps2 is the default implementation
	They are returned ending with '{' so the properties and methods are to be added after their creation.*/
	private void generateByCollection(Collection<?> theClasses, String collectionName, PrintStream psFact)
	{
		this.ontologyHeader.append("\n    // adding " + collectionName + "(s)\n");

		Iterator<?> classIter = theClasses.iterator();
		while (classIter.hasNext())
		{
			Cls theCls = (Cls)classIter.next();
			
			//Do not generate the interface for 'OntologyException'
//			if(this.converter.toJavaTypeName(theCls).equals(ONTOLOGY_EXCEPTION))
//				continue;
			if(ontologyExceptionExists && this.converter.toJavaTypeName(theCls).equals(ONTOLOGY_EXCEPTION_PREFIX + ONTOLOGY_EXCEPTION))
			  continue;
			
			if (!theCls.isSystem())
			{
				String superClassString = getSuperClasses(theCls);
					
				boolean isSuperBean = true;
				try
				{
					addInheritance(theCls);
				}
				catch (Exception e)
				{
					e.printStackTrace();
					break;
				}
				addClass(theCls, collectionName);
				StringBuffer extern = new StringBuffer();
				StringBuffer intern = new StringBuffer();
				if (this.microSupport)
				{
					extern.append(
							"  public void externalise(AbsObject absObj, Ontology onto) throws OntologyException {\n");
					extern.append("    try {\n");


					extern.append("      Abs");
					extern.append(collectionName);

					extern.append(" abs = (Abs");
					extern.append(collectionName);

					extern.append(") absObj;\n");

					intern.append(
							"  public void internalise(AbsObject absObj, Ontology onto) throws UngroundedException, OntologyException {\n");
					intern.append("    try {\n");


					intern.append("      Abs");
					intern.append(collectionName);

					intern.append(" abs = (Abs");
					intern.append(collectionName);

					intern.append(") absObj;\n");
				}
				String javaClassName = this.converter.toJavaTypeName(theCls);

				PrintStream ps = null;
				PrintStream ps2 = null;
				try
				{
					File file = null;
					File file2 = null;
					if (this.beanGeneration)
					{
						this.hasDirectSlotsWithMultipleCardinality = false;
						this.hasSlotsWithMultipleCardinality = false;

						StringBuffer fileName = new StringBuffer(this.converter.toDir(this.packageName, this.dirName));
						fileName.append("/");
						fileName.append(javaClassName);
						fileName.append(".java");
						file = new File(fileName.toString());
						ps = createBeanInterface(theCls, file, superClassString, isSuperBean);

						fileName = new StringBuffer(this.converter.toDir(this.packageName, this.dirName));
						fileName.append("/impl/");
						fileName.append("Default");
						fileName.append(javaClassName);
						fileName.append(".java");
						file2 = new File(fileName.toString());
						ps2 = createBean(theCls, file2, isSuperBean);

						this.ontologyBeanGeneratorPanel.statusTextArea.append(
								" created file: " + javaClassName + 
								".java\n");
						this.ontologyBeanGeneratorPanel.statusTextArea.append(
								"  added slots:\n");
					}
					Collection<?> directTemplSlots = new HashSet(theCls.getDirectTemplateSlots());
					Collection<Slot> allValidTemplateSlots = getAllJadeTemplateSlots(theCls);
					if (this.factoryGeneration)
					{
						psFact.println("  /**");
						psFact.println(getDocumentation(theCls, "   "));
						psFact.println("  */");
						psFact.println("  public " + javaClassName + " " + this.converter.toFactoryGetMethod(javaClassName) + 
								"(String instanceName) throws " + this.factoryExceptionClassName + "{");
						this.factoryGlobal.append("    typeToMethodName.put(\"" + theCls.getName() + "\", \"" + this.converter.toFactoryGetMethod(javaClassName) + "\");\n");
						psFact.println("    Object jadeInstance = beanInstances.get(instanceName);");
						psFact.println("    if (jadeInstance != null)");
						psFact.println("      return (" + javaClassName + ")jadeInstance;");
						psFact.println();
						if ((this.itsKB instanceof OWLModel)) {
							psFact.println("    OWLIndividual protegeInst = owlModel.getOWLIndividual( instanceName );");
						} else {
							psFact.println("    Instance protegeInst = kb.getInstance( instanceName );");
						}
						psFact.println();
						psFact.println("    if (protegeInst == null) {");
						psFact.println("      throw new " + this.factoryExceptionClassName + 
								"(\"Invalid Protege instance name: \" + instanceName + \". No Jade instance was created.\");");
						psFact.println("    }");
						psFact.println();

						psFact.println("    " + javaClassName + " javaInst = new Default" + javaClassName + "(instanceName);");
						psFact.println();
						psFact.println("    beanInstances.put(instanceName, javaInst);");
						psFact.println();
						psFact.println("    //javaInst set all properties");
						psFact.println();
						psFact.println();
					}
					Iterator<Slot> slotIter = allValidTemplateSlots.iterator();
					while (slotIter.hasNext())
					{
						Slot theSlot = (Slot)slotIter.next();
						boolean isDirectSlot = directTemplSlots.contains(theSlot);
						if (this.beanGeneration) {
							createSlot(ps, ps2, psFact, theCls, theSlot, extern, intern, isDirectSlot);
						}
						if (isDirectSlot)
						{
							addRole(theCls, theSlot);
							if ((!this.microSupport) && (this.useJADENames)) {
								addJADEMapping(theCls, theSlot);
							}
						}
					}
					if (this.microSupport)
					{
						extern.append("     } catch (ClassCastException cce) {\n");
						extern.append(
								"       throw new OntologyException(\"Error externalising ");
						extern.append(javaClassName);
						extern.append("\");\n");
						extern.append("     }\n");
						extern.append("   }\n\n");

						intern.append("     } catch (ClassCastException cce) {\n");
						intern.append(
								"       throw new OntologyException(\"Error internalising ");
						intern.append(javaClassName);
						intern.append("\");\n");
						intern.append("     }\n");
						intern.append("   }\n\n");

						ps2.print(extern.toString());
						ps2.print(intern.toString());
					}
					finishUpRole(theCls);
					if (this.beanGeneration)
					{
						//EDITED: to add the methods at the end of the interface declaration
						List<Method> methods = classNameToMethodsHashMap.get(theCls.getName());
						if(methods != null) {
							for(Method m : methods) {
								if(m.getOverrideMethod() != null)
									ps.println("   @Override public " + replacePackages(m.getOverrideMethod()));
								else {
									if(theCls.getName().equals(m.getInheritedFrom()))
								    ps.println("   public " + replacePackages(m.getMethod()) + ";");
								}
							}
						}
						ps.println("}");
						ps.flush();
						ps.close();
						
					  //EDITED: to give a default implementation to the methods
						if(methods != null) {
							for(Method m : methods) {
							  MethodData methodData = MethodParser.getMethodData(m.getMethod());
							  
							  if(methodData.getReturnType().equals("void")) {// if return type is void
							  	if(m.getOverrideMethod() != null)
							  		ps2.println("  @Override public " + 
							  				replacePackages(m.getOverrideMethod().substring(0, m.getOverrideMethod().length() - 1)) + "{\n  }");
							  	else {
							  		//if(theCls.getName().equals(m.getInheritedFrom()))
							  			ps2.println("  public " + replacePackages(m.getMethod()) + "{\n  }");
							  	}
							  }
							  else { // if return type is not void
							  	Object obj = null;
							  	String returnDefaultValue = "null";
							  	
							  	obj = PrimitiveDefaults.getDefaultValue(methodData.getReturnType());
							  	if(obj != null)
							  		returnDefaultValue = obj.toString();
							  	
							  	if(m.getOverrideMethod() != null)
							  		ps2.println("  @Override public " + 
							  				replacePackages(m.getOverrideMethod().substring(0, m.getOverrideMethod().length() - 1)) + "{\n    " + "return " + returnDefaultValue + ";\n  }");
							  	else {
							  		//if(theCls.getName().equals(m.getInheritedFrom()))
							  			ps2.println("  public " + replacePackages(m.getMethod()) + "{\n    " + "return " + returnDefaultValue + ";\n  }");
							  	}
							  }
							}
						}
						ps2.println("}");
						ps2.flush();
						ps2.close();
						
						if (!this.hasDirectSlotsWithMultipleCardinality) {
							formatImports(file);
						}
						if (!this.hasSlotsWithMultipleCardinality) {
							formatImports(file2);
						}
						if (this.applyFormatting)
						{
							format(file);
							format(file2);
						}
					}
					if (this.factoryGeneration)
					{
						psFact.println("    return javaInst;");
						psFact.println("  }");
						psFact.println();
					}
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
			}
			this.ontologyBeanGeneratorPanel.progress.setValue(
					this.ontologyBeanGeneratorPanel.progress.getValue() + 1);
			this.ontologyBeanGeneratorPanel.progress.repaint();
		}
	}
	
	private void setCorrectPackages(MethodData m) {
		
		m.setReturnType(replacePackages(m.getReturnType()));
		
		List<String> parameters = new ArrayList<String>();
		for(String p : m.getParameters())
			parameters.add(replacePackages(p));
		m.setParameters(parameters);
	}
	
	//EDITED: To force the correct packages
	private String replacePackages(String s) {
		s = " " + s;
//		s = s.replaceAll(" List ", " jade.util.leap.List ");
//		s = s.replaceAll(" Set ", " jade.util.leap.Set ");
//		s = s.replaceAll(" Map ", " jade.util.leap.Map ");
		
		s = s.replaceAll(" List ", " java.util.List ");
		s = s.replaceAll(" Set ", " java.util.Set ");
		s = s.replaceAll(" Map ", " java.util.Map ");
		
		s = s.replaceAll(" Object ", " java.lang.Object ");
		s = s.replaceAll(" String ", " java.lang.String ");
		s = s.replaceAll(" Byte ", " java.lang.Byte ");
		s = s.replaceAll(" Short ", " java.lang.Short ");
		s = s.replaceAll(" Integer ", " java.lang.Integer ");
		s = s.replaceAll(" Long ", " java.lang.Long ");
		s = s.replaceAll(" Float ", " java.lang.Float ");
		s = s.replaceAll(" Double ", " java.lang.Double ");
		s = s.replaceAll(" Boolean ", " java.lang.Boolean ");
		s = s.replaceAll(" Character ", " java.lang.Character ");
		
		return s;
	}

	public void initialize()
	{
		//EDITED: set jade.jar path for cmd commands
		protegeFolderPath = System.getProperty("user.dir");
		
		// DANY 
		// questa stringa  da cambiare...
		jadePath = protegeFolderPath + "\\plugins\\nl.uva.psy.swi.beangenerator\\jade.jar";
		
		//Set the xml file path
		//		xmlFilePath = protegeFolderPath + "\\plugins\\nl.uva.psy.swi.beangenerator\\methods.xml";
		//		File f = new File(xmlFilePath);
		//		if(f.exists() && !f.isDirectory())
		//			xmlFileExists = true;
		//		else
		//			xmlFileExists = false;

		updateConstantsNames(getKnowledgeBase());

		ImageIcon beanIcon = new ImageIcon(getClass().getResource("graphics/bean.gif"));
		setIcon(beanIcon);
		setLabel("Ontology Bean Generator 5.0");
		setFont(new Font("SansSerif", 3, 12));

		this.itsKB = getKnowledgeBase();
		this.ontologyBeanGeneratorPanel = new OntologyBeanGeneratorPanel(this);
		add(this.ontologyBeanGeneratorPanel);
		if (!isSimpleJADEAbstractOntIncluded(getKnowledgeBase()))
		{
			String errorMsg;

			if ((getKnowledgeBase() instanceof OWLModel)) {
				errorMsg = "Project should import OWLSimpleJADEAbstractOntology.owl";
			} else {
				errorMsg = "Project should include SimpleJADEAbstractOntology.pprj";
			}
			Log.getLogger().warning(errorMsg);

			JOptionPane.showMessageDialog(this, errorMsg, "Missing include/import", 0);
			return;
		}
		Cls projectAnno = this.itsKB.getCls(this.PROJECTANNOTATION);
		if (projectAnno == null) {
			return;
		}
		if (projectAnno.getInstanceCount() == 0)
		{
			this.projectAnnoInstance = projectAnno.createDirectInstance("project annotation");
		}
		else
		{
			this.projectAnnoInstance = ((Instance)projectAnno.getInstances().iterator().next());
			retrieveProjectAnnotation();
		}
	}

	private boolean isAnyInstanceTypeSlot(Collection<Slot> templateSlots)
	{
		for (Slot slot : templateSlots) {
			if (slot.getValueType().equals(ValueType.INSTANCE)) {
				return true;
			}
		}
		return false;
	}

	private void removeIgnoredClasses(Collection<?> col)
	{
		Iterator<?> iter = col.iterator();
		while (iter.hasNext())
		{
			Cls theCls = (Cls)iter.next();
			if (isIgnored(theCls)) {
				iter.remove();
			}
		}
	}

	private void reset() throws Exception
	{
		this.ontologyHeader = new StringBuffer();
		this.ontologyInit = new StringBuffer();
		this.ontologyBody = new StringBuffer();
		this.ontologyInheritance = new StringBuffer();
		this.nameMapping = new StringBuffer();
		this.factoryGlobal = new StringBuffer();

		this.ontologyName = ((String)
				this.ontologyBeanGeneratorPanel.ontologyComboBox.getSelectedItem());
		this.ontologyBeanGeneratorPanel.progress.setValue(0);

		this.currentSerialVersionUID = new Random().nextLong();

		this.jadeTemplateSlots = new HashMap();

		Cls jadeClass = this.itsKB.getCls(this.JADE_CLASS);
		if (jadeClass != null)
		{
			try
			{
				Collection<?> jadeClassSlots = jadeClass.getDirectTemplateSlots();
				Iterator<?> iter = jadeClassSlots.iterator();
				while (iter.hasNext())
				{
					Slot slot = (Slot)iter.next();
					if (slot.getName().equals(this.JADE_JAVA_BASE_CLASS_SLOT)) {
						this.javaSuperClassSlot = slot;
					} else if (slot.getName().equals(this.JADE_JAVA_CODE_SLOT)) {
						this.javaJavaCodeSlot = slot;
					} else if (slot.getName().equals(this.JADE_IGNORED_SLOT)) {
						this.jadeIgnoredSlot = slot;
					} else if (slot.getName().equals(this.PROTEGE_DOC)) {
						this.protegeDoc = slot;
					}
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
			try
			{
				Collection<?> jadeClassSlots = jadeClass.getDirectTemplateSlots();
				Iterator<?> iter = jadeClassSlots.iterator();
				while (iter.hasNext())
				{
					Slot slot = (Slot)iter.next();
					if (slot.getName().equals(this.JADE_JAVA_BASE_CLASS_SLOT)) {
						this.javaSuperClassSlot = slot;
					} else if (slot.getName().equals(this.JADE_JAVA_CODE_SLOT)) {
						this.javaJavaCodeSlot = slot;
					} else if (slot.getName().equals(this.JADE_IGNORED_SLOT)) {
						this.jadeIgnoredSlot = slot;
					} else if (slot.getName().equals(this.PROTEGE_DOC)) {
						this.protegeDoc = slot;
					}
				}
				Cls jadeSlotClass = this.itsKB.getCls(this.JADE_SLOT);
				jadeClassSlots = jadeSlotClass.getDirectTemplateSlots();
				iter = jadeClassSlots.iterator();
				while (iter.hasNext())
				{
					Slot slot = (Slot)iter.next();
					if (slot.getName().equals(this.JADE_UNNAMED_SLOT)) {
						this.jadeUnnamedSlot = slot;
					}
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
		}
		this.microSupport = false;
		this.fullBeanSupport = false;
		if (this.ontologyBeanGeneratorPanel.microSupport.isSelected()) {
			this.microSupport = true;
		} else if (this.ontologyBeanGeneratorPanel.fullBeanSupport.isSelected()) {
			this.fullBeanSupport = true;
		}
		this.dirName = ((String)
				this.ontologyBeanGeneratorPanel.locationComboBox.getSelectedItem());
		this.packageName = ((String)
				this.ontologyBeanGeneratorPanel.packageComboBox.getSelectedItem());
		this.factoryClassName = ((String)
				this.ontologyBeanGeneratorPanel.factoryComboBox.getSelectedItem());
		this.factoryExceptionClassName = (this.factoryClassName + "Exception");
		this.beanGeneration = this.ontologyBeanGeneratorPanel.beansCheckBox.isSelected();
		this.ontologyGeneration = 
				this.ontologyBeanGeneratorPanel.jadeCheckBox.isSelected();
		this.useJADENames = 
				this.ontologyBeanGeneratorPanel.useJadeNamesCheckBox.isSelected();
		//EDITED
//		this.factoryGeneration = this.ontologyBeanGeneratorPanel.factoryCheckBox.isSelected();
		this.prologGeneration = this.ontologyBeanGeneratorPanel.factoryCheckBox.isSelected();
		this.applyFormatting = 
				this.ontologyBeanGeneratorPanel.doFormatting.getModel().isSelected();


		boolean abortBecauseError = false;
		String message_null = "MUST be specified!\n";
		String message_empty = "MUST NOT be empty!\n";
		if (((this.beanGeneration) || (this.ontologyGeneration) || (this.factoryGeneration)) && (
				(this.packageName == null) || (this.packageName.equals(""))))
		{
			this.ontologyBeanGeneratorPanel.statusTextArea.append(
					"Package name " + (this.packageName == null ? message_null : message_empty));
			abortBecauseError = true;
		}
		if (((this.beanGeneration) || (this.ontologyGeneration) || (this.factoryGeneration)) && (
				(this.dirName == null) || (this.dirName.equals(""))))
		{
			this.ontologyBeanGeneratorPanel.statusTextArea.append(
					"Location " + (this.dirName == null ? message_null : message_empty));
			abortBecauseError = true;
		}
		if (((this.beanGeneration) || (this.ontologyGeneration)) && (
				(this.ontologyName == null) || (this.ontologyName.equals(""))))
		{
			this.ontologyBeanGeneratorPanel.statusTextArea.append(
					"Ontology domain " + (this.ontologyName == null ? message_null : message_empty));
			abortBecauseError = true;
		}
		if ((this.factoryGeneration) && ((this.factoryClassName == null) || (this.factoryClassName.equals(""))))
		{
			this.ontologyBeanGeneratorPanel.statusTextArea.append(
					"Factory class name " + (this.factoryClassName == null ? message_null : message_empty));
			abortBecauseError = true;
		}
		if (abortBecauseError) {
			throw new Exception("Missing or invalid values specified!\nFor more information please check the status panel.");
		}
		this.converter = new NameConverter(this.itsKB, this.useJADENames);

		// DANY
		// Before creating the files, I delete the previous directory, if any, with the same name as the one I'm creating.
		// This why otherwise, some previous files could remain, and could be not compilable with the new ones
		File outputDir = new File(this.converter.toDir(this.packageName, this.dirName));
		if (!outputDir.exists()) {
			outputDir.mkdirs();
		}
		else {
			recursiveDelete(outputDir);
			outputDir.mkdirs();
		}
		
		outputDir = new File(this.converter.toDir(this.packageName + ".impl", this.dirName));
		if (!outputDir.exists()) {
			outputDir.mkdirs();
		}
	}

	void retrieveProjectAnnotation()
	{
		try
		{
			Slot packageSlot = this.itsKB.getSlot(this.PACKAGE_SLOT);
			Slot ontologySlot = this.itsKB.getSlot(this.ONTOLOGYNAME_SLOT);
			Slot factoryClassNameSlot = this.itsKB.getSlot(this.FACTORY_CLASS_SLOT);
			Slot locationSlot = this.itsKB.getSlot(this.LOCATION_SLOT);
			Slot supportSlot = this.itsKB.getSlot(this.SUPPORT_SLOT);
			String packageName = (String)this.projectAnnoInstance.getOwnSlotValue(
					packageSlot);
			String ontologyName = (String)this.projectAnnoInstance.getOwnSlotValue(
					ontologySlot);
			String factoryClassName = (String)this.projectAnnoInstance.getOwnSlotValue(
					factoryClassNameSlot);
			String locationName = (String)this.projectAnnoInstance.getOwnSlotValue(
					locationSlot);
			String supportLine = (String)this.projectAnnoInstance.getOwnSlotValue(
					supportSlot);
			if (packageName != null)
			{
				this.ontologyBeanGeneratorPanel.packageBoxModel.addElement(packageName);
				this.ontologyBeanGeneratorPanel.packageComboBox.setSelectedItem(
						packageName);
			}
			if (ontologyName != null)
			{
				this.ontologyBeanGeneratorPanel.ontologyBoxModel.addElement(
						ontologyName);
				this.ontologyBeanGeneratorPanel.ontologyComboBox.setSelectedItem(
						ontologyName);
			}
			if (factoryClassName != null)
			{
				this.ontologyBeanGeneratorPanel.factoryBoxModel.addElement(
						factoryClassName);
				this.ontologyBeanGeneratorPanel.factoryBoxModel.setSelectedItem(
						factoryClassName);
			}
			if (locationName != null)
			{
				this.ontologyBeanGeneratorPanel.locationBoxModel.addElement(
						locationName);
				this.ontologyBeanGeneratorPanel.locationComboBox.setSelectedItem(
						locationName);
			}
			this.ontologyBeanGeneratorPanel.support.setSelected(
					this.ontologyBeanGeneratorPanel.standardSupport.getModel(), true);
			this.ontologyBeanGeneratorPanel.exampleTextArea.setText(
					Examples.getStandardExample());
			if (supportLine != null)
			{
				if (supportLine.equalsIgnoreCase("j2me"))
				{
					this.ontologyBeanGeneratorPanel.support.setSelected(
							this.ontologyBeanGeneratorPanel.microSupport.getModel(), true);
					this.ontologyBeanGeneratorPanel.exampleTextArea.setText(
							Examples.getMicroExample());
				}
				if (supportLine.equalsIgnoreCase("j2se"))
				{
					this.ontologyBeanGeneratorPanel.support.setSelected(
							this.ontologyBeanGeneratorPanel.standardSupport.getModel(), true);
					this.ontologyBeanGeneratorPanel.exampleTextArea.setText(
							Examples.getStandardExample());
				}
				if (supportLine.equalsIgnoreCase("javabeans"))
				{
					this.ontologyBeanGeneratorPanel.support.setSelected(
							this.ontologyBeanGeneratorPanel.fullBeanSupport.getModel(), true);
					this.ontologyBeanGeneratorPanel.exampleTextArea.setText(
							Examples.getFullExample());
				}
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}

	void saveProjectAnnotation()
	{
		try
		{
			Slot packageSlot = this.itsKB.getSlot(this.PACKAGE_SLOT);
			Slot ontologySlot = this.itsKB.getSlot(this.ONTOLOGYNAME_SLOT);
			Slot factoryClassNameSlot = this.itsKB.getSlot(this.FACTORY_CLASS_SLOT);
			Slot locationSlot = this.itsKB.getSlot(this.LOCATION_SLOT);
			Slot supportSlot = this.itsKB.getSlot(this.SUPPORT_SLOT);
			String packageName = (String)
					this.ontologyBeanGeneratorPanel.packageComboBox.getSelectedItem();
			String ontologyName = (String)
					this.ontologyBeanGeneratorPanel.ontologyComboBox.getSelectedItem();
			String locationName = (String)
					this.ontologyBeanGeneratorPanel.locationComboBox.getSelectedItem();
			String factoryClassName = (String)
					this.ontologyBeanGeneratorPanel.factoryComboBox.getSelectedItem();
			String supportLine = "j2se";
			if (this.ontologyBeanGeneratorPanel.microSupport.isSelected()) {
				supportLine = "j2me";
			}
			if (this.ontologyBeanGeneratorPanel.fullBeanSupport.isSelected()) {
				supportLine = "javabeans";
			}
			if (this.projectAnnoInstance == null) {
				return;
			}
			if (packageName != null) {
				this.projectAnnoInstance.setOwnSlotValue(packageSlot, packageName);
			}
			if (ontologyName != null) {
				this.projectAnnoInstance.setOwnSlotValue(ontologySlot, ontologyName);
			}
			if (factoryClassName != null) {
				this.projectAnnoInstance.setOwnSlotValue(factoryClassNameSlot, factoryClassName);
			}
			if (locationName != null) {
				this.projectAnnoInstance.setOwnSlotValue(locationSlot, locationName);
			}
			this.projectAnnoInstance.setOwnSlotValue(supportSlot, supportLine);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}

	private String packageName = "";
	private boolean hasSlotsWithMultipleCardinality;
	private boolean hasDirectSlotsWithMultipleCardinality;
	Instance projectAnnoInstance;
	private Slot protegeDoc;
	private boolean useJADENames;
	private long currentSerialVersionUID = 0L;
	private String JADE_CLASS = ":JADE-CLASS";
	private String JADE_IGNORED_SLOT = ":JADE-IGNORE";
	private String JADE_JAVA_BASE_CLASS_SLOT = ":JADE-JAVA-BASE-CLASS";
	private String JADE_JAVA_CODE_SLOT = ":JADE-JAVA-CODE";
	private String JADE_NAME_SLOT = ":JADE-NAME";
	private String JADE_SLOT = ":JADE-SLOT";
	private String JADE_UNNAMED_SLOT = ":JADE-UNNAMED";
	private String PROTEGE_DOC = ":DOCUMENTATION";
	private String AGENTACTION = "AgentAction";
	private String JAVA_AGENTACTION_CLASS_NAME = "AgentAction";
	private String AID = "AID";
	private String JAVA_AID_CLASS_NAME = "AID";
	private String CONCEPT = "Concept";
	private String JAVA_CONCEPT_CLASS_NAME = "Concept";
	private String LOCATION_SLOT = ":LOCATION";
	private String ONTOLOGYNAME_SLOT = ":ONTOLOGYNAME";
	private String FACTORY_CLASS_SLOT = ":FACTORYCLASSNAME";
	private String PACKAGE_SLOT = ":PACKAGE";
	private String PREDICATE = "Predicate";
	private String JAVA_PREDICATE_CLASS_NAME = "Predicate";
	private String PROJECTANNOTATION = ":PROJECT-ANNOTATION";
	private String SUPPORT_SLOT = ":SUPPORT";
	public static final String BEAN_GENERATOR_NAMESPACE = "http://jade.cselt.it/beangenerator#";
	public static String DEFAULT_FACTORY_CLASS_NAME = "KbBasedBeanFactory";
	private static final String ONE_TAB = "   ";
	public static final String INDENT_0 = "";
	public static final String INDENT_1 = "   ";
	public static final String INDENT_2 = "      ";
	private static final String IMPORT_ALL_JADE_CONTENT_ONTO = "import jade.content.onto.*;";
	private static final String IMPORT_ALL_JADE_CONTENT_SCHEMA = "import jade.content.schema.*;";
	private static final String IMPORT_ALL_JADE_CONTENT_ABS = "import jade.content.abs.*;";
	private static final String IMPORT_ALL_JADE_UTIL_LEAP = "import jade.util.leap.*;";

	protected Collection<Slot> getAllJadeTemplateSlots(Cls theCls)
	{
		Collection<Slot> allJadeTemplateSlots = (Collection)this.jadeTemplateSlots.get(theCls);
		if (allJadeTemplateSlots != null) {
			return allJadeTemplateSlots;
		}
		allJadeTemplateSlots = new HashSet(theCls.getDirectTemplateSlots());

		this.jadeTemplateSlots.put(theCls, allJadeTemplateSlots);

		Collection<?> directSuperClses = getDirectSuperclasses(theCls);
		for (Iterator<?> it = directSuperClses.iterator(); it.hasNext();)
		{
			Cls superCls = (Cls)it.next();
			if (isJADEClass(superCls)) {
				allJadeTemplateSlots.addAll(getAllJadeTemplateSlots(superCls));
			}
		}
		return allJadeTemplateSlots;
	}

	private void updateConstantsNames(KnowledgeBase kb)
	{
		if (!(kb instanceof OWLModel)) {
			return;
		}
		String prefix = ((OWLModel)kb).getNamespaceManager().getPrefix("http://jade.cselt.it/beangenerator#");
		if (prefix == null) {
			return;
		}
		this.JADE_CLASS = "http://jade.cselt.it/beangenerator#JADE-CLASS";
		this.JADE_IGNORED_SLOT = "http://jade.cselt.it/beangenerator#JADE-IGNORE";
		this.JADE_JAVA_BASE_CLASS_SLOT = "http://jade.cselt.it/beangenerator#JADE-JAVA-BASE-CLASS";
		this.JADE_JAVA_CODE_SLOT = "http://jade.cselt.it/beangenerator#JADE-JAVA-CODE";
		this.JADE_NAME_SLOT = "http://jade.cselt.it/beangenerator#JADE-NAME";
		this.JADE_SLOT = "http://jade.cselt.it/beangenerator#JADE-SLOT";
		this.JADE_UNNAMED_SLOT = "http://jade.cselt.it/beangenerator#JADE-UNNAMED";
		this.PROTEGE_DOC = "http://jade.cselt.it/beangenerator#DOCUMENTATION";
		this.AGENTACTION = "http://jade.cselt.it/beangenerator#AgentAction";
		this.AID = "http://jade.cselt.it/beangenerator#AID";
		this.CONCEPT = "http://jade.cselt.it/beangenerator#Concept";
		this.LOCATION_SLOT = "http://jade.cselt.it/beangenerator#LOCATION";
		this.ONTOLOGYNAME_SLOT = "http://jade.cselt.it/beangenerator#ONTOLOGYNAME";
		this.PACKAGE_SLOT = "http://jade.cselt.it/beangenerator#PACKAGE";
		this.FACTORY_CLASS_SLOT = "http://jade.cselt.it/beangenerator#FACTORYCLASSNAME";
		this.PREDICATE = "http://jade.cselt.it/beangenerator#Predicate";
		this.PROJECTANNOTATION = "http://jade.cselt.it/beangenerator#PROJECT-ANNOTATION";
		this.SUPPORT_SLOT = "http://jade.cselt.it/beangenerator#SUPPORT";
	}

	private static boolean isSimpleJADEAbstractOntIncluded(KnowledgeBase kb)
	{
		String CONCEPT_NAME = "Concept";
		String PREDICATE_NAME = "Predicate";
		if (!(kb instanceof OWLModel)) {
			return (kb.getCls(CONCEPT_NAME) != null) && (kb.getCls(PREDICATE_NAME) != null);
		}
		String prefix = ((OWLModel)kb).getNamespaceManager().getPrefix("http://jade.cselt.it/beangenerator#");
		if (prefix != null)
		{
			CONCEPT_NAME = "http://jade.cselt.it/beangenerator#" + CONCEPT_NAME;
			PREDICATE_NAME = "http://jade.cselt.it/beangenerator#" + PREDICATE_NAME;
		}
		return (kb.getCls(CONCEPT_NAME) != null) && (kb.getCls(PREDICATE_NAME) != null);
	}

	public static boolean isSuitable(Project p, Collection  errors)
	{
		if (!isSimpleJADEAbstractOntIncluded(p.getKnowledgeBase()))
		{
			if ((p.getKnowledgeBase() instanceof OWLModel)) {
				errors.add("Project should import OWLSimpleJADEAbstractOntology.owl");
			} else {
				errors.add("Project should include SimpleJADEAbstractOntology.pprj");
			}
			return false;
		}
		return true;
	}
}



