  package nl.uva.psy.swi.beangenerator;
  
  import jade.content.abs.AbsHelper;
  import jade.content.abs.AbsObject;
  import jade.content.onto.Ontology;
  import jade.content.onto.OntologyException;
  import jade.content.onto.ReflectiveIntrospector;
  import jade.content.onto.UngroundedException;
  import jade.content.schema.ObjectSchema;
  import java.lang.reflect.Method;
  
  public class ProtegeIntrospector
    extends ReflectiveIntrospector
  {
    private static final long serialVersionUID = 1L;
    private ProtegeTools.ProtegeOntology protegeOntology;
    
    public void setOntology(ProtegeTools.ProtegeOntology onto)
    {
      this.protegeOntology = onto;
    }
    
    public AbsObject externalise(Object obj, ObjectSchema schema, Class javaClass, Ontology referenceOnto)
      throws OntologyException
    {
      String slotName = "<unknown>";
      try
      {
        AbsObject abs = schema.newInstance();
        String[] names = schema.getNames();
        for (int i = 0; i < names.length; i++)
        {
          slotName = names[i];
          
          String transLatedName = translateName(slotName, javaClass);
          if (transLatedName != null)
          {
            String methodName = "get" + transLatedName;
            
            Method getMethod = findMethodCaseInsensitive(methodName, javaClass);
            
            AbsObject value = (AbsObject)invokeAccessorMethod(getMethod, obj);
            if (value != null) {
              AbsHelper.setAttribute(abs, slotName, value);
            }
          }
        }
        return abs;
      }
      catch (OntologyException oe)
      {
        throw oe;
      }
      catch (Throwable t)
      {
        throw new OntologyException("Schema and Java class do not match for the slot:" + slotName + " and class: " + javaClass.getClass().toString(), t);
      }
    }
    
    public Object internalise(AbsObject abs, ObjectSchema schema, Class javaClass, Ontology referenceOnto)
      throws UngroundedException, OntologyException
    {
      try
      {
        Object obj = javaClass.newInstance();
        String[] names = schema.getNames();
        for (int i = 0; i < names.length; i++)
        {
          String slotName = names[i];
          AbsObject value = abs.getAbsObject(slotName);
          if (value != null)
          {
            String methodName = "set" + translateName(slotName, javaClass);
            
            Method setMethod = findMethodCaseInsensitive(methodName, javaClass);
            invokeSetterMethod(setMethod, obj, value);
          }
        }
        return obj;
      }
      catch (OntologyException oe)
      {
        throw oe;
      }
      catch (InstantiationException ie)
      {
        throw new OntologyException("Class " + javaClass + " can't be instantiated", ie);
      }
      catch (IllegalAccessException iae)
      {
        throw new OntologyException("Class " + javaClass + " does not have an accessible constructor", iae);
      }
      catch (Throwable t)
      {
        throw new OntologyException("Schema and Java class do not match", t);
      }
    }
    
    protected String translateName(String slotName)
    {
      return ProtegeTools.firstUpper(ProtegeTools.toJavaStyleString(slotName));
    }
    
    private String translateName(String jadeName, Class<?> javaClass)
    {
      SlotHolder jadeSlot = new SlotHolder(javaClass.getName(), jadeName);
      if (jadeSlot == null) {
        return null;
      }
      SlotHolder methodKey = this.protegeOntology.getSlotNameFromJADEName(jadeSlot);
      if (methodKey == null) {
        return null;
      }
      return translateName(methodKey.slotName);
    }
  }



 