package xml_parser;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import javax.swing.JTable;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import antlr4_parser.Method;
import antlr4_parser.MethodParser;
import exceptions.NotWellFormedFileException;
import exceptions.OpenFileException;

public class XMLParser {
	
	public static HashMap<String, List<Method>> readXMLFile(String path) throws OpenFileException {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		Document doc = null;
		
	  DocumentBuilder db;
	  try {
	  	db = dbf.newDocumentBuilder();
	    doc = db.parse(path);
    } 
	  catch (ParserConfigurationException | SAXException | IOException e) {
	    throw new OpenFileException();
    }
	  
	  HashMap<String, List<Method>> classToMethodHashMap = parseDocument(doc);
//	  if(classToMethodHashMap.size() == 0)
//	  	throw new NotWellFormedFileException();
		return classToMethodHashMap;
	}
	
	private static HashMap<String, List<Method>> parseDocument(Document doc) {
		HashMap<String, List<Method>>classToMethodHashMap =  new HashMap<String, List<Method>>();
		
		Element docElement = doc.getDocumentElement();
		NodeList classList = docElement.getElementsByTagName("class");
		for(int i = 0; i < classList.getLength(); i++) {
			Element classElem = (Element) classList.item(i);
			
			List<String> rawMethods = new ArrayList<String>();
			List<String> inheritedFrom = new ArrayList<String>();
			List<String> overrideMethods = new ArrayList<String>();
			NodeList methodList = classElem.getElementsByTagName("method");
			for(int j = 0; j < methodList.getLength(); ++j) {
				Element methodElem = (Element) methodList.item(j);
				rawMethods.add(getTextValue(methodElem, "value"));
				inheritedFrom.add(getTextValue(methodElem, "from"));
				overrideMethods.add(getTextValue(methodElem, "override"));
			}
			
			String toParseMethods = "";
			for(String m : rawMethods)
				toParseMethods += m + ";";
			List<Method> methods = MethodParser.parseString(toParseMethods);
			for(int k = 0; k < methods.size(); k++) {
				methods.get(k).setInheritedFrom(inheritedFrom.get(k));
				methods.get(k).setOverrideMethod(overrideMethods.get(k));
				if(methods.get(k).getOverrideMethod() != null) {
					Method overrideMethod = MethodParser.parseString(methods.get(k).getOverrideMethod()).get(0);
					String newNiceFormatting = "<html> <font color=gray >@Override </font> <font color=green >" + 
							methods.get(k).getInheritedFrom().split("#")[1] + ":: </font>" + 
							overrideMethod.getNiceFormatting();     
					methods.get(k).setNiceFormatting(newNiceFormatting);
				}
			}
			
			classToMethodHashMap.put(getTextValue(classElem, "name"), methods);
		}
		return classToMethodHashMap;
	}
	
	public static boolean writeXMLFile(String path, ConcurrentHashMap<String, JTable> classToMethodTableMap) {
		PrintWriter writer;
    try {
	    writer = new PrintWriter(path, "UTF-8");
			writer.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			writer.println("<root>");
			for(String className : classToMethodTableMap.keySet()) {
				JTable table = classToMethodTableMap.get(className);
				if(table.getRowCount() > 0) {
					writer.println("  <class>");
					writer.print("    <name>");
					writer.print(className);
					writer.println("</name>");
					for(int i = 0; i < table.getRowCount(); i++) {
						writer.println("    <method>");
						writer.print("      <value>");
						writer.print(table.getValueAt(i, 0));
						writer.println("</value>");
						writer.print("      <from>");
						writer.print(table.getValueAt(i, 3));
					  writer.println("</from>");
					  if(table.getValueAt(i, 4) != null) {
					  	writer.print("      <override>");
					  	writer.print(table.getValueAt(i, 4));
					  	writer.println("</override>");
					  }
						writer.println("    </method>");
					}
					writer.println("  </class>");
				}
			}
			writer.println("</root>");
			writer.close();
			return true;
    } 
    catch (FileNotFoundException | UnsupportedEncodingException e) {
	    return false;
    }
	}

	private static String getTextValue(Element ele, String tagName) {
		String textVal = null;
		NodeList nl = ele.getElementsByTagName(tagName);
		if(nl != null && nl.getLength() > 0) {
			Element el = (Element) nl.item(0);
			textVal = el.getFirstChild().getNodeValue();
		}
		return textVal;
	}
}
